/*****************************************************************************************
 *  Copyright (C), 2004
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orva_ccmerge.sas                                             
 *  TITLE:   Merges the Dataset with the Cross Classification Variables with the project
 *           Mapping Datasets. Cannot be used with Scenario Datasets that have been set up
 *           with standard business lines or risk categories.  
 *           See orva_baslccmerge for that.
 *
 *  SUPPORT: stchri
 *
 *  PARAMETERS:
 *
 *  numCCVs = number of Cross Classification variables, defaults to 2.
 *
 *  data  =  the name of the "raw" loss dataset. It should contain the
 *           original Cross Classification columns from the Relational Database. 
 *
 *  mapCols  = space-separated list of Cross Classification columns (left-hand side), 
 *             ex: INTERNAL_BL_UID INTERNAL_RC_UID.
 *             The special case of __TOTALS__ONLY__ signifies that the mapping can be
 *             ignored for that dimension.  In that case we map all values to the total
 *             for that dimension.
 *
 *  ccData = the name of the Cross Classification UID dataset with the following variables:
 *           CCUID, CCVUID_<n> (one for each Cross Classification)
 *
 *  mapDataPrefix = prefix for the name of the mapping datasets. Should be one
 *                  for each mapping called &mapDataPrefix<n>.
 * 
 *  dataOut = the name of the output dataset (VaR Loss Dataset).
 *
 *  ostat = name of the out status dataset.  Will contain the following:
 *           status = 0 on success, 1 otherwise.
 *           reason = printable message.
 *           
 *  missingRanges = are we merging the missing_ranges table?
 *  
 *  CTool - Are we in tthe Consortium Tool?
 *  
 *  Example: %orva_ccmerge(numCCVs=2,
 *                         data=VARLOSS.rawlossi,
 *                         mapCols=INTERNAL_BL_UID INTERNAL_RC_UID,
 *                         ccData=VARPRJ.cc,
 *                         mapDataPrefix=VARPRJ.ccmapi_,
 *                         dataOut=VARPRJ.lossi,
 *                         ostat=status);
 * 
 *****************************************************************************************/
%macro orva_ccmerge(numCCVs=       2,
                    data=           ,
                    mapCols=        ,
                    ccData=         ,
                    mapDataPrefix=  ,
                    dataOut=       LossI,
                    oStat=         status,
                    missingRanges= false,
                    CTool=         false
                    );

   /*- create the status dataset up front with Unknown status -*/
   %let reason=%sysfunc(sasmsg(&orvads, unknown_error, noquote));
   %orva_status_ds( statusds=&oStat, status=1, reason=&reason );

   %let status=0;
   %let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote)) ;
   %let abort=0;

   /*- check input parameters -*/
   %if &data=%str() or &ccData=%str() or &mapCols=%str() or &mapDataPrefix=%str() %then %do;
     %put &orvar_error %sysfunc(sasmsg(&orvads, ccm_specify_these_fields_error, noquote)) ;
     %let abort=1;
     %let reason=%sysfunc(sasmsg(&orvads, ccm_missing_parameters_error, noquote)) ;
     %goto DONE;
   %end;

   
   %let mapCols=%upcase(&mapCols);    
   
       
   /*- merge each Mapping Dataset into Loss Dataset to get corresponding CCVUID_&i -*/
   proc sql noprint;
         CREATE TABLE 
                        _hash 
         AS
         SELECT 
                        * 
         FROM 
                        &mapDataPrefix.0 as a, 
                        &mapDataPrefix.1 as b,
						&ccdata as c
                  %if &numCCVs > 2 %then %do; , &mapDataPrefix.2 as d %end;
         where
                  a.ccvuid_0 = c.ccvuid_0 and b.ccvuid_1 = c.ccvuid_1
                  %if &numCCVs > 2 %then %do; and d.ccvuid_2 = c.ccvuid_2 %end; ;
      quit;
  
   
   
   /*- Finally, merge the Loss Dataset with the CC Dataset -*/
   data &dataout(compress=binary); set &data; 
       %let nmaps=1;
       %let map=%scan(&mapCols, &nmaps, ' ');
       %do i=0 %to &numCCVs-1;      
           %if &map eq __TOTALS__ONLY__ %then %do;
             CCMAP_&i = .;
           %end;
           %else %do;  
             CCMAP_&i = &map;
           %end;
           %let nmaps=%eval(&nmaps+1);
           %let map=%scan(&mapCols, &nmaps, ' ');
       %end;        
      if _N_ eq 1 then do;
          declare hash ccHash(dataset: "_hash");
		  rc = ccHash.defineKey("ccmap_0");
		  rc = ccHash.defineKey("ccmap_1");
		  %if &numCCVs > 2 %then %do;
		  rc = ccHash.defineKey("ccmap_2");
		  rc = ccHash.defineData("ccvuid_2");
		  call missing(ccvuid_2);
		  %end;
          rc = ccHash.defineData("ccuid");
          rc = ccHash.defineData("ccvuid_0");
		  rc = ccHash.defineData("ccvuid_1");

          rc = ccHash.defineDone(); 
		  call missing(ccvuid_0);
		  call missing(ccvuid_1);
          call missing(ccuid);  
	end;

   rc  = ccHash.find();

   %if &missingRanges eq false or &CTool eq false %then %do;
       if ccvuid_0 ne . and ccvuid_1 ne . 
              %if &numCCVs > 2 %then and ccvuid_1 ne .;
			  then output;
   %end;
   drop ccmap_0 ccmap_1  %if &numCCVs > 2 %then ccmap_2;  rc feed_ref_id; 

   run;
   

   
%DONE:
   %if &abort %then %do;
      %put &orvar_error %sysfunc(sasmsg(&orvads, ccm_abnormal_error, noquote)) ;
      %let status=1;    
   %end;   
   %orva_status_ds( statusds=&ostat, status=&status, reason=&reason );

  
   /*- delete unwanted datasets -*/
   proc datasets library=work memtype=data nolist nodetails;
     delete _work: ;
   quit;

%mend orva_ccmerge;
