/******************************************************************************
 * Copyright (C), 2012
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  Title: orvd_Create_Empirical.sas
 *  
 *  -creates a set of indexed functions from a dataset that provide empirical 
 *   distribution functionality (cdf, simulation, etc.)
 *
 *  NOTE: the names of the arrays used in the fcmp portion are taken directly from 
 *        the Kaplan-Meier description in the paper by Klugman and Rioux: 
 *        "Toward a Unified Approach to Fitting Loss Models", 
 *        NORTH AMERICAN ACTUARIAL JOURNAL, VOLUME 10, NUMBER 1
 * 
 *  Arguments:
 *    funclib-          = Where the functions will be stored- library.funclib.package
 *
 *    func_id-          = Tag to be placed at the end of the function names (func_TAG)
 *
 *    indata-           = Dataset to read loss values and thresholds from
 *
 *    variable-         = Column name for the losses
 *
 *    threshold-        = Column name for the threshold values
 *
 *    threshold_percent = Mass that lies below the smallest threshold ( 1-prob(obs) )
 *
 *    n_points          = Number of points to use in resampling (inverse cdf only)
 *
 *    max_points        = Maximum number of points to store for cdf and inverse
 *                        cdf.  If more points are observed, then resampling is
 *                        used to store the cdf.
 *
 *    importance_param  = Parameter used in importance sampling (1 is uniform)
 *                        (NOT CURRENTLY IN USE)
 *
 *    interpolateCdf    = true if cdf is interpolated - linear interpolation between points.
 *                        Otherwise cdf is a step function and inv_cdf will only
 *                        taker on a finite number of values.
 ******************************************************************************/
%macro orva_Create_Empirical(funclib           = , 
                             func_id           = ,
                             indata            = ,
                             variable          = ,
                             threshold         = threshold,
                             threshold_percent = 0,
                             n_points          = ,
                             max_points        = ,
                             importance_param  = 1,
                             interpolateCdf    = true
                            );
         
   %let func_id = %unquote(&func_id);

   /* Adjust the threshold_percent (missing and -1 are mapped to zero) */
   %let threshold_percent = %sysfunc(max(&threshold_percent,0));

   /* Sort losses and thresholds together. Get counts of each unique pair. 
      These will be used to create the t[] array in the fcmp code */
   proc freq 
      data = &indata.(keep = &variable. &threshold.) 
      noprint
   ; 
      tables 
         &threshold. * &variable./
         out = _ce_t(keep = &threshold. 
                            count )
      ; 
       /* Also get the counts of just the unique losses.  This will be the y[] and s[] arrays */
      tables 
         &variable. /
         out = _ce_y(keep = &variable. 
                            count )
      ; 
   run;

   /* Kaplan-Meier Algorithm */
   /* NOTE: The c[] array in the algorithm description (see paper ref. in top coments) is not needed, because we 
            already have the number of times each unique loss appears - the s[i] */
   proc fcmp;

      threshold_percent = &threshold_percent.;

      if ( (missing(threshold_percent)) or (threshold_percent < 0) ) then do;
         threshold_percent=0;
      end;

      /* The list of thresholds that goes with c[].*/
      array t[1] /NOSYMBOLS;
      rc = read_array("_ce_t", t, "&threshold.");  

      /* The number of times the pair t[j],c[ind[j]] appears, where ind[j] is theindex of t[j] in the sorted list of c[j] t[j]
         pairs */
      array w[1] /NOSYMBOLS;
      rc = read_array("_ce_t", w, "count");

      /* The sorted list of unique loss amounts */
      array y[1] /NOSYMBOLS;
      rc = read_array("_ce_y", y, "&variable.");

      /* s[i] is the number of times y[i] appears */
      array s[1] /NOSYMBOLS;
      rc = read_array("_ce_y", s, "count");

      /* r[i] is the number of events of size y[i] with thresholds less than y[i] */
      array r[1] /NOSYMBOLS;
      call dynamic_array(r,dim(s));

      /* The cdf values to write out */
      array cdf[1] /NOSYMBOLS;
      call dynamic_array(cdf,dim(s));

      emp_points = dim(cdf);

      /* Compute the r[i] */
      start_pos    = 1;
      total_count  = 0;
      total_count2 = 0;
      prod         = 1;

      do i=1 to dim(r);
         r[i] = 0;

         do j = start_pos to dim(t);
            if t[j] <= y[i] then do;
               total_count = total_count + w[j];
               if j = dim(t) then do;
                  start_pos = dim(t) + 1;
               end;
            end;
            else do;
               start_pos = j;
               j = dim(t);
            end; 
         end;
         r[i] = total_count;

         r[i] = r[i] - total_count2;
         total_count2 = total_count2 + s[i];

         prod   = prod * ( r[i] - s[i] ) / r[i];
         cdf[i] = threshold_percent + (1-threshold_percent)*(1-prod);
      end;

      /* ------ GET POINTS FOR INVERSE CDF AND CDF ---------*/
      /* We now resample  on (0,1) and get new cdf points. This makes the lookup for inverse cdf calls
         much faster, since we now know the spacing of the values on the y axis. */
      n_points_adj = min(&n_points.,&max_points.);

      array out_cdf[1] /NOSYMBOLS;
      array out_vals[1] /NOSYMBOLS;
      call dynamic_array(out_cdf,n_points_adj);
      call dynamic_array(out_vals,n_points_adj);

      ind = 1;
      inc = 1/n_points_adj;
      do i = 1 to n_points_adj;
         p = inc*i;
         out_cdf[i] = p;

         if (out_cdf[i] <= cdf[1]) then do;
            m = y[1]/cdf[1];
            out_vals[i] = m*out_cdf[i];
         end;
         else if out_cdf[i] >= cdf[emp_points] then do;
            m = (y[emp_points]-y[emp_points-1])/(cdf[emp_points]-cdf[emp_points-1]);
            out_vals[i] = y[emp_points]+m*(out_cdf[i]-cdf[emp_points]);
         end;
         else do;
            do while (out_cdf[i] > cdf[ind] );
                ind = ind + 1;
            end;
            m = (y[ind]-y[ind-1])/(cdf[ind]-cdf[ind-1]);
            out_vals[i] = y[ind-1] + m*(out_cdf[i]-cdf[ind-1]);
         end;
      end;

      /* Now rescale the loss values by their mean */
      mean=y[1]*cdf[1];
      do i=2 to emp_points;
         mean = mean + y[i]*(cdf[i]-cdf[i-1]);
      end;
      do i=1 to n_points_adj;
         out_vals[i] = out_vals[i]/mean;
      end;

      call symputx("Param_1_init",put(mean,BEST20.));

      /* resampled values- for inverse cdf */
      rc = write_array("CDFS",out_cdf,"CDF");
      rc = write_array("VALS",out_vals,"VAL");

      /* Get the likelihood */
      logl = 0;
      do i=2 to emp_points;
         logl = logl+safe_log((cdf[i]-cdf[i-1])**s[i]);
      end;
      logl = logl+safe_log(cdf[1]**s[1]);
      call symputx("logl",logl);

      call symputx("n_points_adj",n_points_adj);

      /* -------------- GET MOMENTS FOR THE MOMENT FUNCTION ------------------ */
      m1 = out_cdf[1]*out_vals[1];
      m2 = out_cdf[1]*out_vals[1]**2;
      m3 = out_cdf[1]*out_vals[1]**3;
      m4 = out_cdf[1]*out_vals[1]**4;

      do i=2 to n_points_adj;
         weight = (out_cdf[i]-out_cdf[i-1]);
         m1 = m1 + weight*out_vals[i];
         m2 = m2 + weight*out_vals[i]**2;
         m3 = m3 + weight*out_vals[i]**3;
         m4 = m4 + weight*out_vals[i]**4;
      end;

      call symputX("m1", 1);
      call symputX("m2", m2);
      call symputX("m3", m3);
      call symputX("m4", m4);
   run;

   proc sql noprint; 
      select put(VAL,BEST20.) 
      into :_AMOUNT_
      separated by " " 
      from VALS; 
   quit;

   proc sql noprint; 
      select put(CDF,BEST20.)
      into :_CDF_ 
      separated by " " 
      from CDFS; 
   quit;

   /* Get Kernel Density Estimate for PDF */
   proc kde 
      data = &indata(keep = &variable.)
   ;
      univar 
         &variable. (BWM=3) / 
         method     = sjpi 
         out        = work.density
      ;
   run;
   %let dsid   = %sysfunc(open(density,I));
   %let nobs_d = %sysfunc(attrn(&dsid.,nlobs));
   %let rc     = %sysfunc(close(&dsid.));

   proc sql noprint; 
      select put(value,BEST20.) 
      into :_PDF_AMOUNT_
      separated by " " 
      from density; 
   quit;

   proc sql noprint; 
      select put(density,BEST20.) 
      into :_DENSITY_ 
      separated by " " 
      from density; 
   quit;

   /*Now write the functions */
   proc fcmp outlib = &funclib.;
      %if %upcase(&interpolateCdf) = TRUE %then %do;
         function cdf_EMP_&func_id.(x, Param_1);
            scale2 = Param_1;

            if missing(scale2) then do;
               scale2=1;
            end;
            array cdf[&n_points_adj.] /nosym (&_CDF_.);
            array amount[&n_points_adj.] /nosym (&_AMOUNT_.);
            count = &n_points_adj.;

            if (x < scale2*amount[1]) then do;
               return(&threshold_percent.);
            end;
            else if (x >= scale2*amount[count]) then do;
               m = (cdf[count]-cdf[count-1])/(amount[count]-amount[count-1])/scale2;
               p=min( cdf[count]+m*(x-scale2*amount[count]) , 1 );
               return(p);
            end;
            else do;
               /* Search for the right index.  Bisection method. */
               i = floor(count/2);
               found = 0;
			   UB = count;
			   LB = 1;

               do while( found = 0 );
                  if ( (x >= scale2*amount[i]) and (x <= scale2*amount[i+1]) ) then do;
                     found = 1;
                  end;
                  else if x < scale2*amount[i] then do;
                     UB = i;
                     i = floor( (LB + i)/2 );
                  end;
                  else if x > scale2*amount[i] then do;
				     LB = i;
                     i = floor( (UB + i)/2 );
                  end;

               end;

               /* linear interpolation */
               m = (cdf[i+1] - cdf[i])/(amount[i+1]-amount[i])/scale2;
               p = cdf[i] + m*(x-scale2*amount[i]);
               return(p);
            end;
         endsub;

         function inv_cdf_EMP_&func_id.(p, Param_1);
            scale2 = Param_1;

            if missing(scale2) then do;
               scale2=1;
            end;
            array cdf[&n_points_adj.] /nosym (&_CDF_);
            array vals[&n_points_adj.] /nosym (&_AMOUNT_);
            count = &n_points_adj.;

            inc = 1/(count);
            ind = ceil(p/inc);       
            if ind <= 1 then do;
               m = scale2*vals[1]/cdf[1];
               return(m*p);
            end;
            else if ind > count then do;
               m = scale2*(vals[count]-vals[count-1])/(cdf[count]-cdf[count-1]);
               return( scale2*vals[count]+m*(p-cdf[count]) );
            end;
            else do;
               m = scale2*(vals[ind]-vals[ind-1])/(cdf[ind]-cdf[ind-1]);
               return( scale2*vals[ind-1] + m*(p-cdf[ind-1]) );
            end;
         endsub;
      %end;
      %else %do;
         /* CDF IMPLEMENTED AS STEP FUNCTION */
         function cdf_EMP_&func_id.(x, Param_1);
            scale2 = Param_1;

            if missing(scale2) then do;
               scale2=1;
            end;
            array cdf[&n_points_adj.] /nosym (&_CDF_.);
            array amount[&n_points_adj.] /nosym (&_AMOUNT_.);
            count = &n_points_adj.;

            if (x < scale2*amount[1]) then do;
               return(&threshold_percent.);
            end;
            else if (x >= scale2*amount[count]) then do;
               return(1);
            end;
            else do;
               i=2;
               do while (x >= scale2*amount[i]);
                  i=i+1;
               end;
               p = cdf[i-1];
               return(p);
            end;
         endsub;

         /* Inverse CDF as a step function */
         function inv_cdf_EMP_&func_id.(p, Param_1);
            scale2 = Param_1;

            if missing(scale2) then do;
               scale2=1;
            end;
            array cdf[&n_points_adj.] /nosym (&_CDF_);
            array vals[&n_points_adj.] /nosym (&_AMOUNT_);
            count = &n_points_adj.;

            inc = 1/(count);
            ind = floor(p/inc); 
 
            if ind < 1 then do;
               return(0);
            end;
            else if ind >= count then do;
               return(scale2*vals[count]);
            end;
            else do;
               return( scale2*vals[ind] );
            end;
         endsub;
      %end;

      function pdf_EMP_&func_id.(x);
         array density[&nobs_d.] /nosym (&_DENSITY_);
         array amount[&nobs_d.] /nosym (&_PDF_AMOUNT_);
         count = &nobs_d.;
         if (x <= amount[1]) then do;
            return(density[1]/amount[1]*x);
         end;
         else if (x >= amount[count]) then do;
            m = (density[count]-density[count-1])/(amount[count]-amount[count-1]);
            p=min( density[count]+m*(x-amount[count]) , 1 );
            return(p);
         end;
         else do;
            i=1;
            do while (x > amount[i]);
               i=i+1;
            end;
            /* linear interpolation */
            m = (density[i] - density[i-1])/(amount[i]-amount[i-1]);
            p = density[i-1] + m*(x-amount[i-1]);
            return(p);
         end;
      endsub;
 
      subroutine mom_EMP_&func_id.(Param_1, m1, m2, m3, m4);
         outargs m1, m2, m3, m4;

         m1 = (Param_1)*&m1.;
         m2 = (Param_1)**2*&m2.;
         m3 = (Param_1)**3*&m3.;
         m4 = (Param_1)**4*&m4.;
      endsub;

      function LmMom_EMP_&func_id(x, i, Param_1);
         scale2 = Param_1;

         if missing(scale2) then do;
            scale2=1;
         end;
         array cdf[&n_points_adj.] /nosym (&_CDF_.);
         array amount[&n_points_adj.] /nosym (&_AMOUNT_.);
         count = &n_points_adj.;

         limom = 0;
         do j=1 to count;
            val = min(amount[j],x);
            if j=1 then do;
               weight = cdf[1];
            end;
            else do;
               weight = (cdf[j]-cdf[j-1]);
         end;
            limom = limom + weight*(scale2*val)**i;
         end;
         return(limom);
      endsub;
   run;
%mend orva_Create_Empirical;
