/*****************************************************************************************
 * $Id: orva_designmatrix.sas,v 1.1.2.5.2.11.2.1.8.2 2012/05/02 17:35:45 sasdje Exp $
 *  Copyright (C), 2005
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orva_designmatrix.sas 
 *  PURPOSE: Input:  A dataset &A containing only numeric variables.  This macro creates 
 *           a dummy variable for each unique value ("level") of each variable in the input 
 *           dataset.
 *           The output dataset contains all possible combinations of the levels 
 *           of the variables from &A.  
 * 
 *           Suppose there are &nvars variables in &A.  Suppose variable i has n_i possible levels.  
 *           Then the number of observations in the output dataset is: 
 *                the product of the n_i (for i=1 to &nvars). 
 *
 *           Columns in the output dataset are:  
 * 
 *           (1) The first &nvars columns are the same variables as in &A.  
 *               Each observation is one combination.
 *
 *           (2) The remaining columns are the dummy variables.  The total number of dummy 
 *               variables is the sum of the number of levels for each input variable.
 *               In a particular observation, the dummy variable corresponding to 
 *               (variable i, level j) is set to 1 if variable i equals level j 
 *               in that observation.

 *
 *  SUPPORT: lajack
 *
 *  PARAMETERS:
 *    A       = the input dataset.  This dataset should contain numeric variables only.  
 *              For each distinct value (level) of each variable, a dummy variable will be created.
 *
 *    design  = the output dataset.
 *    
 *    sparse  = true or false if we should use the sparse option on proc freq, tables statement.
 *              SPARSE==true will create all possible combos of variable levels even when 
 *				    a combo does not occur
 *
 *    combosonly = flag.  if true, output dataset contains only the combos.  if false, output dataset 
 *               contains both the combos and the columns with the dummy vars corresponding to the 
 *               different levels.
 *
 *    drop = flag.  if true, drop one dummy var column in output dataset per original variable 
 *            (so that we only make n-1	dummy variables for an original variable 
 *            that has n levels).
 *
 *    outDroppedDums = dataset of the "dropped" dummies.  
 *
 *    keepallowndummies = flag.  if true, and if "drop=true", then DON'T drop one dummy var for owner.
 * 
 *    status_design   = name of the out status dataset.  Will contain the following:
 *                    STATUS = 0 on success, 1 otherwise.
 *                    RESASON = printable message.
 *
 * NOTES: see sample code at end of file.
 * 
 *****************************************************************************************/
%macro orva_designmatrix(A=A,design=designoutput,sparse=false,combosonly=false,drop=false,outDroppedDums=droppedDummies,keepallowndummies=false,status_design=status_designs);
	
   /**** create the status data set upfront with unknown status ****/
   %let reason=%sysfunc(sasmsg(&orvads, unknown_error, noquote));    
   %orva_status_ds( statusds=&status_design, status=1, reason=&reason );

	%let status=0;
   %let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote));
   %let abort=0;

	%let nobs = 0;
	%let nvars = 0;
	%let dsid = 0;
	%if &A NE "" %then %do;
		%let dsid = %sysfunc(open(&A,I));                                                                                        
		%if &dsid NE 0 %then %do;                                                                                                      
			%let nobs = %sysfunc(attrn(&dsid,nlobs));   
			%let nvars= %sysfunc(attrn(&dsid,nvars));   
			%let rc = %sysfunc(close(&dsid));                                                                                             
		%end;        

	%end;
	%else %do;
		%put &orvar_note %sysfunc(sasmsg(&orvads, desm_specify_input_ds_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, desm_specify_input_ds_error, noquote));
        %goto DONE;
	%end;
	%if &nobs EQ 0 %then %do;
		%put &orvar_note %sysfunc(sasmsg(&orvads, desm_no_obs_in_req_ds_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, desm_no_obs_in_req_ds_error, noquote));
        %goto DONE;
	%end;
	%else %if &nvars EQ 0 %then %do;
		%put &orvar_note %sysfunc(sasmsg(&orvads, desm_no_vars_in_req_ds_warning, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, desm_no_vars_in_req_ds_warning, noquote));
        %goto DONE;
	%end;
	%if &dsid EQ 0 %then %do;
		%put &orvar_note %sysfunc(sasmsg(&orvads, desm_cant_open_req_ds_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, desm_cant_open_req_ds_error, noquote));
        %goto DONE;
	%end;

	/*  *******************************************************************  */
	/*  NOW WE KNOW THE INPUT WAS OKAY */
		
	/* get list of variable names using proc contents */
	proc contents data=&A noprint out=_design_foo(keep=name);
	run; 
	/* We already know the total number of vars: &nvars  */
	proc SQL noprint;
		select NAME into :NAME1 - :NAME&nvars
		from _design_foo;
	quit;

/*	%if &nvars GT 1 %then %do; ods output List=_design_mylist; %end; */
	/* !!!maybe remove the above line, if ljblah is same as _design_mynewlist... */
  /* when sparse is false we can use sort instead of freq -------------------------------------*/
  %if &sparse EQ true %then %do; 

      ods listing close;
      ods output nlevels=_design_mylevels;
      proc freq data = &A nlevels;
         tables  
            %do freqindex=1 %to %eval(&nvars-1);
               &&NAME&freqindex *
            %end;
            &&NAME&nvars
         / sparse 
         list 
         out=_design_thelist( keep = 
                              %do keepindex=1 %to &nvars; 
                                 &&NAME&keepindex 
                              %end;
                            ); 
         /* vars appear in ods tables in order in which they appear on tables stmt. */
      run;
      ods listing;
   %end;
   %else %do;   /* non sparse and combosonly -----------------*/
      proc sort data=&A( keep = 
                              %do keepindex=1 %to &nvars; 
                                 &&NAME&keepindex 
                              %end;
                            ) out=_design_thelist nodupkey; 
          by  %do freqindex=1 %to %eval(&nvars-1);
               &&NAME&freqindex 
            %end;
            &&NAME&nvars ;
      run;        

   %end;

	
/*	data _design_thelist;
			set _design_thelist;
			keep %do keepindex=1 %to &nvars; &&NAME&keepindex %end;  ;
	run; */

	%if &combosonly EQ true %then %do;
		data &design(compress=binary);
			set _design_thelist; 
		run;
		%goto DONE;	/***** Macro ends here if &combosonly EQ true!  *****/
	%end;




/* we don't have to sort, because proc freq makes _design_mylist in a certain order */
	
	proc SQL noprint;
		select NLEVELS into :NLEVELS1 - :NLEVELS&nvars
		from _design_mylevels;	
	quit;

	/* make the rightmost identity matrix, of size &&NLEVELS&nvars */
	data _design_outfilelast(compress=binary  drop=i theindex);
		array Dummy_&&NAME&nvars.._[&&NLEVELS&nvars];
		retain Dummy_&&NAME&nvars.._ 0;
		do i=1 to &&NLEVELS&nvars;
			theindex = mod(i, &&NLEVELS&nvars);
			if theindex EQ 0 then theindex = &&NLEVELS&nvars;
			Dummy_&&NAME&nvars.._[theindex] = 1;
			output;
			Dummy_&&NAME&nvars.._[theindex]=0; 
		end;
	run;
	%let prev_answer = _design_outfilelast;
	%do loopindex=%eval(&nvars-1) %to 1 %by -1;
		%make(&&NLEVELS&loopindex,Dummy_&&NAME&loopindex,&prev_answer,outfile&loopindex);
		%let prev_answer = outfile&loopindex;
	%end;
	
	data &design(compress=binary);
		set _design_thelist; set &prev_answer;
	run;

	%if &drop EQ true %then %do;
		%if &keepallowndummies EQ false %then %do;
			data _design_dds(keep=_NAME_ VALUE Level_Variable_Name);
				length _NAME_ $ 36;
				set &design;
			      %do i=1 %to &nvars;
						if Dummy_&&NAME&i.._&&NLEVELS&i = 1 then do;
							_NAME_  = "Dummy_&&NAME&i.._&&NLEVELS&i";
							Level_Variable_Name = "&&NAME&i";
							VALUE = &&NAME&i;
							output;
						end;
	              %end;
			run;
			proc sort data=_design_dds out=&outDroppedDums nodupkey;
				by Level_Variable_Name;
			run;
			data &design;
				set &design;
	            drop 
	                  %do i=1 %to &nvars;
	                         Dummy_&&NAME&i.._&&NLEVELS&i
	                  %end;
	            ;
	                         
			run;
		%end;
		%else %do;
			data _design_dds(keep=_NAME_ VALUE);
				length _NAME_ $ 36;
				set &design;
			      %do i=1 %to &nvars;
					%if %upcase(&&NAME&i) NE OWN %then %do;
						if Dummy_&&NAME&i.._&&NLEVELS&i = 1 then do;
							_NAME_  = "Dummy_&&NAME&i.._&&NLEVELS&i";
							Level_Variable_Name = "&&NAME&i";
							VALUE = &&NAME&i;
							output;
						end;
					%end;
	              %end;
			run;
			proc sort data=_design_dds out=&outDroppedDums nodupkey;
				by Level_Variable_Name;
			run;
			data &design;
				set &design;
	            drop 
	                  %do i=1 %to &nvars;
					  	  %if %upcase(&&NAME&i) NE OWN %then %do;
	                         Dummy_&&NAME&i.._&&NLEVELS&i
						  %end;
	                  %end;
	            ;
	                         
			run;
		%end;
	%end;
	%else %do;
		data &outDroppedDums;
			set _NULL_;
		run;
	%end;






%DONE:

    %if &abort eq 1 %then %do;
        %put &orvar_note %sysfunc(sasmsg(&orvads, desm_abnormal_error, noquote));
        %let status=1;
    %end;   
    %orva_status_ds( statusds=&status_design, status=&status, reason=&reason );

	
   /**** delete temporary datasets ****/
   proc datasets library=work memtype=data nolist nodetails;
       delete _design_:  ;
   quit;

%mend orva_designmatrix;

/*   
Adds dummies for one more variable (named a_varname, with a_size levels) 
to the current matrix of dummies (called dset).

*/
%macro make(a_size,a_varname,dset,out);
/*
a_size is the number of levels for some variable.
a_varname is the name of that variable (which will be used as the prefix for the dummy variables we create).
dset is the dataset of ones & zeroes, to be combined with each row of an identity matrix of size a_size.

*/
	%let nobs=0;
	%let dsid=%sysfunc(open(&dset,I));
	%if &dsid NE 0 %then %do;
		%let nobs=%sysfunc(attrn(&dsid,nlobs));
		%let rc=%sysfunc(close(&dsid));
	%end;
	%if &nobs GT 0 %then %do;

		data first (drop= index_make aindex);
			array &a_varname._[&a_size];
			retain &a_varname._  0;
			do index_make=1 to %eval(&a_size * &nobs);
				aindex = ceil( index_make/&nobs );
				&a_varname._[aindex] = 1;
				output;
				&a_varname._[aindex]=0;
			end;
		run;
		data ontop;
			set 
				%do i=1 %to &a_size;
					&dset					
				%end;
			;
		run;
		data &out(compress=binary);
			set first; set ontop;
		run;
	%end;
%mend;

/**/
/*options symbolgen notes mprint;*/
/**/
/*data One;*/
/*	a=1;*/
/*	b=2;*/
/*	c=3;*/
/*	output;	*/
/*	a=10;*/
/*	b=2;*/
/*	c=3;*/
/*	output;*/
/*	a=10;*/
/*	b=20;*/
/*	c=3;*/
/*	output;*/
/*	a=10;*/
/*	b=2;*/
/*	c=30;*/
/*	output;*/
/*	a=1;*/
/*	b=2;*/
/*	c=30;*/
/*	output;*/
/*run;*/
/*%orva_designmatrix(One,ljdesign1,thestatus1);*/
/**/
/**/
/*data Two;*/
/*	a=1;*/
/*	b=2;*/
/*	c=3;*/
/*	d=4;*/
/*	output;	*/
/*	a=10;*/
/*	b=2;*/
/*	c=30;*/
/*	d=4;*/
/*	output;*/
/*	a=100;*/
/*	b=20;*/
/*	c=300;*/
/*	d=4;*/
/*	output;*/
/*	a=10;*/
/*	b=200;*/
/*	c=3000;*/
/*	d=4;*/
/*	output;*/
/*	a=1;*/
/*	b=200;*/
/*	c=30000;*/
/*	d=400;*/
/*	output;*/
/*run;*/
/*%orva_designmatrix(Two,ljdesign2,thestatus2);*/
