/*****************************************************************************************
 * Copyright (c) 2004 by SAS Institute Inc.  Cary NC 27513  USA
 *
 * NAME: ORVA_freq_kri_effect.sas                                             
 * DESCRIPTION: Compute the effect of the kris on the frequency distribution
 * SUPPORT: lajack       
 *
 * PARAMETERS: 
 *
 *    freqPriorityDataset = Name of the frequency priority dataset. 
 *                          The data set has one row for each cell containing:
 *                              SOURCE_TYPE   I,C,T,S,B
 *                              SOURCE_NAME   for scenarios
 *                              DATASET_NAME  freq fits dataset
 *                              CCUID         cell key
 *                              FMODELS_DS    dataset from %orva_ffit
 *                              BLESTS_DS     dataset from %orva_ffit
 *    freqKriDatasetBl    = Dataset with the simulation values for the Businessline
 *                          frequency KRI variables. Must have:
 *                             BLUID_KRI and <risk_factors> and RISK_FACTOR_VARIABLE_DATE
 *    freqFriDatasetFirm  = Dataset with the simulation values for the Firm
 *                          frequency KRI variables. Must have:
 *                             BLUID_KRI and <risk_factors> and RISK_FACTOR_VARIABLE_DATE
 *    ccData              = Name of the Cross Classification dataset containing:
 *                              CCUID, CCVUID_0, CCVUID_1
 *    outStatus           = Success or failure, default is outstatus 
 *    outFParmEffect      = Frequency parameters output dataset 
 *                                             
 **************************************************************************************/
%macro ORVA_freq_kri_effect(  
                              freqPriorityDataset = ,
                              freqKriDatasetBl    = ,   /* Only needed for data w/ KRIs */
                              freqKriDatasetFirm  = ,   /* Only needed for data w/ KRIs */
                              ccData              = ,
                              outStatus           = outStatus,
                              outFParmEffect      = outfParmEffect,
                              interval            = YEAR,
                              proxyScenVersion    = 42
                              );

   %_fke_preamble&proxyScenVersion(
                                   freqPriorityData = &freqPriorityDataset,
                                   outData          = VARPRJ.Priority&proxyScenVersion
                                   ); 
   /*- create the status data set upfront -*/
   %let reason = %sysfunc(sasmsg(&orvads, unknown_error, noquote));
   %orva_status_ds( statusds = &outstatus, status = 1, reason = &reason );
   
   %let status = 0;
   %let reason = %sysfunc(sasmsg(&orvads, successful_note, noquote)) ;
   %let abort  = 0;
   %let CELL_WHERE = 1;
   %let NUMCCVS = 0;

   /*- check &freqPriorityDataset input parameter -*/
   %if "&freqPriorityDataset" = "" %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fke_specify_freqPriority_ds_error, noquote));
        %let abort  = 1;
        %let reason = %sysfunc(sasmsg(&orvads, fke_freqPriority_missing_error, noquote));
        %goto DONE;
   %end;
   %let ds_nobs=0;
   %let dsid=%sysfunc(open(&freqPriorityDataset,I));
   %if &dsid NE 0 %then %do;
        %let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
        %let rc=%sysfunc(close(&dsid));
   %end;
   %else %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_open_freqPriority_ds_error, noquote));
        %let abort  = 1;
        %let reason = %sysfunc(sasmsg(&orvads, fke_cant_open_freqPriority_ds_error, noquote));
        %goto DONE;     
   %end;
   %if &ds_nobs LE 0 %then %do; /* should have at least one obs */
        %put &orvar_error %sysfunc(sasmsg(&orvads, fke_freqPriority_ds_empty_warning, noquote, &freqPriorityDataset));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, fke_freqPriority_ds_empty_warning, noquote, &freqPriorityDataset));
        %goto DONE;
   %end;

    /*- check  KRI datasets -*/
   %let abort = 0;
   %local _freqKriAll_exists_ ;
   %local _only_Firm_kris_ ;

   %_fke_Check_Kri_Datasets(Exist = _freqKriAll_exists_, onlyFirmKRIs = _only_Firm_kris_ ); 

   %if &abort = 1 %then %do;
      %goto DONE;
   %end;

    /*- Find the number of rows in freqPriorityDataset, excluding rare events -*/
     /*- Get the list of CCUID's, datasets, etc. for which calculations need to be made -*/
      proc SQL noprint;
         SELECT 
               put(count(SOURCE_TYPE),8.-L)
         INTO
               :NROWS
         FROM
               &freqPriorityDataset
         WHERE
               SOURCE_TYPE NE "R"
         ;
         SELECT 
               dataset_name, 
               CCUID, 
               SOURCE_TYPE, 
               SCENARIO_TYPE,
               FMODELS_DS,
               BLESTS_DS
         INTO 
               :DSNAME1 - :DSNAME&NROWS , 
               :CCUID1 - :CCUID&NROWS , 
               :SOURCE_TYPE1 - :SOURCE_TYPE&NROWS, 
               :SCENARIO_TYPE1 - :SCENARIO_TYPE&NROWS,
               :FREQMODEL1 - :FREQMODEL&NROWS,
               :BLESTS1 - :BLESTS&NROWS
         FROM 
               &freqPriorityDataset 
         WHERE
               SOURCE_TYPE NE "R"
         ;
     quit;  
     %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
         %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_make_macros_with_these_names, noquote));
         %let abort  = 1;
         %let reason = %sysfunc(sasmsg(&orvads, fke_cant_make_macros_with_these_names, noquote));
         %goto DONE;
     %end;

    /************************************************************************/
    /* Loop through the rows to calculate the KRI & dummy effects           */
    /************************************************************************/
    /*  Take care of source types that would not have output datasets       */
    /*  from calling orva_ffit().                                           */
    /*  These would include mixed (Relative Relations) cells and scenarios  */
    /************************************************************************/
   %do _fke_index = 1 %to &NROWS;
      /* These cases amount to all the possibilities except for scenarios */
      %if 
         "&&SOURCE_TYPE&_fke_index" NE "I" 
         AND 
         "&&SOURCE_TYPE&_fke_index" NE "C" 
         AND 
         "&&SOURCE_TYPE&_fke_index" NE "T" 
         AND 
         "&&SOURCE_TYPE&_fke_index" NE "F" 
      %then %do;      
         %_fke_Handle_Scenarios(
                                 dsName       = &&DSNAME&_fke_index, 
                                 CCUID        = &&CCUID&_fke_index,
                                 sourceType   = &&SOURCE_TYPE&_fke_index,
                                 scenarioType = &&SCENARIO_TYPE&_fke_index
                                 );
         %goto NEXTCCUID; 
      %end; 
      %else %do;
         /* check for User-Defined distributions */
         %let USERDEFINED_DIST = FALSE;
         %_fke_Handle_User_Defined_Dist(
                                        dsName         = &&DSNAME&_fke_index, 
                                        CCUID          = &&CCUID&_fke_index,
                                        sourceType     = &&SOURCE_TYPE&_fke_index
                                        );

         %if %upcase(&USERDEFINED_DIST) EQ TRUE %then %do;
            %orvu_append(base=_foo_, new=_tmp_, canUseAppend=false);
            %goto NEXTCCUID;
         %end;
      %end;
      
      /**** end of data without strataprops and countdata *****/
      %local breakReason;
      %_fke_Handle_Fitted_Models(
                                 dsName          = &&DSNAME&_fke_index, 
                                 CCUID           = &&CCUID&_fke_index,
                                 sourceType      = &&SOURCE_TYPE&_fke_index,
                                 hasOnlyFirmKRIs = &_only_Firm_KRis_
                                 );

      %if &breakReason = DONE %then %do;
         %goto DONE;
     %end;
      
%NEXTCCUID:
   %end; 

   /*********** adjust fparams for CTool models (Source=T) **********/
   data &outFParmEffect(keep= Distribution Fparam_: CCUID d_: Source );
      set _foo_;

      if Source="T" then do;
         width_fit = intnx( Freq_Fit_Interval, '03Apr2001'd, 1 ) - intnx( Freq_Fit_Interval, '03Apr2001'd, 0 );
         width_sim = intnx( "&interval", '03Apr2001'd, 1 ) - intnx( "&interval", '03Apr2001'd, 0 );
         ratio = width_sim / width_fit ;
         call ChangeInterval( Distribution, ratio, &theFreqSimCallParams );
      end;
   run;

%DONE:
  
    %if &abort %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fke_abnormal_error, noquote));
        %orva_status_ds( statusds = &outstatus, status = 1, reason = &reason );
    %end;  
    %else %do;
        %orva_status_ds( statusds = &outstatus, status = 0, reason = &reason );
    %end;
    
    proc datasets library = work memtype = data nolist;
        delete 
               _foo_ 
               _freqKriAll_ 
               _tmp_ 
               _tmpDS_ 
               _tmpDS0_ 
               _TheResults_ 
               __tmp__ 
               ;
    quit;    
%mend ORVA_freq_kri_effect;

%macro _fke_Handle_Scenarios(dsName =, CCUID =, sourceType =, scenarioType =);
   data _NULL_;
      length DATASET_NAME $&ORV_DATASET_NAME_LENGTH.;
      DATASET_NAME = "&dsName";
      dotPosition = index(DATASET_NAME,".");
      scenarioPosition = index(DATASET_NAME,"Scenario");

      if scenarioPosition > 0 then do;
         DATASET_NAME = substr(DATASET_NAME,1, dotPosition ) || 
                        substr(DATASET_NAME,scenarioPosition,8) || 
                        substr(DATASET_NAME,scenarioPosition + 8)
                        ;
         call symput("dsName",DATASET_NAME);
      end;
   run;

   /*****************************************************************/
   /*   If this is a bucket scenario, I need to look at the dataset */
   /*   that is named VARPRJ??.scenario_plotbrkdwn_XXXX             */
   /*****************************************************************/
   %if &scenarioType eq B %then %do;
      %let plotLoc = %index(&dsName,_plot_);

      %if &plotLoc > 0 %then %do;
         %let dsName = %substr(&dsName,1,&plotLoc+4)brkdwn%substr(&dsName,&plotLoc+5);
      %end;

      /********************************************/
      /*   Get the number of rows in the scenario */
      /*   put it  in RowCount                    */
      /********************************************/
      proc SQL noprint;
         SELECT count(CCUID)
         INTO   :rowCount
         FROM   &dsName
         WHERE  CCUID  = &CCUID
         ;
      quit;

      data _SScen_; 
         length 
                  Distribution      $12 
                  Freq_Fit_Interval $12 
                  source            $2
                  ;

         set &dsName;
         where CCUID = &CCUID;
         Source            = "&sourceType";

         Freq_Fit_Interval = "";
         Distribution = "";
         FParam_1 = .;
         FParam_2 = .;

         /********************************************************/
         /*  Obtain the name and parameters of the frequency     */
         /*  distribution based on the Mean, Min, and Max inputs */
         /********************************************************/
         call BucketFreqSimDef(  Frequency_Mean,
                                 Frequency_Min,
                                 Frequency_Max,
                                 Distribution,
                                 FParam_1,
                                 FParam_2
                                 );

         keep 
               CCUID 
               Distribution 
               Fparam_: 
               Frequency_: 
               Scenario_: 
               Source 
               Freq_Fit_Interval;
      run;
   %end;
   %else %do;/* Model scenario */
      proc SQL noprint;
         SELECT 
                  count(CCUID)
         INTO   
                  :rowCount
         FROM   
                  &dsName
         WHERE  
                  CCUID  = &CCUID
         ;
      quit;

      data _SScen_; 
         length 
                  Distribution      $12 
                  Freq_Fit_Interval $12 
                  source            $2
                  ;

         set &dsName;
         where CCUID = &CCUID;
         Source       = "&sourceType";
         Distribution = Frequency_Dist;
         Freq_Fit_Interval = "";

         keep 
               CCUID 
               Distribution 
               Fparam_: 
               Frequency_: 
               Scenario_: 
               Source 
               Freq_Fit_Interval;
      run;
   %end;

   %if &rowCount GT 1 %then %do;
      %orva_freq_sum_model(
                           inData       = _SScen_, 
                           distModelVar = Distribution, 
                           outData      = _tmp_
                           );
   %end;
   %else %do;
      data _tmp_;
         set _SScen_; 
      run; 
   %end;

   %orvu_append(base = _foo_, new = _tmp_, canUseAppend = false);
%mend _fke_Handle_Scenarios;


/***** get models from FREQMODEL dataset *****/
%macro _fke_Handle_Fitted_Models(dsName =, CCUID =, sourceType =, hasOnlyFirmKRIs = );
   /***********************************************************************/
   /* Start by handling the case where one has a fitted model scenario,   */
   /* i.e. the case where a model is fitted and then copied as scenario.  */
   /* Presently the UI does not allow KRI fits to be copied such. So we   */
   /* first check whether the frequency model is "User Defined" and copy  */
   /* it as such when that is the case. If it is not user defined then it */
   /* shouldbe handled as a regular fit, so we let it follow the rest of  */
   /* this handling                                                       */
   /***********************************************************************/
   %if %upcase(&sourceType) = F %then %do;
      %let USERDEFINED_DIST = FALSE;
      %_fke_Handle_User_Defined_Dist(
                                    dsName         = &&DSNAME&_fke_index, 
                                    CCUID          = &&CCUID&_fke_index,   
                                    sourceType     = &&SOURCE_TYPE&_fke_index
                                    );

      %if %upcase(&USERDEFINED_DIST) EQ TRUE %then %do;
         %orvu_append(base=_foo_, new=_tmp_, canUseAppend=false);
         %goto DONE_fke_Handle_Fitted_Models;
      %end;
      %else %do;
         data _TMP_; 
            set  &&DSNAME&_fke_index; 
            where CCUID = &ccuid;
            Source = "&sourceType";
            keep ccuid distribution source FParam_:;
        run;
        %goto COMPLETE_Handle_Fitted_Models;
      %end;
   %end;

   %local breakReason;
   %let breakReason = NONE;

   %let FREQ_ITERATION = -1;   /* initializing */
      /* Both Internal & Consortium data have Iteration */
      PROC SQL noprint;
         SELECT 
               Iteration, 
               Freq_Fit_Interval 
         INTO 
               :FREQ_ITERATION, 
               :FITINTERVALUSED
         FROM 
               &dsName
         WHERE 
               CCUID = &CCUID
         ;
      QUIT;

      data _fke_models1;
         set &&FREQMODEL&_fke_index;
         where 
               CCUID     = &CCUID 
               AND 
               Iteration = &FREQ_ITERATION;
      run;

      /*** check for empty dataset ***/
      %let nobs=0;
      %orvu_getObsCount(_fke_models1,nobs);

      %if &nobs LE 0 %then %do;
         %put &orvar_error %sysfunc(sasmsg(&orvads, fke_FModels_empty_warning, noquote));
         %goto DONE_fke_Handle_Fitted_Models;
      %end;    

      proc sql noprint;
         SELECT 
               StratumModel
         INTO 
               :Model1-:Model&nobs
         FROM 
               _fke_models1
         ;
         %if 
            "&sourceType" EQ "I" 
            OR
            "&sourceType" EQ "T" 
         %then %do;
            SELECT 
                  internal_bl_uid
            INTO 
                  :ibl1-:ibl&nobs
            FROM 
                  _fke_models1
            ;
         %end;
      quit;

      /***- Merge in bl and firm KRIs  -***/
      %let COUNT_KRI = -1;        /* initializing */
      %let FREQ_KRI_LIST = %str();  

      /* Internal and CTool data may or may not have KRIs */
      %if 
         "&sourceType" EQ "I" 
         OR 
         "&sourceType" EQ "T" 
      %then %do;
         PROC SQL noprint;
            SELECT 
                  COUNT_KRI 
            INTO 
                  :COUNT_KRI
            FROM 
                  &dsName
            WHERE 
                  CCUID = &CCUID
            ;
         quit;
         %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
            %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_count_selected_kri_vars_error, noquote));
            %let abort=1;
            %let reason=%sysfunc(sasmsg(&orvads, fke_cant_count_selected_kri_vars_error, noquote));
            %let breakReason = DONE;
            %goto DONE_fke_Handle_Fitted_Models;
         %end;

         %if %sysevalf(&COUNT_KRI + 0 GT 0, BOOLEAN) %then %do;
            PROC SQL noprint;
               SELECT 
                     SelectedKRIs 
               INTO 
                     :FREQ_KRI_LIST
               FROM 
                     &dsName /* Fix (1/2) for wrong way of keeping fits --BEGIN-- */
               WHERE 
                     CCUID = &CCUID 
                  AND 
                     status NE 1
               ; /* Only after the AND */
                /* Fix (1/2) for wrong way of keeping fits --BEGIN-- */
            QUIT;
            %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
               %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_get_kri_vars_list_error, noquote));
               %let abort=1;
               %let reason=%sysfunc(sasmsg(&orvads, fke_cant_get_kri_vars_list_error, noquote));
               %let breakReason = DONE;
               %goto DONE_fke_Handle_Fitted_Models;
            %end;

            %let FREQ_KRI_LIST = %trim(&FREQ_KRI_LIST);
         %end;
      %end;
         
         /*** Merge KRI info into _fke_models1 to make _fke_data2 ***/
      %if &_freqKriAll_exists_ and %sysevalf(&COUNT_KRI+0 GT 0, BOOLEAN) %then %do;
         %if &hasOnlyFirmKRIs %then %do;
            /* Then _freqKriAll has only one observation.  Has variables for:  
            the value of each firm-KRI,
            and RISK_FACTOR_VARIABLE_DATE (date for which we are doing the prediction of the mean/variance) */
            data _fke_data2;
               if _n_ EQ 1 then set _FreqKriAll_;
               set  _fke_models1 ;
            run;
         %end;
         %else %do; 
            /* Then _freqKriAll has one observation per iBL in this cell.  Has variables for:
            the value of each KRI (firm or BL or both), possibly different for each BL, 
            and RISK_FACTOR_VARIABLE_DATE (date for which we are doing the prediction of the mean/variance) */
            %if 
               "&sourceType" EQ "I" 
               OR
               "&sourceType" EQ "T" 
            %then %do;
            proc sort 
                     data = _fke_models1 
                     out  = _fke_thisfreqmodel
                     ;
                  by internal_bl_uid;
               run;
            %end;
            %else %do;
               data _fke_thisfreqmodel;
                  set _fke_models1;
               run;
            %end;
            data _fke_data2;
               merge _fke_thisfreqmodel (IN=InFreqModelData1) _FreqKriAll_;
               by internal_bl_uid; 
               if InFreqModelData1 eq 1 then output;
            run;
            %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
               %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_merge_fkeds_bl_kri_ds_error, noquote));
               %let abort=1;
               %let reason=%sysfunc(sasmsg(&orvads, fke_cant_merge_fkeds_bl_kri_ds_error, noquote));
               %let breakReason = DONE;
               %goto DONE_fke_Handle_Fitted_Models;
            %end;
         %end;
      %end;
      %else %do; 
         data _fke_data2;
            set _fke_models1;
         run;
      %end;

      /* 2. Get info about Dummies in the model (if any)     */
      /* SKIP THIS SECTION IF DATATYPE IS "C" for consortium */
      /* (no dummies were ever considered)                   */
      %let KEPT_DUMMY_COUNT = -1; /* initializing */
      %let ALL_DUMMY_COUNT = -1;  /* initializing */

      %if 
         "&sourceType" EQ "I" 
         OR 
         "&sourceType" EQ "T" 
      %then %do;
         /* Create KEPT_DUMMY_COUNT = # of significant dummies (panels) w/ this CCUID */
         /*    and ALL_DUMMY_COUNT  = # of all possible dummies (panels) w/ this CCUID */
         %let ds_nobs=0;
         %if &&BLESTS&_fke_index NE %str() %then %do;
            %let dsid=%sysfunc(open(&&BLESTS&_fke_index,I));
            %if &dsid NE 0 %then %do;
               %let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
               %let rc=%sysfunc(close(&dsid));
            %end;
            %else %do;
               %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_open_dummy_ds_error, noquote, &&BLESTS&_fke_index, &CCUID)); 
               data _TMP_; 
                  length Distribution $12;
                  length Freq_Fit_Interval $12;
                  set &dsName;
                  where CCUID = &CCUID;
                  Source      = "&sourceType";

                  if Source="T" then do; 
                     Freq_Fit_Interval = "&FITINTERVALUSED";
                  end;
                  else do;
                     Freq_Fit_Interval = "";
                  end;
               run;
/*             data _foo_;*/
/*                set _foo_ _tmp_;*/
/*             run; */
               %orvu_append(base=_foo_,new=_tmp_,canUseAppend=false);
               %goto DONE_fke_Handle_Fitted_Models:;  /* We are done for this row.  Now do the next row (in the big do-loop).  */
            %end;
         %end;
         %if &ds_nobs LE 0 %then %do; /* should have at least one obs */
            %put &orvar_error %sysfunc(sasmsg(&orvads, fke_dummy_ds_empty_warning, noquote, &&BLESTS&_fke_index, &CCUID));
            %let abort=1;
            %let reason=%sysfunc(sasmsg(&orvads, fke_dummy_ds_empty_warning, noquote, &&BLESTS&_fke_index, &CCUID));
            %let breakReason = DONE;
            %goto DONE_fke_Handle_Fitted_Models; 
         %end;
         %else %do;
            proc SQL noprint;
               SELECT 
                     count(VALUE) 
               INTO    
                     :KEPT_DUMMY_COUNT
               FROM 
                     &&BLESTS&_fke_index
               WHERE 
                     CCUID = &CCUID
                  AND 
                     Iteration = &FREQ_ITERATION
                  AND
                     Estimate NE .
               ;
            quit;
            %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
               %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_count_dummy_vars_for_count_reg_error, noquote));
               %let abort=1;
               %let reason=%sysfunc(sasmsg(&orvads, fke_cant_count_dummy_vars_for_count_reg_error, noquote));
               %let breakReason = DONE;
               %goto DONE_fke_Handle_Fitted_Models;
            %end;

            proc SQL noprint;
               SELECT 
                     count(VALUE) 
               INTO    
                     :ALL_DUMMY_COUNT
               FROM 
                     &&BLESTS&_fke_index
               WHERE 
                     CCUID = &CCUID
                  AND 
                     Iteration = &FREQ_ITERATION
               ;
            quit;
            %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
               %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_count_dummy_vars_error, noquote));
               %let abort=1;
               %let reason=%sysfunc(sasmsg(&orvads, fke_cant_count_dummy_vars_error, noquote));
               %let breakReason = DONE;
               %goto DONE_fke_Handle_Fitted_Models;
            %end;

            %if &KEPT_DUMMY_COUNT GT 0 %then %do;
               proc SQL noprint;
                  SELECT 
                        Level_Variable_Name, 
                        VALUE, 
                        Estimate 
                  INTO 
                        :PROP1 - :PROP%left(&KEPT_DUMMY_COUNT), 
                        :VAL1 - :VAL%left(&KEPT_DUMMY_COUNT), 
                        :EST1 - :EST%left(&KEPT_DUMMY_COUNT) 
                  FROM 
                        &&BLESTS&_fke_index
                  WHERE 
                        CCUID = &CCUID
                     AND
                        Iteration = &FREQ_ITERATION
                     AND
                        Estimate NE .
                  ;
               quit;
               %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
                  %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_get_dummy_vars_list_error, noquote, &CCUID, &FREQ_ITERATION));
                  %let abort=1;
                  %let reason=%sysfunc(sasmsg(&orvads, fke_cant_get_dummy_vars_list_error, noquote, &CCUID, &FREQ_ITERATION));
                  %let breakReason = DONE;
                  %goto DONE_fke_Handle_Fitted_Models;
               %end;

               data _NULL_;
                  length name value $128;
                  %do iii=1 %to &KEPT_DUMMY_COUNT;
                     name = "DUMMY&iii" ;
                     value = "Dummy_" || strip("&&PROP&iii") || "_" || strip("&&VAL&iii");
                     call symputx(name,value);
                  %end;
               run;
                 
            %end;  /* &KEPT_DUMMY_COUNT GT 0 */
         %end; /* &ds_nobs GT 0 , where ds_nobs is the num of obs in &&BLESTS&_fke_index */
      %end;  /*  "&sourceType" EQ "I" or "&sourceType" EQ "T" */



      /* Main Idea: Evaluate each stratumMean with simulation-date KRI values, and sum them up. */
      /* *************************************************************************************** */
      /* The rest of this do-loop applies to everybody,           */
      /* data WITH either KRIs or dummies or BOTH (or Without)    */
   
      /***- Get TheDistribution, TheBeta, and BetaOrOne -***/
      %let TheBeta = %str();
      %let TheDistribution = %str();
      %let BetaOrOne = %str();

      data _null_;
         set &dsName;
         /* Fix (2/2) for wrong way of keeping fits --BEGIN-- */
         where 
               CCUID = &CCUID 
            AND 
               status NE 1; /* Only after the AND */
         /* Fix (2/2) for wrong way of keeping fits --END-- */

         call symputX("TheDistribution",Distribution);

         if upcase(Distribution)=:'N' then do; /* =: operator does character comparison */
            call symputX("TheBeta", put(Beta, BEST20.));
            call symputX("BetaOrOne", put(Beta, BEST20.));
         end;
         else do;
            call symputX("BetaOrOne",1);
         end;
      run;  
      %if &TheDistribution EQ %str() %then %do; /* there is no obs with ccuid and status ne 1 */
         data _TMP_; 
            length Distribution Freq_Fit_Interval $12;

            CCUID             = &CCUID;
            Distribution      = "";
            FPARAM_1          = .;
            FPARAM_2          = .;
            Freq_Fit_Interval = "";
         run;
         %orvu_append(base=_foo_,new=_tmp_,canUseAppend=false);   
         %goto DONE_fke_Handle_Fitted_Models;
      %end;

      /***- Calc derivatives -***/
      data _fke_data3;
         set _fke_data2;
         %if 
            "&sourceType" EQ "I" 
            OR 
            "&sourceType" EQ "T" 
         %then %do;
            %do i = 1 %to &nobs;
               if internal_bl_uid = &&ibl&i then do;
                  meanWith = &&Model&i;
               end;
            %end;
         %end;
         %else %do;
            /* ????????? */
            /*  SHOULD BE ONLY ONE OBSERVATION!!!! */
            meanWith = &Model1;
         %end; 

         /***** Add columns of derivatives   ************/
         /*  e.g.  s_KRI_1102 =  ;  */
         %let nkri=1;
         %let currkri=%scan(&FREQ_KRI_LIST, &nkri, ' ');
         %do %while (&currkri ne );  
            s_&currkri = &currkri * meanWith / &BetaOrOne;  
            %let nkri=%eval(&nkri+1);
            %let currkri=%scan(&FREQ_KRI_LIST, &nkri, ' ');
         %end;

         %let nkri = %eval(&nkri-1); 

         %do _fke_i = 1 %to &KEPT_DUMMY_COUNT; 
            if &&PROP&_fke_i = &&VAL&_fke_i then do;  
               s_&&DUMMY&_fke_i = meanWith / &BetaOrOne;
            end;
            else do;
               s_&&DUMMY&_fke_i = 0;
            end;
         %end;
      run;

      data _TheResults_ _TheDerivatives_(keep=d_:);
         set _fke_data3 end=last;
         retain MTot 0;

         %let nkri=1;
         %let currkri=%scan(&FREQ_KRI_LIST, &nkri, ' ');
         %do %while (&currkri ne );  
            retain d_&currkri 0;  
            %let nkri=%eval(&nkri+1);
            %let currkri=%scan(&FREQ_KRI_LIST, &nkri, ' ');
         %end;
         %let nkri=%eval(&nkri-1); 

         %do _fke_i = 1 %to &KEPT_DUMMY_COUNT; 
            retain d_&&DUMMY&_fke_i 0;
         %end;

         Mean = meanWith;

         if Mean ne . then do;
            MTot=Mtot+Mean;

            %let nkri=1;
            %let currkri=%scan(&FREQ_KRI_LIST, &nkri, ' ');
            %do %while (&currkri ne );  
               d_&currkri = d_&currkri + s_&currkri;  
               %let nkri=%eval(&nkri+1);
               %let currkri=%scan(&FREQ_KRI_LIST, &nkri, ' ');
            %end;
            %let nkri=%eval(&nkri-1); 

            %do _fke_i = 1 %to &KEPT_DUMMY_COUNT; 
               d_&&DUMMY&_fke_i =   d_&&DUMMY&_fke_i + s_&&DUMMY&_fke_i;
            %end;
         end;

         if last then do;
            call symputX("TotalMean", put(MTot, BEST20.));
            /* Create d_Intercept & d_Beta  */
            %if &TheDistribution EQ POIS %then %do;
               d_Intercept = MTot;
            %end;
            %else %do;
               d_Intercept = MTot/&TheBeta;
               d_Beta =   (-1) * MTot / (&TheBeta * &TheBeta) ;
            %end;

            output _TheDerivatives_;
            output _TheResults_;
         end;
         else do;
            output _TheResults_;
         end;  
      run;
      %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
         %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_evaluate_model_effects_error, noquote));
         %let abort=1;
         %let reason=%sysfunc(sasmsg(&orvads, fke_cant_evaluate_model_effects_error, noquote));
         %let breakReason = DONE;
         %goto DONE_fke_Handle_Fitted_Models;
      %end;  

      data _TMP_; 
         length Distribution $12;
         length Freq_Fit_Interval $12;

         CCUID        = &CCUID;
         Distribution = upcase("&TheDistribution");

         /* check whether poisson or negbin */
         %if &TheDistribution EQ POIS %then %do;
            FPARAM_1= &TotalMean; /* the mean of the Poisson distribution */
         %end;
         %else %do;
            FPARAM_1 = &TotalMean / &TheBeta; /* =n (from SAS documentation for CDF fn for NegBin)*/ 
            FPARAM_2 = &TheBeta; /* Note that 1/(1+Beta) is the probability of success p. */
         %end;

         set _TheDerivatives_;
         Source = "&sourceType";

         if Source = "T" then do; 
            Freq_Fit_Interval = "&FITINTERVALUSED";
         end;
         else do;
            Freq_Fit_Interval = "";
         end;
      run;

%COMPLETE_Handle_Fitted_Models:
      %orvu_append(base=_foo_,new=_tmp_,canUseAppend=false);
%DONE_fke_Handle_Fitted_Models:
%mend _fke_Handle_Fitted_Models;


%macro _fke_Check_Kri_Datasets(Exist =, onlyFirmKris=  );
   %local __freqKriAll_exists_ ; 
   %let   __freqKriAll_exists_ = 1;

   %let   &onlyFirmKRIs  = 0;

   %if    "&freqKriDatasetBl" ne "" 
         and 
         "&freqKriDatasetFirm" ne "" 
   %then %do;   
      /* merge the variable values in the single obs found in                 */
      /* &freqKriDatasetFirm as new columns to every obs in &freqKriDatasetBl */
      proc sort 
            data = &freqKriDatasetBl 
            out  = _freqKriDatasetBl
            ; 
         by risk_factor_variable_date; 
      run;

      proc sort 
            data = &freqKriDatasetFirm 
            out  = _freqKriDatasetFirm
            ; 
         by risk_factor_variable_date; 
      run;

      data _freqKriAll_ ;
         merge 
               _freqKriDatasetBl 
               _freqKriDatasetFirm
               ;
         by risk_factor_variable_date;
         rename BLUID_KRI = internal_bl_uid;
      run;

      proc sort data= _freqKriAll_; 
         by internal_bl_uid; 
      run;
   %end;
   %else %if "&freqKriDatasetBl" ne "" %then %do;
      data _freqKriAll_;
         set &freqKriDatasetBl;
         rename BLUID_KRI=internal_bl_uid;
      run;

      proc sort data= _freqKriAll_; 
         by internal_bl_uid; 
      run;
   %end;
   %else %if "&freqKriDatasetFirm" ne "" %then %do;
      %let &onlyFirmKRIs = 1;

      data _freqKriAll_;
         set &freqKriDatasetFirm;
      run;
   %end;  
   %else %do;
      %let __freqKriAll_exists_ = 0;   
   %end;
   %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
      %put &orvar_error %sysfunc(sasmsg(&orvads, fke_cant_merge_all_kris_error, noquote)) ;
      %let abort=1;
      %let reason=%sysfunc(sasmsg(&orvads, fke_cant_merge_all_kris_error, noquote));
   %end;

   %let &Exist = &__freqKriAll_exists_;
%mend _fke_Check_Kri_Datasets; 


%macro _fke_Handle_User_Defined_Dist(dsName=, CCUID=, sourceType=);
   data _TMP_; 
      length Freq_Fit_Interval $12;
      set &dsName;
      where CCUID  = &CCUID;

      if 
         Distribution NE "POIS" 
         AND 
         Distribution NE "NEGBIN" 
      then do;
         call symput('USERDEFINED_DIST','true');
      end;

      Source            = "&sourceType";
      Freq_Fit_Interval = "";
   run;
%mend _fke_Handle_User_Defined_Dist;

%macro _fke_preamble(freqPriorityData= , outData= ); 
   %put Running orva_freq_kri_effect as in VaR 4.1.;
%mend _fke_preamble;

%macro _fke_preamble42(freqPriorityData= , outData= ); 
   %put Running orva_freq_kri_effect to provide enhanced simulation algorithm for VaR 4.2.;

   %let dotLocation = %index( &freqPriorityData, .);
   %let library     = %substr(&freqPriorityData, 1, &dotLocation);

   data &outData;
      length SimPriorityData $32;
      SimPriorityData = "&library.simPriority_Sev";
   run;
%mend _fke_preamble42;
