/*****************************************************************************************
 * $Id: orva_freq_make_counts.sas,v 1.1.2.8.2.40.2.2.8.1 2012/05/03 15:22:06 sasdje Exp $
 *  Copyright (C), 2005
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orva_freq_make_counts.sas 
 *  PURPOSE: Data prep for frequency fitting, the goal is to make a count dataset.  
 *  SUPPORT: lajack
 *
 *  NOTES: FOR INTERNAL DATA: KRIs are recorded on an INTERNAL_BL_UID basis.  So we MUST include 
 *         INTERNAL_BL_UID in the set of STRATAPROPS (properties that define a stratum on which to record 
 *         a count).
 *  
 *         If INTERNAL_BL_UID was never used to define a missing range, and if no businessline KRIs 
 *         were loaded at all, then there is no reason to force INTERNAL_BL_UID to be one of the "STRATAPROPS"
 *         (i.e. there is no reason to create counts on a per-ibl basis).
 *  
 *         There may be more intervals per fit-interval, because a KRI may have been recorded at some point
 *         other than a fit-interval boundary.  
 *
 *  PARAMETERS:
 *
 *        lossdata   = the losses (e.g. VARPRJ.lossi),
 *        dataType   =   I=internal, C=consortium, E=external/public 
 *        cTool      = true or false, are we in the Consortium Tool?
 *        kribl_ds   = 
 *        krifirm_ds = Projects kri dataset with filters applied.  
 *                     EACH OF THESE DATASETS has every possible interval, from global start to BEYOND global end,
 *                     and maybe some extra intervals in between, if the user recorded KRIs at a date other than a
 *                     fit interval boundary.
 *        interval   = user-selected fit interval
 *        global_range  = the global range table
 *        missing_range = the MODIFIED missing range table.  
 *                        This was the output dataset "outmissrange" from %orva_getfreqmomts macro.
 *        inprops = some of the properties that will be used to make strata (subgroupings to make counts).  
 *                  This was the output dataset "outprops" from %orva_getfreqmomts macro.
 *                  [In addition, we must add OWN (iff dataType==I), 
 *                  plus: INTERNAL_BL_UID (if dataType==I), missing_props
 *        inmissprops = properties which have defined any missing range in the user-loaded missing range table.  
 *                      This was the output dataset "outmissprops" from %orva_getfreqmomts macro.
 *        NUMCCVS = number of cross-classification defined for the project.
 *        CCV_NAMES = space-separated list of the cross-classifications defined for the project.
 *        ccData =  Needed for input to %orva_ccmerge.  The name of the Cross Classification UID dataset with the 
 *                  following variables:  CCUID, CCVUID_<n> (one for each Cross Classification). 
 *        mapDataPrefix = Needed for input to %orva_ccmerge.   Prefix for the name of the mapping datasets. 
 *                        Should be one for each mapping called &mapDataPrefix<n>. 
 *        allibls = INTERNAL_BUSINESSLINE dataset.
 *        outstrataprops = the properties that were used to make strata (subgroupings to make counts).  
 *                         Needed by %orva_freq_calc_mean to calculate the historical mean.
 *        outCounts = output Counts dataset.
 *        outStatus = name of the out status dataset.  Will contain the following:
 *                  STATUS = 0 on success, 1 otherwise.
 *                  REASON = printable message.outfreqfittingstatus
 * 
 *****************************************************************************************/


%macro orva_freq_make_counts(  lossdata       = ,
                               dataType       = I,
                               cTool          = false,
                               kribl_ds       =,
                               krifirm_ds     =,
                               interval       = YEAR,
                               global_range   =,
                               missing_range  =,
                               inprops        = ,
                               inmissprops    = ,
                               NUMCCVS        =2,
                               CCV_NAMES      = INTERNAL_BL_UID INTERNAL_RC_UID,
                               ccData         = VARPRJ_M.cc,
                               mapDataPrefix  = VARPRJ_M.ccmapi_,
                               allibls        = VO4.internal_businessline,
                               outCounts      = outCounts,
                               outstrataprops = somestrataprops2,
                               outStatus      = outfreqmakecountsstatus
                            );

    %local MISSING_BLS INTALL_BLS;
    %let dataType = %upcase(&dataType);
         
    /*- create the status data set up front -*/
    %let reason=%sysfunc(sasmsg(&orvads, unknown_error, noquote));
    %orva_status_ds( statusds=&outStatus, status=1, reason=&reason );
    
    %let status=0;
    %let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote));
    %let abort=0;

    /*- error checking -*/
    %if &lossData=%str() %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fmc_specify_lossdat_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, fmc_lossData_missing_error, noquote));
        %goto DONE;
    %end;
    %if &global_range=%str() %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fmc_specify_global_range_ds_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, fmc_specify_global_range_ds_error, noquote));
        %goto DONE;
    %end;
    data _null_; set &global_range; 
        call symputX( "range", end_date-start_date+1);
        call symputX( "global_end", end_date);
        call symputX( "global_start", start_date);
    run;
    %if %eval(&range) LE 0 %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fmc_start_not_before_end_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, fmc_start_not_before_end_error, noquote));
        %goto DONE;
    %end;

/******************************************************************************************/
/*                          STEP 1: Create merged KRI dataset                             */
/******************************************************************************************
 *  1) Get firm KRIs (if any)
 *  2) Get BL KRIs (if any)
 *  3) Merge them
 *
 *  Main output from this portion is the _intall dataset, which has the merged KRIs
 *****************************************************************************************/

/****** 1. For dataType==I: Get rid of unneeded "future" KRI values from krifirm_&interval & kribl_&interval.
			(They are used later for VaR Simulation, not for Freq Fitting.) *****/ 
/****** 1. For dataType!=I: Make intervals from global_start to global_end. *****/ 
/*(a)  Adjust krifirm_&interval so that the last interval (per OWN) ends exactly on global_end date.
  (b)  Adjust kribl_&interval so that the last interval (per OWN and bl) ends exactly on global_end date.
*/
	%let KRIS_PRESENT = false;
	%let KRISBL_PRESENT = false;
	%let KRISFIRM_PRESENT = false;

	%if &dataType EQ I %then %do;
	    /*  Handle KRIBL dataset */
        %orv_handle_kribl_data;

      /*  Handle KRIFIRM dataset */
        %let nobs=%orv_getNumRec(dsn=&krifirm_ds);
		%let kricount=0;
		%if &nobs NE 0 %then %do;
			proc contents data=&krifirm_ds out=_fmc_kritemp(keep=NAME) noprint; run;                        
			data _NULL_;
				set _fmc_kritemp end=last;
				if SUBSTRN(NAME, 1, %length(KRI)) EQ "KRI" then kricount+1; 
				if last then call symputx ("kricount", kricount);
			run;
		%end;
		%if &kricount GT 0 %then %do; /* kri_firm present */
            %orv_handle_krifirm_data;
		%end; 
		%else %do; /* no kri_firm present */
			%if &KRISBL_PRESENT = true %then %do;
				data _intall;
					set _intbl;        /* if CTool, this already has OWN */
					%if (&cTool EQ true) %then %do;
						if missing(OWN) then OWN = 0;
					%end;
				run;
			%end;
			%else %do;  /* neither kri_bl nor kri_firm present */
			/*** Create a dataset w/ global start and global end dates. To be expanded by orva_interval_datasets.***/
                %orv_createInterval(intervalStart=&global_start, intervalEnd=&global_end, 
                                interval=&interval, cTool=&cTool);
			%end;
		%end;
	%end; 

	%else %do; /* whenever dataType NE I */ 
		/*** Create a dataset w/ global start and global end dates. To be expanded by orva_interval_datasets.***/
        %orv_createInterval(intervalStart=&global_start, intervalEnd=&global_end, 
                        interval=&interval, cTool=&cTool);
	%end;

    %if &KRISBL_PRESENT = true %then %do;
       /* Get a list of IBLs in &allibls. and compare to those in _intall.  For each IBL missing from _intall, add in
          an observation to _intall with that IBL, interval defined by global start and end dates, and all KRIs set to 0 */
       proc sql noprint;
          SELECT DISTINCT
                          BLUID_KRI
          INTO
                          :INTALL_BLS separated by " "
          FROM
                          _intall
          ;
       quit;

       proc sql noprint;
          SELECT DISTINCT
                          INTERNAL_BL_UID
          INTO
                          :MISSING_BLS separated by " "
          FROM
                          &allibls.
          WHERE
                          INTERNAL_BL_UID not in (&INTALL_BLS.)
          ;
       quit;

       %let NMISS = %sysfunc(COUNTW(%str(&MISSING_BLS.)));

       %if &NMISS. > 0 %then %do;
          data _intall;
             set _intall end=last;
             array kris kri:;
             output;
             if last then do;
                do i=1 to &NMISS.;
                   BLUID_KRI      = scan("&MISSING_BLS.",i,' ');
                   interval_start = &global_start.; 
                   interval_end   = &global_start.;
                   do j=1 to dim(kris);
                      kris[j] = 0;
                   end;
                   output;
                end;
             end;
             drop i j;
       %end;
   %end;

/******************************************************************************************/
/*               STEP 2: Create STRATAPROPS, the properties to make counts.               */
/******************************************************************************************/

/* STRATAPROPS partition the losses into strata.   One count per stratum per time interval -- 
        and there may be more time intervals than just every fit interval (if e.g. KRIs were recorded on a date that
 isn't a fit-interval boundary.)  */
/* What properties make strata?  
(1)   Any of the PANEL_PROPS (which orva_getfreqmomts saved into "outprops", called here "&inprops").

(2)   OWN (only if CTool = true and if OWN was used to define a missing range. 9/26/08.)

(3)   internal_bl_uid (only if dataType==I), 

(4)   missing_props

(5)   CCVUID_i (only if missingprops doesn't include the actual property 
        that maps to CCVUID_i.  9/26/08.    E.g., if internal_risk_category maps to CCVUID_1, and 
			internal_risk_category isn't one of the missingprops, then add CCVUID_1 to the list of STRATAPROPS.)

 
Strategy:  Start with missing props.  Add CCVUID_i if missingprops doesn't include the actual property 
        that maps to CCVUID_i.  For i=1 to NUMCCVs.
        Add internal_bl_uid if dataType==I and it's not already in missingprops.		
*/

    %let STRATAPROPS = &CCV_NAMES; 
    %let nstrataprops = &NUMCCVS;
    %if %sysfunc(findw(&CCV_NAMES, INTERNAL_BL_UID )) = 0 %then %do;
       %let STRATAPROPS = INTERNAL_BL_UID &CCV_NAMES; 
       %let nstrataprops = %eval(1 + &NUMCCVS);
    %end;

	%let nmissprops = 0;
	%if &inmissprops NE %then %do;
        %let nmissprops = %orv_getNumRec(dsn=&inmissprops);
	%end;
	%if &nmissprops GT 0 %then %do;
		proc SQL noprint;
            select 
                    missprop
            into 
                    :MISSPROPs separated by " "
            from 
                    &inmissprops
            ;    
		quit;

        
        %do i=1 %to &nmissprops.;
            %let MISSPROP&i. = %scan(&MISSPROPs.,&i.," ");
            %if %sysfunc(findw(&STRATAPROPS, &MISSPROP&i.)) = 0 %then %do;
                %let STRATAPROPS=&STRATAPROPS &MISSPROP&i.;
                %let nstrataprops = %eval(&nstrataprops +1);
	    %end;
	%end;
           
     %end;
        

/*  Make macro variable STRATAPROPS_COMMA to be used in Proc SQL statements. */

	%let STRATAPROPS_COMMA = %scan(&STRATAPROPS, 1, %str( ));
	%do i=2 %to &nstrataprops;
		%let STRATAPROPS_COMMA = &STRATAPROPS_COMMA , %scan(&STRATAPROPS, &i, %str( ))  ;
	%end;

	data &outstrataprops;
		length THESTRATAPROPS $ 1024;
		THESTRATAPROPS = "&STRATAPROPS";
	run;

/****** 3. Create design matrix with all possible combos of strataprops for which a loss occurs.******/
/* Make a temp dataset for input to &orva_designmatrix:  keep only the STRATAPROPS in lossdata */
	data _fmc_designinput;
		set &lossData(keep=&STRATAPROPS %if (&cTool EQ true) %then %do; OWN %end; );
		%if (&cTool EQ true) %then %do;        
			if missing(OWN) then OWN = 0; 
		%end;
	run;
/*  SPARSE==false only creates those combos of variable levels that actually occur in "designinput".
    Otherwise we would have waaaaay too many zero counts.  And we might be estimating for strata that
    don't exist in the customer's organization.  */
    %orva_designmatrix(
                       A             = _fmc_designinput, 
                       design = _fmc_design, 
                       sparse = false, 
                       combosonly = true, 
                       drop = false, 
                       status_design = _fmc_status_design
                      );
        
	/*******/
	%if &dataType EQ I %then %do;
		/* Get ibls (with no losses)  */
		data _fmc_temp(keep=internal_bl_uid own);
			set &allibls;
			%if (&cTool EQ true) %then %do; where internal_bl_level ne 1; %end;
			%orva_assignown();
		run;
		proc sort data=_fmc_temp out=_fmc_allibls nodup;
			by internal_bl_uid;
		run;
		data _fmc_temp(keep=internal_bl_uid);
			set _fmc_designinput;
		run;
		proc sort data=_fmc_temp out=_fmc_iblswithlosses nodup;
			by internal_bl_uid;        
		run;
		/* find the list of ibls that are in allibls but not in iblswithlosses.
		add one obs each for such ibls...   with missing values for all other props?  
		*/
		data _fmc_havenolosses;
			merge _fmc_allibls (in=inallbls) _fmc_iblswithlosses(in=inblswithlosses);
			by internal_bl_uid;
			if (inallbls) AND (^inblswithlosses);
		run;
		data _fmc_design_i;
			set _fmc_design;
		run;
		data _fmc_designlj;
			set _fmc_design _fmc_havenolosses; 
			/* appends 2nd one at bottom of 1st one */
		run;
		data data _fmc_design;
			set _fmc_designlj;
		run;
	%end;
	/*******/

	%if &KRIS_PRESENT = false %then %do;
		proc sql noprint;
            create table 
                        _int3 
            as select 
                        * 
            from 
                        _fmc_design, 
                        _intall
            ;
		quit;
	%end;
	%else %do;
	/* In _intall:  Rename "OWN" to "theOWN" to avoid collision with "OWN" in design. */
	%if (&cTool EQ true) %then %do;
		data _intall;
			set _intall (rename=(OWN=theOWN));
		run;
	%end;

	proc sql noprint;
            create table 
                            _int3
            as select 
                            distinct &STRATAPROPS_COMMA, 
                            %if (&cTool EQ true) %then %do; 
                                OWN, 
                            %end;
                            interval_start, 
                            interval_end
            from 
                            _fmc_design 
            as 
                            l left join _intall as r
            on 
                           %if ((&dataType EQ I) and (&KRISBL_PRESENT EQ true)) %then %do; 
                              l.INTERNAL_BL_UID EQ r.BLUID_KRI 
                           %end;
                           %if (&cTool EQ true) %then %do; 
                              and l.OWN EQ r.theOWN 
                           %end;
            group by 
                           &STRATAPROPS_COMMA , 
                           %if (&cTool EQ true) %then %do; 
                              OWN, 
                           %end;
                           interval_start, 
                           interval_end
            ;
	quit;

	%end;


	data _int3;
		set _int3;
		interval_length = interval_end - interval_start + 1;
	run;






/****** 4. Apply missing ranges to _int3 according to the MISSPROPS.  */
/* i.e. subtract missing_days from interval_length */
	%let flagown=0;
	%let flaginternal_bl_uid=0;    %let flaginternal_rc_uid=0;     %let flaglegal_entity_uid=0;
	%let flagprocess_uid=0;        %let flagclass_1=0;             %let flagclass_2=0; 
    %let flagclass_3=0;            %let flagclass_4=0;             %let flagclass_5=0; 
    %let flagclass_6=0;            %let flagclass_7=0;  	       %let flagclass_8=0; 
    %let flagclass_9=0;            %let flagclass_10=0;

	%if &nmissprops GT 0 %then %do;
		%do i=1 %to &nmissprops;
			%let flag&&MISSPROP&i = 1;
		%end;        
	%end;

	%let nobs = 0;
	%if &missing_range NE %then %do;
        %let nobs = %orv_getNumRec(dsn=&missing_range);
	%end;
	%if &nobs GT 0 %then %do;
		data missing_range_adjusted;
			set &missing_range;
		run;
		%if &nmissprops EQ 0 %then %do;
			/* ERROR!  There are NO missing props, but there ARE missing range table observations??? */
			%put &orvar_error %sysfunc(sasmsg(&orvads, fmc_range_tblobs_missing_error, noquote)) ;
			%let abort=1;
			%let reason=%sysfunc(sasmsg(&orvads, fmc_range_tblobs_missing_error, noquote));
			%goto DONE;
		%end;
	%end;
	%else %do;
		/* There was no missing range table outputted from orva_getfreqmomts; or it wasn't passed to this macro.   
		This is only an error if there were missing props outputted. */
		%if &nmissprops GT 0 %then %do;
			/* ERROR!  There ARE missing props, but there are no missing range table observations??? */
			%put &orvar_error %sysfunc(sasmsg(&orvads, fmc_props_missing_error, noquote));
			%let abort=1;
			%let reason=%sysfunc(sasmsg(&orvads, fmc_props_missing_error, noquote));
			%goto DONE;
		%end;
	%end;     

	%if &nmissprops GT 0 and &nobs GT 0 %then %do;  /* if there is A MISSING RANGE TABLE and missing props... */
		/* Consider 4 cases:  
		Case 1: Missing range falls inside interval.  
		Case 2: start missing date is before interval start, end missing date is w/in interval 
		Case 3: start missing date is w/in interval, end missing date is after interval end 
		Case 4: Missing range fully contains interval.  */

		proc sql noprint;
            create table 
                  _int4
            as select 
                  distinct &STRATAPROPS_COMMA, 
                  %if (&cTool EQ true) %then %do; 
                     OWN, 
                  %end;
                     interval_start, 
                  interval_end,  
                     (min(interval_end,end_missing_date)-max(interval_start,start_missing_date)+1)*(missing(start_missing_date)-1)*(-1) as the_missing_length,
                     interval_length-((min(interval_end,end_missing_date)-max(interval_start,start_missing_date)+1)*(missing(start_missing_date)-1)*(-1) ) as adj_length         
            from 
						_int3 
			as 
						l left join missing_range_adjusted as r

            on 
                  (
                     (l.interval_start LE r.start_missing_date AND l.interval_end GE r.end_missing_date) /* case 1 */
			OR (l.interval_start LT r.end_missing_date AND l.interval_end GE r.end_missing_date) /* case 2 */
			OR (l.interval_start LE r.start_missing_date AND l.interval_end GT r.start_missing_date) /* case 3 */
			OR (l.interval_start GT r.start_missing_date AND l.interval_end LT r.end_missing_date) /* case 4 */
			) /*Note the above 4 conditions exclude two cases that the missing intervals completely outside the interval(left or right) */
                     AND(
                     (%if &flaginternal_bl_uid %then missing(r.miss_internal_bl_uid) OR missing(l.internal_bl_uid) OR l.internal_bl_uid EQ r.miss_internal_bl_uid; %else 1; )
			AND ( %if &flaginternal_rc_uid %then missing(r.miss_internal_rc_uid) OR missing(l.internal_rc_uid) OR l.internal_rc_uid EQ r.miss_internal_rc_uid; %else 1; )
			AND ( %if &flaglegal_entity_uid %then missing(r.miss_legal_entity_uid) OR missing(l.legal_entity_uid) OR l.legal_entity_uid EQ r.miss_legal_entity_uid; %else 1; )
			AND ( %if &flagprocess_uid %then missing(r.miss_process_uid) OR missing(l.process_uid) OR l.process_uid EQ r.miss_process_uid; %else 1; )
			AND ( %if &flagclass_1 %then missing(r.miss_class_1) OR missing(l.class_1) OR l.class_1 EQ r.miss_class_1; %else 1;)
			AND ( %if &flagclass_2 %then missing(r.miss_class_2) OR missing(l.class_2) OR l.class_2 EQ r.miss_class_2; %else 1; )
			AND ( %if &flagclass_3 %then missing(r.miss_class_3) OR missing(l.class_3) OR l.class_3 EQ r.miss_class_3; %else 1; )
			AND ( %if &flagclass_4 %then missing(r.miss_class_4) OR missing(l.class_4) OR l.class_4 EQ r.miss_class_4; %else 1; )
			AND ( %if &flagclass_5 %then missing(r.miss_class_5) OR missing(l.class_5) OR l.class_5 EQ r.miss_class_5; %else 1; )
			AND ( %if &flagclass_6 %then missing(r.miss_class_6) OR missing(l.class_6) OR l.class_6 EQ r.miss_class_6; %else 1; )
			AND ( %if &flagclass_7 %then missing(r.miss_class_7) OR missing(l.class_7) OR l.class_7 EQ r.miss_class_7; %else 1; )
			AND ( %if &flagclass_8 %then missing(r.miss_class_8) OR missing(l.class_8) OR l.class_8 EQ r.miss_class_8; %else 1; )
			AND ( %if &flagclass_9 %then missing(r.miss_class_9) OR missing(l.class_9) OR l.class_9 EQ r.miss_class_9; %else 1; )
			AND ( %if &flagclass_10 %then missing(r.miss_class_10) OR missing(l.class_10) OR l.class_10 EQ r.miss_class_10; %else 1; )
			AND ( %if &flagown %then missing(r.miss_own) OR missing(l.own) OR l.own EQ r.miss_own; %else 1; )
			)
			/*        either match all other props, or "missing" in r matches everything in l (per property), and vice versa...*/

            order by 
                  &STRATAPROPS_COMMA , 
                  %if (&cTool EQ true) %then %do; 
                     OWN, 
                  %end;
                     interval_start, 
                  interval_end
         ;
		quit;

	%end;
	%else %do; /* There are no missing ranges */
		data _int4;
			set _int3(rename=(interval_length=adj_length));
			the_missing_length = 0;
		run;
	%end;

/******* 5. Make counts ******/

	/* Make a temp dataset w/ columns we need:  keep only the STRATAPROPS & FIRST DATE OF EVENT  & THRESHOLD */

	data _fmc_work1(compress=binary);
		/* No need to keep OWN. _int4 will have it. */
		set &lossData(keep=&STRATAPROPS FIRST_DATE_OF_EVENT THRESHOLD CCVUID_: CCUID);
		%let nstrataprops=1;
		%let thisprop = %scan(&STRATAPROPS, &nstrataprops, %str( ));
		%do %while (&thisprop NE ); 
			rename &thisprop = &thisprop.work;
			%let nstrataprops = %eval(&nstrataprops+1);        
			%let thisprop = %scan(&STRATAPROPS, &nstrataprops, %str( ));
		%end;
		%let nstrataprops = %eval(&nstrataprops-1);
		count=1;
	run;

	/* Note the "LE" and "GE" comparing the dates, below.  We are finding the right interval... */
	proc sql noprint;
		create table _counts1
		as select distinct &STRATAPROPS_COMMA, %if (&cTool EQ true) %then %do; OWN, %end;
			THRESHOLD, /* CCVUID_0, CCVUID_1,*/ CCUID, interval_start, interval_end, adj_length, 
			sum(count) as the_count
		from _int4 as l left join _fmc_work1 as r
		on 
			%let nstrataprops=1;
			%let thisprop = %scan(&STRATAPROPS, &nstrataprops, %str( ));
			%do %while (&thisprop NE ); 
				(l.&thisprop = r.&thisprop.work) AND
				%let nstrataprops = %eval(&nstrataprops+1);
				%let thisprop = %scan(&STRATAPROPS, &nstrataprops, %str( ));
			%end;
			%let nstrataprops = %eval(&nstrataprops-1);
			(l.interval_start LE r.FIRST_DATE_OF_EVENT) AND (l.interval_end GE r.FIRST_DATE_OF_EVENT)
		group by &STRATAPROPS_COMMA , %if (&cTool EQ true) %then %do; OWN, %end;
			THRESHOLD, CCUID, interval_start, interval_end;
	quit;

/*  FOR DEBUGGING: sum of counts should add up to total number of observations in the loss table */
/*        proc summary data=_counts1;*/
/*                var the_count;*/
/*                output out=_countsummary sum(the_count)=sumcount;*/
/*        run;*/



/****** 6. Add the columns of KRI values to the count dataset. ******/
	%if &KRIS_PRESENT EQ true %then %do;
		data _intallrenamed;
			set _intall(drop=interval_end rename=(interval_start=theinterval_start));
			/* I already renamed OWN to be theOWN. */
		run;
		proc sql noprint;
            create table 
                  _counts2 
         as select 
                  *
            from 
                  _counts1 
         as 
                  l left join _intallrenamed as r
            on 
                  %if (&cTool EQ true) %then %do; 
                     l.OWN EQ r.theOWN and  
                  %end;
                     %if ((&dataType EQ I) and (&KRISBL_PRESENT EQ true)) %then %do; 
                     l.INTERNAL_BL_UID EQ r.BLUID_KRI and 
				%end;
                     l.interval_start = r.theinterval_start
         ;
		quit;
		data _counts3;
			set _counts2(drop= theinterval_start  
				%if (&cTool EQ true) %then %do;  theOWN %end;
				%if ((&dataType EQ I) and (&KRISBL_PRESENT EQ true)) %then %do; BLUID_KRI %end;
			);
			if missing(the_count) then the_count=0;
		run;
	%end;
	%else %do; /* There are no KRIs */
		data _counts3;
			set _counts1;
			if missing(the_count) then the_count=0;
		run;
	%end;

/*  FOR DEBUGGING: sum of counts should add up to total number of observations in the loss table */
/*        proc summary data=_counts3;*/
/*                var the_count;*/
/*                output out=_countsummary sum(the_count)=sumcount;*/
/*        run;*/


/***** 7.  Create columns for CCVUID_<x> and CCUID. *****/

	%let nfix=1;
    %let this = %scan(&CCV_NAMES, &nfix, %str( ));
	%do %while (&this NE ); 
        %orv_map_one_ccv(mapDataPrefix =&mapDataPrefix, 
			col           =&this, 
			num           =%eval(&nfix-1),
			data          =_counts3, 
			outdata       =_counts4);

		data _counts3; set _counts4; run;
		%let nfix = %eval(&nfix + 1);
        %let this = %scan(&CCV_NAMES, &nfix, %str( ));
	%end;
	%let nfix = %eval(&nfix - 1);

	/*- merge the _counts with the CC Dataset to get CCUID -*/
	data _counts3(drop= CCUID); set _counts3; run;
	proc sql noprint; 
		create table _counts4
		as select * from _counts3 L left join &ccData C
		on L.ccvuid_0=C.ccvuid_0
			%do i = 1 %to &numCCVs-1; 
				and L.ccvuid_&i=C.ccvuid_&i
			%end;
		;   
	quit;



	data &outCounts(compress=binary);
		set _counts4;
        format start_of_curr_fitinterval end_of_curr_fitinterval interval_start interval_end date9.;
		start_of_curr_fitinterval = intnx("&interval", interval_start, 0);
		end_of_curr_fitinterval = intnx("&interval", interval_start, 1);
		exposure = adj_length / (end_of_curr_fitinterval - start_of_curr_fitinterval);
		/* Note that obs with adj_length==0 are not deleted.  We are keeping obs with exposure==0. */
	run;

%DONE:
     
	proc datasets library=work memtype=data nolist;
		delete _int: _fmc_: _counts:;
	quit;

	%if &abort %then %do;
		%put &orvar_error %sysfunc(sasmsg(&orvads, fmc_ffting_abnormal_error, noquote));
		%orva_status_ds( statusds=&outStatus, status=1, reason=&reason );
	%end;  
	%else %do;
		%orva_status_ds( statusds=&outStatus, status=0, reason=&reason );
	%end;

%mend orva_freq_make_counts;


/****************************************************************************************************/
/* if kris_bl also presents, Merge firm and BL KRIs, while maintaining possibly different intervals */
/****************************************************************************************************/ 
%macro orv_merge_kri_interval_datasets( 
                                        intbl     =, 
                                    intfirm=, 
                                    intall=, 
                                        outStatus = _fmc_outmergekridatasets
                                      );

/*- create the status data set up front -*/
	%let reason=%sysfunc(sasmsg(&orvads, unknown_error, noquote));
	%orva_status_ds( statusds=&outStatus, status=1, reason=&reason );
	%let status=0;
	%let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote));
	%let abort=0;

	%if &cTool EQ true %then %do;
		proc freq noprint data=&intbl;
			tables BLUID_KRI * OWN / out=_justbls;
		run;
	%end;
	%else %do;
		proc freq noprint data=&intbl;
			tables BLUID_KRI / out=_justbls;
		run;
	%end;

    %let nbluids=%orv_getNumRec(dsn=_justbls);

	%if &nbluids EQ 0 %then %do;
		/* ERROR!    */
		%put &orvar_error %sysfunc(sasmsg(&orvads, fmc_zero_internal_bl_error, noquote));
		%let abort=1;
		%let reason=%sysfunc(sasmsg(&orvads, fmc_zero_internal_bl_error, noquote));
        %goto DONE_merge_kri_interval_datasets;
	%end;
	/* Otherwise, continue.  */
	/* Remember, the sets of internal_bl_uids (per owner) are pairwise non-intersecting. */

	data _justbls2;
		set _justbls(keep=BLUID_KRI %if &cTool EQ true %then %do; OWN %end;);
	run;

	data _intfirm2;
		set &intfirm %if &cTool EQ true %then %do; (rename=(OWN=OWN_F)) %end; ;
	run;

	proc sql noprint;
		create table _firmkrisbyibl
		as select *
		from _justbls2 as l left join _intfirm2 as r

		on %if &cTool EQ true %then %do; l.OWN EQ r.OWN_F and %end;
			1
		order by %if &cTool EQ true %then %do; OWN, %end; BLUID_KRI, interval_start;
	quit;

	proc sort data=_firmkrisbyibl(keep=%if &cTool EQ true %then %do; OWN %end; BLUID_KRI interval_start interval_end);
		by %if &cTool EQ true %then %do; OWN %end; BLUID_KRI interval_start;
	run;

    /* The above handling of the _intfirm and _intbls is for this purpose: apply the intfirm definitions(here only 
     *  the interval_start and interval_end are interested) to each BLUID_KRIs(as the result of left joint in the 
     *  proc SQL). Result: The interval_start and interval_end that are defined in intfirm are applied to each 
     *  BLUID_KRIs
     */ 

	proc sort data=&intbl(keep=%if &cTool EQ true %then %do; OWN %end; BLUID_KRI interval_start interval_end) out=_fmc_blkris2;
		by %if &cTool EQ true %then %do; OWN %end; BLUID_KRI interval_start;
	run;

	data _all;
		set _firmkrisbyibl _fmc_blkris2; 
		by  %if &cTool EQ true %then %do; OWN %end;  BLUID_KRI interval_start;
	run;

	/* Get rid of identical intervals */
	proc sql noprint;
		create table _allsql
		as select distinct  %if &cTool EQ true %then %do; OWN, %end; BLUID_KRI, interval_start, interval_end
		from _all;
	quit;

	/* Get rid of too long intervals, when shorter broken-up intervals exist that, 
	taken together, cover the too-long interval. */
	proc sort data=_allsql;
		by  %if &cTool EQ true %then %do; OWN %end; BLUID_KRI interval_start interval_end;
	run;        
	data _fmc_better(drop=prev:);
		retain  %if &cTool EQ true %then %do; prev_OWN %end; prev_BLUID prev_end;
		set _allsql;
		if _N_ EQ 1 then do;
			%if &cTool EQ true %then %do; prev_OWN = OWN; %end; 
			prev_BLUID = BLUID_KRI;
			prev_end = interval_end;        
		end;
		else do;
			if ( %if &cTool EQ true %then %do; (OWN=prev_OWN) and %end;
					(BLUID_KRI=prev_BLUID) and 
					(interval_start NE prev_end+1)
				) 
			then do;
				delete; 
			end;
			else do;
				%if &cTool EQ true %then %do; prev_OWN = OWN; %end; 
				prev_BLUID = BLUID_KRI;
				prev_end = interval_end;        
			end;
		end;
	run;


	/**** FIND ibl KRIs and firm KRIs that were in effect during the various intervals...  ****/
	/* First, ibl KRIS: */
	data _intblrename(keep=the_: KRI_:);
		set &intbl(rename=(interval_start=the_interval_start interval_end=the_interval_end  
			%if &cTool EQ true %then %do; OWN=the_OWN %end; BLUID_KRI=the_BLUID_KRI));
	run;
	proc sql noprint;
		create table _withblkris
		as select *
		from _fmc_better as l left join _intblrename as r 
		on l.interval_start GE r.the_interval_start 
			AND l.interval_end LE r.the_interval_end /*  b/c of how I created better, shouldn't need this 2nd condition.. */
			%if &cTool EQ true %then %do; AND l.OWN EQ r.the_OWN %end;
			AND l.BLUID_KRI EQ r.the_BLUID_KRI ;
	quit;
	data _withblkris;
		set _withblkris(drop=the_:);
	run;

	/* Next, firm KRIS:                                  */
	/* NOTE:  FIRM LEVEL KRIS WILL BE DIFFERENT BY OWNER!*/
	/* first rename kris for Firm with _F                */
	%let kriFirmNames= ;
	proc contents data=&intfirm(keep=KRI_:) out=_c(keep=NAME LABEL) noprint; run;
	proc sql noprint;
		select NAME, LABEL
		into :kriFirmNames separated by " ",	
			:kriFirmLabels separated by ","
		from _c;
	quit;
	data _intfirmrename(keep=the_: KRI_:);
		set &intfirm(rename=(interval_start=the_interval_start interval_end=the_interval_end  
			%if &cTool EQ true %then %do; OWN=the_OWN %end;
			%let nkris=1;
			%let kri=%scan(&kriFirmNames, &nkris, %str( ));
			%do %while (&kri ne ); 
				&kri=&kri._F  
				%let nkris=%eval(&nkris+1);
				%let kri=%scan(&kriFirmNames, &nkris, %str( ));
			%end;   
		));
		label 
			%let nkris=1;
			%let kri=%scan(&kriFirmNames, &nkris, %str( ));
			%let krilabel="%trim(%left(%scan(%bquote(&kriFirmLabels),&nkris, %str(,) )))(F)"; 
			%do %while (&kri ne ); 
				&kri._F = &krilabel
				%let nkris=%eval(&nkris+1);
				%let kri=%scan(&kriFirmNames, &nkris, %str( ));
				%let krilabel="%trim(%left(%scan(%bquote(&kriFirmLabels),&nkris, %str(,) )))(F)";
			%end;   
		;
	run;



	proc sql noprint;
		create table _withboth
		as select *
		from _withblkris as l left join _intfirmrename as r 
		on l.interval_start GE r.the_interval_start 
			AND l.interval_end LE r.the_interval_end /*  b/c of how I created better, shouldn't need this 2nd condition.. */
			%if &cTool EQ true %then %do; AND l.OWN EQ r.the_OWN %end; 
		;
	quit;
	data &intall;
		set _withboth(drop=the_:);
	run;



%DONE_merge_kri_interval_datasets:
	%if &abort %then %do;
		%put &orvar_error %sysfunc(sasmsg(&orvads, fmc_abnormal_error, noquote));
		%orva_status_ds( statusds=&outStatus, status=1, reason=&reason );
	%end;  
	%else %do;
		%orva_status_ds( statusds=&outStatus, status=0, reason=&reason );
	%end;

	proc datasets library=work memtype=data nolist;
		delete _justbls: 
               _all: 
               _with: 
               _firmkrisbyibl 
               _fmc_: ;
	quit;


%mend orv_merge_kri_interval_datasets;

%macro orv_map_one_ccv(
                       mapDataPrefix = varprj.ccmapi_, 
                   col           =INTERNAL_BL_UID, 
                   num           =0, 
                   data          =, 
                       outdata       =
                      );

	proc sql noprint; 
		create table &outdata(drop=ccmap_:) 
			as select * from &data L left join &mapDataPrefix.&num M
		on L.&col = M.ccmap_&num;   
	quit;

%mend orv_map_one_ccv;


/******************************************************************************************
 *The macro reads the dataset dsn and return the number of record (i.e. by querying nlobs *
 ******************************************************************************************/
%macro orv_getNumRec(dsn=);
    %local nRecs dsid rc;
    %let nRecs=0;
	%let dsid=%sysfunc(open( &dsn, I));                                                                                        
	%if &dsid NE 0 %then %do;                                                                                                      
		%let nRecs=%sysfunc(attrn(&dsid,nlobs));                                                                                     
    	%let rc=%sysfunc(close(&dsid));                                                                                             
	%end;   
    &nRecs 
%mend orv_getNumRec;


/********************************************************************************************\ 
 * The macro read the dataset amd make sure the interval start and end are aligned with the *
 * global end.                                                                              *
 * in:  &kribl_ds,  &global_end                                                             *
 * out: &KRIS_PRESENT, &KRISBL_PRESENT, _intbl;                                             *
\********************************************************************************************/
%macro orv_handle_kribl_data;
        %local nobs dsid kricount; 
        %let nobs = %orv_getNumRec(dsn=&kribl_ds);
		%let kricount=0; 
		%if &nobs NE 0 %then %do;
			proc contents data=&kribl_ds out=_fmc_kritemp(keep=NAME) noprint; run;
			data _fmc_kritemp; set _fmc_kritemp end=LAST; 
				where SUBSTRN(NAME, 1, %length(KRI_)) EQ "KRI_"; 
				if last then call symputx("kricount", _N_);
			run;
		%end;
		%if &kricount NE 0 %then %do;
			%let KRIS_PRESENT = true;
			%let KRISBL_PRESENT = true;
			data _intbl;
				set &kribl_ds(drop=imputed_: interval_length);
				where interval_start LE &global_end and interval_end GT &global_start ;
				if interval_end GT &global_end then do;
					interval_end = &global_end;
				end;
				if interval_start LT &global_start then do;
					interval_start = &global_start;
				end;
			run;
			proc sort data=_intbl out=_intbl;
				by BLUID_KRI interval_start;
			run;
		%end;
%mend orv_handle_kribl_data;

/***************************************************************************************\
 *The macro                                                                            * 
 * 1. reads the dataset and make sure the interval end is aligned with the global end.  *
 * 2. merges the kris with kri_bl if it presents.                                       *
 * in: &krifirm_ds,  &global_end, KRISBL_PRESENT                                       *
 * out: &KRIS_PRESENT, &KRISFIRM_PRESENT, _intfirm, _intall;                           *
 \**************************************************************************************/
%macro orv_handle_krifirm_data;
	%let KRIS_PRESENT = true;
	%let KRISFIRM_PRESENT = true;
	data _intfirm;
		set &krifirm_ds(drop=interval_length);
		where interval_start LE &global_end and interval_end GT &global_start ;
		if interval_end GT &global_end then do;
			interval_end = &global_end;
		end;
		if interval_start LT &global_start then do;
			interval_start = &global_start;
		end;
	run;
	proc sort data=_intfirm out=_intfirm;
		by interval_start;
	run;
    /* if kris_bl also presents, Merge firm and BL KRIs, while maintaining possibly different intervals */    
    %if &KRISBL_PRESENT = true %then %do;
        %orv_merge_kri_interval_datasets(intbl=_intbl, intfirm=_intfirm, intall=_intall, outStatus=_fmc_outmergekridatasets);
	%end;
	%else %do; /* only kri_firm presents */
    	/* NOTE:  FIRM LEVEL KRIS WILL BE DIFFERENT BY OWNER! */
    	/*** first rename kris for Firm with _F  ***/
        %local kriFirmNames kriFirmLabels;
		%let kriFirmNames= ;
		proc contents data=_intfirm(keep=KRI_:) out=_c(keep=NAME LABEL) noprint; run;
		proc sql noprint;
			select NAME, LABEL
			into :kriFirmNames separated by " ",	
			:kriFirmLabels separated by ","
			from _c;
		quit;
		data _intfirmrename; /* adjust variable names and labels to have "_F" or "(F)" */
			set _intfirm(rename=(
            %local nkris kri;
			%let nkris=1;
			%let kri=%scan(&kriFirmNames, &nkris, %str( ));
			%do %while (&kri ne ); 
				&kri=&kri._F
				%let nkris=%eval(&nkris+1);
				%let kri=%scan(&kriFirmNames, &nkris, %str( ));
			%end;   
			));
			label 
			%let nkris=1;
			%let kri=%scan(&kriFirmNames, &nkris, %str( ));
            %local krilabel;
			%let krilabel="%trim(%left(%scan(%bquote(&kriFirmLabels),&nkris,',')))(F)";
			%do %while (&kri ne ); 
				&kri._F = &krilabel
				%let nkris=%eval(&nkris+1);
				%let kri=%scan(&kriFirmNames, &nkris, %str( ));
				%let krilabel="%trim(%left(%scan(%bquote(&kriFirmLabels),&nkris,',')))(F)";
			%end;   
			;
		run;
		data _intall;
    		set _intfirmrename;
    		%if (&cTool EQ true) %then %do;
    			if missing(OWN) then OWN = 0;
    		%end;
		run;
	%end; /* end of case that only kri_firm presents */
%mend orv_handle_krifirm_data;


/******************************************************************************\
 * The orv_createInterval macro creates intervals(each per interval) according to *
 * %orva_interval_datasets.                                                   *
 * Input: intervalStart, intervalEnd, and the interval definition             *
 * Output: _intall dataset.                                                   *   
\******************************************************************************/
%macro orv_createInterval(
                           intervalStart =, 
                           intervalEnd   =, 
                           interval      =, 
                           cTool         = 
                         );
		data _fmc_plainintervals;
			format interval_start interval_end date.;
			interval_start = &intervalStart;
			interval_end = &intervalEnd;
		run;
		%orva_interval_datasets(Withsort = _fmc_plainintervals,
			intervalout = _fmc_plainintervals2,
		    interval    = &interval);
		data _intall;
			set  _fmc_plainintervals2;
			%if (&cTool EQ true) %then %do;
				if missing(OWN) then OWN = 0;
			%end;
		run;
%mend orv_createInterval;
