/*****************************************************************************************
 * $Id: orva_freqsim.sas,v 1.1.4.17.2.15.2.4.2.1.2.1 2012/05/03 14:53:18 sasdje Exp $
 *   Copyright (c) 2004 by SAS Institute Inc.  Cary NC 27513  USA
 *
 *  NAME:        ORVA_freqsim.sas                                             
 *  DESCRIPTION: Compute correlated draws frequency frequency of the losses.
 *          The correlations are computed using ORVA_FREQCORR. The covariance
 *          matrix computed should be from the normal copula.
 *  SUPPORT: sasdje        
 *
 *  PARAMETERS
 *
 * fParmData =  the name of the internal frequency distribution data set. 
 *              The data set has one row for each cell containing the FPARM data + &CCKEY.
 *
 * freqKRIs  = A dataset with the set values for the frequency KRI variables
 *
 * COVDATA  = The covariance matrix
 *
 * NREPS    =  the number of simulation replications,must be a positive integer.   
 *
 * SEEDDATA =  dataset of seeds for each cell CCUID.
 *
 * FITINTERVAL = The interval use for the frequency fit.
 *
 * SIMINTERVAL = The simulation interval
 *
 * OUTSTATUS = Success or failure - Default is outstatus 
 *
 * OUTDRAWS =  Matrix of simulated frequencies               
 *                                                      
 * OUTKRIWEIGHTS =  Dataset containing ccUID, BL_UID, and the proportions
 *                  of losses of each BL_UID by CCUID, call it
 *
 * copulaFamily   = NORMAL, NORMALMIX, or T
 *
 * copulaParams   = the parameters going along the copula specification.
 *                  See documentation for orva_aggr_corr().
 *                                                     
 * correlation_is_id = true/false- whether or not the correlation matrix is the  
 *                    identity matrix- used to speed processing in freqsim_stream
 *
 * copula_type_agg  = true/false- whether or not the correlation is at the aggregate
 *                    level
 *
 * PARALLEL = 1: use parallel processing
 *            0: don't use parallel processing
 *     
 * NSTREAMS = number of parallel streams
 *     
 ****************************************************************/
 %macro ORVA_freqsim(fParmData         = ,  
                     covData           = ,
                     nreps             = 100,
                     seedData          = ,
                     fitInterval       = YEAR,
                     simInterval       = YEAR,
                     outStatus         = outstatus,
                     outDraws          = outdraws,
                     copulaFamily      = NORMAL,
                     copulaParams      = ,
                     correlation_is_id = false, 
                     copula_type_agg   = false,
                     parallel          = 0,
                     nstreams          = 1,
                     proxyScenVersion  = 42  ,
                     debug             = false
                    );
 
    /*- create the status data set upfront -*/
    %let reason= %sysfunc(sasmsg(&orvads, unknown_error, noquote));
    %orva_status_ds( statusds=&outstatus, status=1, reason=&reason );
    %let status=0;
    %let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote));
    %let abort=0;

    /*- error checking -*/
    %if &fParmData=%str() %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fsim_specify_fparmdata_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, fsim_fParmData_missing_error, noquote));
        %goto DONE;
    %end;
    /*- seedData must be specified -*/
    %if "&seedData" EQ "" %then %do;
      %put &orvar_error %sysfunc(sasmsg(&orvads, fsim_provide_seedData_error, noquote));
      %let abort=1;
      %let reason=%sysfunc(sasmsg(&orvads, fsim_seedData_missing_error, noquote));
      %goto DONE;
   %end;
    
    /*- Adjust the scale to the current interval and check mean -*/
    %let max_freq_mean = 1.0e5;
    %let status = 0;
    %let ScenarioRatio = 1;
    %let ModelRatio = 1;
    data _NULL_;
        ScenarioRatio = intck("YEAR", 0, 366) / intck("&SimInterval", 0, 366);
        ModelRatio = intck("&fitInterval", 0, 366) / intck("&SimInterval", 0, 366);
        call symput('ScenarioRatio', put(ScenarioRatio, BEST20.));
        call symput('ModelRatio', put(ModelRatio, BEST20.));
    run;

    data 
         _fixed_intfparms 
         _fs_failed; 
         set &fParmData;

        /* adjust for interval */
         if Source = "S" then do;
            if &ScenarioRatio NE 1 then do;
               call ChangeInterval( distribution, &ScenarioRatio, &theFreqSimCallParams );
            end;
         end;
         else do;
            if &ModelRatio NE 1 then do;
               call ChangeInterval( distribution, &ModelRatio, &theFreqSimCallParams );
            end;
         end;
        /* check mean */
        call FreqMoments(distribution, &theFreqSimCallParams, m1, m2 );  
        if  m1 > &max_freq_mean then do;
            call symputX("status", 2);
            output _fs_failed;
        end;
        else do;
            output _fixed_intfparms;
        end;

        drop m1 m2; 
    run;

   %if &status NE 0 %then %do;
      PROC SQL noprint;
         select CCUID into :Failed_CCUID_LIST separated by " "
         from _fs_failed;
      QUIT;
      %let reason = %sysfunc(sasmsg(&orvads, fsim_cells_have_big_freq_means_error, noquote, &Failed_CCUID_List));
      %put &reason;
   %end;

    %if &debug eq true %then %do;
       data _NULL_;
          now = datetime();
          call symput('_TimeStart_',now);
       run;
    %end;

    /********************** loop over REPS *****************************/
    %let ORVP_STATUS= 1;
    %if ("&parallel" eq "1") %then %do;
        /************ handle with parallel processing ***************/
        %orvp_distribute_freqsim(parmData        = _fixed_intfparms,  
                                 covData         = &covData,                                
                                 seedData        = &seedData,
                                 fitInterval     = &fitInterval,
                                 simInterval     = &simInterval,
                                 outDraws        = &outDraws,
                                 copulaFamily    = &copulaFamily,
                                 copulaParams    = &copulaParams
                                 );
        %if (&ORVP_STATUS ne 0) %then %do;
            %put &orvar_warning %sysfunc(sasmsg(&orvads, fsim_no_procs_for_paralstim_warning, noquote));
        %end;
    %end;
    /*********** else handle directly **************************/
    %if (&ORVP_STATUS ^= 0) %then %do;
        data _NULL_;
           reps = ceil(&nreps/&nstreams);
           call symputX('REPS', reps);
        run;
        %let NITER= 1;
        %let OFFSETREP= 0;
        %let NREPSLEFT= &nreps;
        %do %while (&NREPSLEFT gt 0);

           %let SEED = 0;
           data _NULL_; 
             set &seedData; 
             where CCUID eq 1 and HOST eq &NITER;
             length tseed $ 32;
             tseed = put(seed,32.);
             call symput("SEED", trim(left(tseed)));
           run;

           %orva_freqsim_stream(parmData          = _fixed_intfparms,  
                                covData           = &covData,
                                nreps             = &nreps,
                                offsetRep         = &OFFSETREP,
                                seed              = &SEED,
                                fitInterval       = &fitInterval,
                                simInterval       = &simInterval,
                                outDraws          = outdraws_&NITER,
                                copulaFamily      = &copulaFamily,
                                copulaParams      = &copulaParams,
                                     correlation_is_id = &correlation_is_id, 
                                     copula_type_agg   = &copula_type_agg    
                               );

           data &outDraws;
                set %if &NITER ne 1 %then %do; &outDraws %end;
                    outdraws_&NITER;
           run;
           
           proc datasets library=work memtype=data nolist nodetails;
              delete outdraws_&NITER;
           quit;
            
           %let OFFSETREP= %eval(&OFFSETREP + &REPS);
           %let NREPSLEFT= %eval(&NREPSLEFT - &REPS);
           %if &NREPSLEFT lt &REPS %then %do;
               %let REPS = &NREPSLEFT;
           %end;
           %let NITER=%eval(&NITER+1);
        %end;
       
    %end;

    %orv_makeMissingMaxPlusOne( data =&outDraws );
    /************** End of the REP loop ***************************************/
    
    %if &debug eq true %then %do;
       data _NULL_;
          now = datetime();
          elapsed = now-symget('_TimeStart_');
          call symput('_TimeElapsed_',elapsed);
       run;
       %put %sysfunc(sasmsg(&orvads, fsim_time_note, noquote, &_TimeElapsed_)) ;
    %end;
    
%DONE:
    %if &abort %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, fsim_abnormal_error, noquote));
        %let status=1;
    %end;  
    %else %do;
      %let Use_42_Scen_Calc = NO;
      
      %let priorityDataSet =;
      %ORVU_find_dataset( namein = Priority&proxyScenVersion, libs = VARPRJ VARPRJ_M, nameout=priorityDataSet);
      
      %if &priorityDataSet ne %str() %then %do;
         %_fsim_epilog&proxyScenVersion(PriorityData = &priorityDataSet,
                                     scenReps     = &nreps
                                     );
      %end;
    %end;

    %orva_status_ds( statusds=&outstatus, status=&status, reason=&reason );
    
    proc datasets library=work memtype=data nolist;
        delete _tmp _fs_failed;
    quit;
%mend ORVA_freqsim;

%macro _fsim_epilog(PriorityData= , scenReps= );
   %put Proxy scenario simulation will be done using VaR 4.1 algorithm.;
%mend _fsim_epilog;

/* This is added for dealing with proxy scenarios */ 
%macro _fsim_epilog42(priorityData= , scenReps= );
   %put Proxy scenario simulation will be done using VaR 4.2 algorithm.;

   data _NULL_;
      set &priorityData;
      call symput("SimPriorityData",SimPriorityData);
   run;
   /*once the prject is saved, the value for SimPriorityData may be outdated. eg, when it has the value of varprj.simpriority_Sev etc */
   %let simPriorityDataSetName=;   
   %_fsim_priorityName_fix(in=&simPriorityData, out=simPriorityDataSetName);
   %let priorityDataSetWithLib =;
   %ORVU_find_dataset( namein = &simPriorityDataSetName, libs = VARPRJ VARPRJ_M, nameout=priorityDataSetWithLib);

   %local Scenario_Count;
   %let Scenario_Count = 0;

   /* Check the number of different sources for severity simulation*/
   proc sql noprint;
      SELECT
            PUT(count(DISTINCT UPPER_BOUND), 2.-L) 
      INTO
            :UpperBound_Count
      FROM
            &priorityDataSetWithLib
      WHERE 
            Source_Type NE "R"
      ;

      SELECT
               PUT(count(DISTINCT dataset_name), 2.-L) 
      INTO
               :Scenario_Count
      FROM
               &priorityDataSetWithLib
      WHERE 
               Source_Type EQ "S"
      ;
   quit;

   /*************************************************************/
   /* Only one UPPER_BOUND, effectively no mixing in simulation */
   /*************************************************************/
   %if &UpperBound_Count = 1 %then %do;
      %if %sysevalf(&Scenario_Count > 0, BOOLEAN) %then %do;
         %let Use_42_Scen_Calc = YES;
         %put There is at least one proxy scenario source available for simulation.;
         %put Each will be simulated without insurance calculation.;
         %_fsim_run_42_scen_simulation();
      %end;
   %end;
   %else %do;
      %put There are multiple sources being combined for simulation.;
      %put The simulation will proceed in the usual way.;
   %end;
%mend _fsim_epilog42;

%macro _fsim_run_42_scen_simulation();
   %local The_Source_Type;
   %local The_Dataset_Name;
   %local The_Scenario_Type;

   proc sql noprint;
      SELECT
               put(count(DISTINCT cat(Dataset_Name, " ", Scenario_Type)),2.-L)
      INTO
               :Dataset_Scenario_Count
      FROM
               &SimPriorityData
      WHERE 
               Source_Type EQ "S"
      ;

      SELECT
               DISTINCT cat(Dataset_Name, " ", Scenario_Type)
      INTO
               :Dataset_Scenario_1-:Dataset_Scenario_&Dataset_Scenario_Count
      FROM
               &SimPriorityData
      WHERE 
               Source_Type EQ "S"
      ;
   quit;

   %do i = 1 %to &Dataset_Scenario_Count;
      %let The_Dataset_Name  = %scan(&&Dataset_Scenario_&i, 1, ' ');
      %let The_Scenario_Type = %scan(&&Dataset_Scenario_&i, 2, ' ');

      proc sql noprint;
         SELECT 
                  CCUID
         INTO
                  :TheCCUIDs separated by ","
         FROM
                  &SimPriorityData
         WHERE
                  Dataset_Name = "&The_Dataset_Name"
         ;
      quit;

      %_fsim_do_sim_for_one_sim( scenIndex   = &i,
                                 scenType    = &The_Scenario_Type,
                                 datasetName = &The_Dataset_Name,
                                 outData     = VARPRJ.Proxy_Sim
                                 );
   %end;
%mend _fsim_run_42_scen_simulation;

%macro _fsim_do_sim_for_one_sim(scenIndex =, scenType =, datasetName =, outData =);
   %if %upcase(&scenType) eq B %then %do;
      %let plotLoc = %index(&datasetName,_plot_);

      %if &plotLoc > 0 %then %do;
         %let datasetName = %substr(&datasetName,1,&plotLoc+4)brkdwn%substr(&datasetName,&plotLoc+5);
      %end;
   %end;

   %if &scenIndex = 1 %then %do;         
      %let The_Data_Out = &outData;
   %end;
   %else %do;
      %let The_Data_Out = tmp_Proxy_Sim;
   %end;

   %orva_sim_scenario_certain(simCount = &scenReps, 
                              dataIn   = &datasetName,
                              scenSpec = &scenType, 
                              seed     = &seed, 
                              scenKind = PROXY,
                              dataOut  = &The_Data_Out
                              ); 

   /* Keep only the cells using this scenario */
   data &The_Data_Out;
      set &The_Data_Out;
      where ccuid in (&TheCCUIDs);
   run;

   %if &scenIndex > 1 %then %do;
      data  &outData;
         set &outData &The_Data_Out;
      run;

      proc datasets library=work memtype=data nolist;
         delete &The_Data_Out;
      quit;
   %end;
%mend _fsim_do_sim_for_one_sim;

/*remove the varprj or varprj_m lib name for priorityDataSetName*/
%macro _fsim_priorityName_fix(in=priorityData, out=priorityDataSetName);
   %let libName = %substr(&in, 1, 8);
   %if "%upcase(&libName)" eq "VARPRJ_M" %then %do;
      %let dsName = %substr(&in, 10);
   %end;
   %else %do;
      %let &out = %substr(&in, 8);
   %end;
%mend _fsim_priorityName_fix;

%macro orv_makeMissingMaxPlusOne( data = );
   proc contents 
                  data = &data 
                  out  = dataContents
                  noprint
                  ;
   quit;

   /* Create macros <varNames> and <varMaxNames> that contain */
   /* space-delimited lists of the variable names as well     */
   /* as the names for the maximum of each variable           */
   proc sql noprint;
      SELECT
               name
      INTO
               :varNames SEPARATED BY " "
      FROM
               dataContents
      WHERE
               substr(name, 1, 5) = "Cell_"
      ORDER BY
               name
               ;

      SELECT
               cat(trim(name),"_Max")
      INTO
               :varMaxNames SEPARATED BY " "
      FROM
               dataContents
      WHERE
               substr(name, 1, 5) = "Cell_"
      ORDER BY
               name
               ;
      
   quit;

   /* Obtain the maximum per column. If all missing, */
   /* then the maximum is also missing.              */
   proc summary data = &data;
      var &varNames;
      output
            out = maxData
            max = &varMaxNames;
   run;

   data maxdata; set maxdata(drop=_freq_ _type_); index=1;run;
   data tmp2; set &data; index=1; run;
   data tmp; merge maxData tmp2; by index; run;
   
   
                     
   data &data;
      set tmp;
      array vars &varNames;
      array maxs &varMaxNames;
      retain aSize 0;

      if _N_ = 1 then do;
         aSize = dim(vars);
      end;

      do i = 1 to aSize;
         if missing(vars[i]) then do;
            Vars[i] = maxs[i] + 1;
         end;
      end;

      drop 
            &varMaxNames
            aSize
            i
            ;
   run;


   proc datasets 
                  library = work 
                  memtype = data 
                  nolist 
                  nodetails
                  ;
      delete 
            dataContents
            maxdata
            tmp 
            ;
   quit;  
%mend orv_makeMissingMaxPlusOne;
