/******************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    orva_get_parm_kris                                             
 * PURPOSE: Get the unique list of KRI variables from a parameter dataset by 
 *          business-line.  Used to determine which kris are 
 *          actually used in simulation!
 *          
 *          We have do this by merging parm dataset in with the loss data set for SEVERITY
 *          and frequency models data set for FREQUENCY.
 *          We cannot use the businessline dataset because businessline may
 *          not be part of our CCVs! 
 *          
 *          
 * SUPPORT: stchri
 *
 * PARAMETERS:
 *
 *   parmData   = the selected parameter dataset (from fitting).
 *   freqModels = the frequency models dataset (for FREQUENCY only).
 *   outData    = output dataset with with NAME column containing kri names.
 *   type       = SEVERITY or FREQUENCY.
 *   ccData     = Cross-Classification dataset for project.
 *   lossData   = loss dataset for project.
 *   ctoolData  = is this ctool data?
 *   cclibref   = NOT YET USED!!!!!!!!
 *   
 * NOTE: see sample code at bottom of file
 *
 *****************************************************************************************/
%macro orva_get_parm_kris(
                           parmData   = , 
                           freqModels = ,
                           outData    = , 
                           type       = SEVERITY, 
                           ccData     = ,
                           lossData   = ,
                           ctoolData  = false,
                           cclibref   =
                           );

   %let BL_COL_NAME = INTERNAL_BL_UID;

   /*** create empty output dataset up front ***/ 
   proc sql;
     CREATE TABLE 
                  &outData(
                           NAME       varchar(32),
                           BLUID_KRI  num 
                           ); 
   quit;
   
   /*** check parmData ***/
   %let NOBS = 0;
   %let dsid = %sysfunc(open(&parmData, I));

   %if &dsid ^= 0 %then %do;
      %let NOBS = %sysfunc(attrn(&dsid, nlobs));
      %let rc   = %sysfunc(close(&dsid));
   %end;

   %if &NOBS eq 0 %then %do;
      %return;
   %end;

   /***** get number of Cross Classification variables *****/
   proc contents 
                noprint 
                data = &ccData 
                out  = _cc; 
   run;

   %let dsid = %sysfunc(open(_cc, I));

   %if &dsid ^= 0 %then %do;
      %let NUMCCVS = %sysfunc(attrn(&dsid, nlobs));
      %let rc      = %sysfunc(close(&dsid));
   %end;

   %if &NUMCCVS eq 0 %then %return;
   %let NUMCCVS = %eval(&numccvs - 1);

   /***************************************************************
   * Transpose parm dataset and keep only KRI variables and CCUID 
   *    _NAME_ will have the KRI name and 
   *    COL1..COLn will have KRI values in it 
   *
   * When done we have a dataset with 2 columns:
   *    NAME - KRI variable name
   *    CCUID - cell it's used in
   ************************************************************-***/
   proc transpose 
                  data = &parmData 
                  out  = _trans; 
   run;

   proc sort 
            data = _trans; 
      by _NAME_; 
   run;

   data _trans; 
      set _trans;
      length NAME $ 32;
      array COL[&NOBS] COL:;  /* store values */
      array CC[&NOBS];        /* store CCUIDs */
      retain CC: ;

      /* store CCUIDs in retained variables */
      if upcase(substr(_NAME_, 1, 5)) eq 'CCUID' then do;
         do i = 1 to &NOBS;
            CC[i] = COL[i];
         end;
      end;

      %if &type eq SEVERITY %then %do;
         if length(_NAME_) > 12 then do;
            if upcase(substr(_NAME_, 1, 12)) eq 'PARAM_1_KRI_' then do;
               NAME = trim(substr(_NAME_, 9));

               do i = 1 to &NOBS;
                  if COL[i] ne . then do;
                     CCUID = CC[i];
                     output;
                  end;
               end;
            end;
         end;
      %end;
      %else %do;
         if length(_NAME_) > 4 then do;
            if upcase(substr(_NAME_, 1, 4)) eq 'KRI_' and upcase(_NAME_) ne 'KRI_INTERCEPT' then do;
               NAME = _NAME_;

               do i = 1 to &NOBS;
                  if COL[i] ne . then do;
                     CCUID = CC[i];
                     output;
                  end;
               end;
            end;
         end;
      %end;

      keep NAME CCUID;
   run;

   %let nKris = 0;
   %orvu_getObsCount(_trans, nKris);

   %if &nKris eq 0 %then %do;
      %goto ALL_DONE;;
   %end;

   %let DS_EXIST_STATUS = 0;

   %if &type eq SEVERITY %then %do;
      %get_unique_bls_from_losses(
                                    lossData    = &lossData, 
                                    ccData      = &ccData,
                                    BL_COL_NAME = &BL_COL_NAME, 
                                    NUMCCVS     = &NUMCCVS, 
                                    outData     = _losses
                                    );
   %end;
   %else %do;
      %get_unique_bls_from_models(
                                    modelsData  = &freqModels, 
                                    ccData      = &ccData, 
                                    BL_COL_NAME = &BL_COL_NAME, 
                                    NUMCCVS     = &NUMCCVS, 
                                    outData     = _losses
                                    );
   %end;
   
   %if &DS_EXIST_STATUS NE 0 %then %do;
      %put &DS_EXIST_STATUS.;
      %GOTO ALL_DONE;
   %end;
   
   /******************************************************************/
   /*** merge subsetted PARM dataset with list of  
        significant kris by cell (CCUID)                           ***/
   /******************************************************************/
   proc sql noprint; 
      CREATE TABLE 
                     _merge 
      AS SELECT 
                     * 
      FROM
                     _losses l 
      LEFT JOIN 
                     _trans t 
         ON 
                     l.CCUID eq t.CCUID
                     ;
    quit;

   /*** drop losses that do not match our significant KRI BLs ***/
   data _merge; 
      set _merge (rename = (&BL_COL_NAME = BLUID_KRI)); 
       where NAME ne ""; 
   run;

   /*** remove dups ***/
   data &outData; 
      set _merge(keep =  BLUID_KRI NAME); 
   run;

   proc sort 
            data = &outData 
            nodups; 
      by 
         BLUID_KRI 
         NAME; 
   run;
  
%ALL_DONE:
   /*** cleanup temporary datasets  ***/
   %orvu_delete_data( data =
                              _trans 
                              _losses 
                              _merge 
                              _cc 
                              _mtotals
                              );
%mend orva_get_parm_kris;

%macro get_unique_bls_from_models(
                                   modelsData  =, 
                                   ccData      =, 
                                   BL_COL_NAME =, 
                                   NUMCCVS     =, 
                                   outData     =
                                   );

   %let dsid = %sysfunc(open(&modelsData., I));

   %if &dsid = 0 %then %do;
      %let DS_EXIST_STATUS = The dataset &modelsData. is empty.;
      %goto DONE_LOCAL;
   %end;

   data &outData; 
      set &modelsData;
      keep 
          &BL_COL_NAME 
          CCUID;
  run;
  
   proc sort 
            data = &outData 
            nodups; 
      by 
         CCUID 
         &BL_COL_NAME; 
  run;
  
   data &outData; 
      set &outData;

     if _N_ eq 1 then do;
        declare hash ccHash(dataset: "&ccData");
        rc = ccHash.defineKey('CCUID');

        %do i = 0 %to &numCCVs-1; 
           rc = ccHash.defineData("ccvuid_&i");
        %end;

        rc = ccHash.defineDone(); 
   
        %do i = 0 %to &numCCVs - 1; 
          call missing(ccvuid_&i);
        %end;
     end;

     rc  = ccHash.find();
     drop rc;
  run;
  
%DONE_LOCAL:
%mend get_unique_bls_from_models;

%macro get_unique_bls_from_losses(
                                  lossData    = , 
                                  ccData      = , 
                                  BL_COL_NAME = , 
                                  NUMCCVS     = , 
                                  outData     =
                                  );
      /******************************************************************/
      /*** subset loss dataset to get one entry per CCUID and BL      ***/
      /******************************************************************/
   data _losses; 
      set &lossData (keep = 
                           CCUID 
                           CCVUID_: 
                           &BL_COL_NAME
                           );
   run;

   proc sort 
            data =_losses 
            nodups; 
      by 
         CCUID 
         &BL_COL_NAME; 
   run;
      
   /******************************************************************/
   /*** add back in total cells                                    ***/
   /******************************************************************/
   /*** create totals dataset ***/
   data _mtotals; 
      set &ccData end = last;     
      where 
            0 
            %do i = 0 %to &NUMCCVS - 1; 
               OR CCVUID_&i eq 0 
            %end;
            ;
      rename 
            CCUID = T_CCUID  
            %do i = 0 %to &NUMCCVS - 1; 
               CCVUID_&i =T_CCVUID_&i
            %end;
            ;
      run;
      
   /*** merge totals into _losses to get businesslines ***/
   proc sql noprint; 
      CREATE TABLE 
                     _merge 
      AS SELECT 
                     * 
      FROM 
                     _losses l 
      LEFT JOIN 
                     _mtotals t 
      ON 
                     1 
                     %do i = 0 %to &NUMCCVS - 1; 
                        AND 
                           (t.T_CCVUID_&i = 0 OR t.T_CCVUID_&i = l.CCVUID_&i)
                     %end;
            ;
   quit;
      
   /*** reduce columns, sort, and throw out duplicates ***/
   data _merge; 
      set _merge(keep = 
                        &BL_COL_NAME 
                        T_CCUID
                        ); 
   run;

   proc sort data = _merge nodups; 
      by 
         T_CCUID 
         &BL_COL_NAME; 
   run;

   data _mtotals; 
      set _merge (rename = T_CCUID = CCUID); 
   run;
      
   /*** stack totals on to end of _losses */
   data _merge; 
      set _losses (keep = 
                          &BL_COL_NAME 
                          CCUID
                          ) 
          _mtotals; 
   run;
      
   /*** put CCVUID columns back ****/
   proc sql noprint; 
      CREATE TABLE 
                     _losses 
      AS SELECT 
                     * 
      FROM 
                     _merge m 
      LEFT JOIN 
                     &ccData cc 
      ON 
                     m.CCUID = cc.CCUID
                     ; 
   quit;
%mend get_unique_bls_from_losses;
