/*****************************************************************************************
 * $Id: orva_getfreqmomts.sas,v 1.1.2.11.2.21.2.5.8.1 2012/05/03 15:14:10 sasdje Exp $
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:     orva_getfreqmomts.sas                                            
 * PURPOSE:  Creates count dataset(s) and calculates empirical frequency moments
 *           Called by orva_getmomts() macro.
 *
 * SUPPORT: lajack
 *
 * PARAMETERS:
 *
 *   lossData      =  the name of the loss data with the following variables:
 *                    AMOUNT, FINANCIALDATE CCUID.
 *   dataType      =  I, C, E
 *   global_range  =  Global range dataset for this project.
 *   interval      =  DATE interval (YEAR, MONTH ...) Default is YEAR
 *   missing_range =  name of the missing ranges dataset with the following variables:
 *                    &CCV_NAMES, START_MISSING_DATE, END_MISSING_DATE.
 *   outStatus     =  Output dataset with Success or failure
 *   outFreqMomts  =  Output dataset with empirical frequency moments
 *   numCCVs       =  Number of cross-classification variables in project.
 *   CCV_NAMES     =  space-delimited list of cross-classification variables in project.
 *   outprops      =  out dataset for panel properties.
 *   outmissprops  =  out dataset for properties which have defined any missing ranges 
 *                    in the user-loaded missing range table.  
 *   outmissrange  =  updated missing range table.
 *   scaleData     =  scaling dataset (currently only for internal data)
 *****************************************************************************************/
 %macro orva_getfreqmomts( lossData   =,
                           dataType       =I,
                           cTool          =false,
                           global_range   =,
                           interval       =YEAR,
                           missing_range  =,
						   outFreqMomts   =,              
                           outStatus      =ostatus,
                           NUMCCVS        =2,
                           CCV_NAMES      =INTERNAL_BL_UID INTERNAL_RC_UID,
                           outprops       =,
                           outmissprops   =,
                           outmissrange   =,
                           outCountData   =,
                           scaleData      =
                         );  


    /*- create the status data set up front -*/
    %let reason=%sysfunc(sasmsg(&orvads, unknown_error, noquote));
    %orva_status_ds( statusds=&outStatus, status=1, reason=&reason );

    %let status=0;
    %let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote));
    %let abort=0;

    /*- error checking -*/
    %if &global_range=%str() %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, gfmom_specify_global_range_ds_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, gfmom_specify_global_range_ds_error, noquote));
        %goto DONE;
    %end;
    data _null_; set &global_range; 
        call symput( "range", end_date-start_date+1);
        call symput( "global_end", end_date);
        call symput( "global_start", start_date);
    run;
    %if %eval(&range) LE 0 %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, gfmom_start_not_before_end_date_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, gfmom_start_not_before_end_date_error, noquote));
        %goto DONE;
    %end;

    
%let MISS_PROPS_I = internal_bl_uid internal_rc_uid legal_entity_uid process_uid class_1 class_2 class_3 class_4 class_5 class_6 class_7 class_8 class_9 class_10;    
    
/****** FIRST:  Handle scaling. ******/
	%let nobs_scale=0;
	%if &scaleData NE %str() %then %do;
		%let dsid=%sysfunc(open(&scaleData,I));                                                                                        
		%if &dsid NE 0 %then %do;                                                                                                      
			%let nobs_scale=%sysfunc(attrn(&dsid,nlobs));                                                                                     
			%let rc=%sysfunc(close(&dsid));                                                                                             
		%end;      
	%end; 

/****** SECOND:  Find any properties for which there is a missing range defined. ******/
    
   %let nmissingranges = 0;
   %let nmissprops = 0;
   %if &dataType EQ I %then %do;
       %if &missing_range ne %str() %then %do;
          %let dsid=%sysfunc(open(&missing_range,I));                                                                                        
          %if &dsid NE 0 %then %do;                                                                                                      
             %let nmissingranges=%sysfunc(attrn(&dsid,nlobs));                                                                                     
             %let rc=%sysfunc(close(&dsid));                                                                                             
          %end;    
       %end;
   %end;    
   %if &nmissingranges GT 0 %then %do;
       data _mom_miss1(keep= &&MISS_PROPS_&dataType %if &cTool eq true %then %do; OWN %end;) ;
          set &missing_range. ;
             %if &cTool eq true %then %do;
                %orva_assignown();
             %end;
       run;
       %findprops(indataset=_mom_miss1, propdataset=_mom_misspropds);
       %if &nmissprops ne 0 %then %do;
           %let MISSPROPS = &CCV_NAMES;
           %let nmissprops = &NUMCCVS;
           proc SQL noprint;
              select TABLEVAR into :MISSPROP1 - :MISSPROP&nmissprops
              from _mom_misspropds;    
           quit;
           %let lastmissprop=&&MISSPROP&nmissprops;
          %end;
   %end;

	%let MISSPROPS = %str(); /* initializing */
	%let foundibl=0;
	/* if internal scaling present, check whether internal_bl_uid is in missprops */
	%do i=1 %to &nmissprops;
		%let MISSPROPS = &MISSPROPS &&MISSPROP&i;
		%if %upcase(&&MISSPROP&i) EQ INTERNAL_BL_UID %then %do;
			%let foundibl=1;
		%end;  
	%end;

/************************************************************/
/* PANELPROPS should have "OWN" if it is in MISSPROPS      */
/* PANELPROPS are {all the MISSPROPS} union {&CCVUID_x if its mapped property isn't there}   */
/************************************************************/

   /***** create PANELPROPS from CCV_NAMES and remove __TOTALS__ONLY__ *****/
/* Create MISSPROPSPLUS, which would contain MISSPROPS and INTERNal_bl_uid (if Internal scaling component is present, and foundibl=0.) 
Then let panelprops = extraprops and misspropsplus */
	%let MISSPROPSPLUS = &MISSPROPS;
	%if &nobs_scale ne 0 AND &dataType EQ I AND &foundibl EQ 0 %then %do; /* Internal scaling component is present, AND ibl is not in MISSPROPS */
		%let MISSPROPSPLUS = &MISSPROPSPLUS INTERNAL_BL_UID;
	%end;

    /* Add CCVUID_x to MISSPROPSPLUS if its mapped property isn't there */
    %let EXTRAPROPS = ;       /* initializing */
    %let nextraprops = 0;  /* initializing */
    %do j=0 %to %eval(&NUMCCVS-1);
        %let nCCV = %eval(&j+1);
        %let thisCCV = %scan(&CCV_NAMES, &nCCV, ' ');
        %if  &thisCCV ne __TOTALS__ONLY__ %then %do;
            %let nmisspropsplus=1;
            %let thisprop = %scan(&MISSPROPSPLUS, &nmisspropsplus, ' ');
            %let found=0;
            %do %while (&thisprop NE );                         
                %if %upcase(&thisCCV) EQ %upcase(&thisprop) %then %do;
                    %let found=1;
                %end;                        
                %let nmisspropsplus = %eval(&nmisspropsplus + 1);
                %let thisprop = %scan(&MISSPROPSPLUS, &nmisspropsplus, ' ');
            %end;
            %if &found EQ 0 %then %do;
                %let EXTRAPROPS = &EXTRAPROPS CCVUID_&j ;
                %let nextraprops = %eval(&nextraprops + 1);
            %end;
            %let nmisspropsplus = %eval(&nmisspropsplus - 1);
        %end;
    %end;

    %let PANELPROPS = &EXTRAPROPS &MISSPROPSPLUS ; 
    %let npanelprops = %eval(&nmisspropsplus + &nextraprops);

    %let lastpanelprop = %scan(&PANELPROPS, &npanelprops, ' ');
    
    %let PANEL_PROPS_COMMA =  %scan(&PANELPROPS, 1, ' ');
    %do i=2 %to &npanelprops;
        %let PANEL_PROPS_COMMA = &PANEL_PROPS_COMMA , %scan(&PANELPROPS, &i, ' ')  ;
    %end;

   /*----- create outprops dataset, if requested -----*/
   /*----- some of the code above may not be neeed either... -----*/
    %if &outprops NE %str() %then %do;
        data &outprops;
            length THEPROPS $ 1024;
            THEPROPS = "&PANELPROPS";
        run;
    %end;
    

   /* In preparation of matching the missing ranges to the correct panels, 
      make flags, one per &&MISSPROP&i, to indicate which properties define missing ranges.
      NOTE:  There could be some missing ranges 
    */
    %let flagown=0;
    %let flaginternal_bl_uid=0; %let flaginternal_rc_uid=0; %let flaglegal_entity_uid=0;
    %let flagprocess_uid=0; %let flagclass_1=0; %let flagclass_2=0; %let flagclass_3=0;
    %let flagclass_4=0; %let flagclass_5=0; %let flagclass_6=0; %let flagclass_7=0;
    %let flagclass_8=0; %let flagclass_9=0; %let flagclass_10=0;
    %do i=1 %to &nmissprops;
        %let flag&&MISSPROP&i = 1;
    %end;


/****** THIRD: Break up missing ranges so they don't straddle fit-interval boundaries  ******/
/*(a)  Output some things having to do with Missing Range table, to be re-used at Freq Fit time.*/
/*    Namely:  The MISSPROPS.   The expanded Missing Range table (per fit-interval). */

    %if &nmissingranges GT 0 %then %do;
        data _mom_miss2(drop=START_MISSING_DATE END_MISSING_DATE 
                MISSING_RANGE_UID FEED_REF_ID DATETIME_LOADED VERSION_UID );
            set &Missing_Range;
            format interval_start interval_end date.;
            interval_start = start_missing_date;
            interval_end = end_missing_date;
            %if &cTool eq true %then %do; 
                %orva_assignown();
            %end;
        run;
        %orva_interval_datasets(Withsort            = _mom_miss2,
                            intervalout            = _mom_miss3,
                            interval            = &interval);
         /*  rename the props in the missing table, so that the PROC SQL statement will work, 
             in "SEVENTH" step below. */
        data _mom_miss4;
            set _mom_miss3(rename=(interval_start=START_MISSING_DATE
                interval_end=END_MISSING_DATE
                interval_length=missing_length
                %if &cTool eq true %then %do; own = miss_own %end;
                internal_bl_uid = miss_internal_bl_uid
                internal_rc_uid = miss_internal_rc_uid
                legal_entity_uid = miss_legal_entity_uid
                process_uid = miss_process_uid
                class_1 = miss_class_1 class_2 = miss_class_2
                class_3 = miss_class_3 class_4 = miss_class_4
                class_5 = miss_class_5 class_6 = miss_class_6
                class_7 = miss_class_7 class_8 = miss_class_8
                class_9 = miss_class_9 class_10 = miss_class_10
            ));
            format missing_tag date.;
            /* the first day of the &interval in which this obs falls */
            missing_tag = max(&global_start, intnx("&interval", start_missing_date, 0)); 
            missing_exposure = (end_missing_date - start_missing_date +1) / 
                               (intnx("&interval", start_missing_date, 1)-intnx("&interval", start_missing_date, 0)); 
            /* don't keep CCUID, because I'm keeping it in the count dataset, and the names will otherwise clash
            in the PROC SQL statement, if I were to keep it in _mom_miss4.  */
            keep missing_tag missing_exposure start_missing_date end_missing_date missing_length 
            miss_:
            ;
        run;

        /*----- create outmissprops dataset, if requested -----*/
        /*----- THIS IS NEEDED FOR ORVA_FREQ_MAKE_COUNTS -----*/
        %if &outmissprops NE %str() %then %do;
          data &outmissprops;
              format missprop $100.;
              %do i=1 %to &nmissprops;
                  missprop = "&&MISSPROP&i";
                  output;
              %end;    
          run;
        %end;
    
        %if &outmissrange NE %str() %then %do;
          data &outmissrange;
              set _mom_miss4(drop=missing_exposure missing_tag);
          run;
        %end;

    %end;

   /* check for nothing left to do */
   %if &outFreqMomts EQ %str() %then %do;
      %goto DONE;
   %end;

   /* error checking */
   %if &lossData=%str() %then %do;
        %put &orvar_error  %sysfunc(sasmsg(&orvads, gfmom_specify_lossdata_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, gfmom_missing_lossData_error, noquote));
        %goto DONE;
    %end;
/****** FOURTH:  Set up the loss_work dataset, keeping only the columns that we need.  ******/
/* Note:  it doesn't cause an error/warning to have the same variable listed multiple times on a keep statement. */

    data _loss_work;
        set &lossData; 
        format interval_start date9.;
        keep interval_start count first_date_of_event CCUID &PANELPROPS ;
        interval_start = max(intnx("&interval", first_date_of_event, 0), &global_start);
        count=1;
    run;
    

/* FIFTH:  Create count dataset.  (Keep CCUID : Just slip it in ....*/

    proc sql;
        create table _gfm_countsOne1
        as select distinct  CCUID,  &PANEL_PROPS_COMMA , interval_start, 
        sum(count) as the_count
        from _loss_work
        group by CCUID,  &PANEL_PROPS_COMMA, interval_start;
    quit;
    proc sort data=_gfm_countsOne1 out=_gfm_countsOne1;
        by &PANELPROPS    interval_start ;
    run;





/*****  SIXTH:  create expanded count dataset with 0s added in for every possible panel
     Even those panels that never ever have a loss  *****/

    /* Add 0s for missing intervals before first observation w/in each by-group, up to global_start */
    data _gfm_countsOne2(drop = n i);
        set _gfm_countsOne1 end=last;
        by     &PANELPROPS    interval_start ;

        if first.&lastpanelprop then do;
            if interval_start NE &global_start then do;
                /* output current one (will need to re-sort later */
                output;    
                /* figure out how many intervals (in addition to the one starting with &global_start) are missing */
                n = intck("&interval", &global_start, interval_start-1);
                interval_start = &global_start; the_count = 0; output;
                do i=1 to n;
                    interval_start = intnx("&interval", interval_start, 1);
                    output;
                end;

            end;    
            else do;
                output;
            end;            
        end;
        else do;  
            output;  
        end;
    run;
    proc sort data=_gfm_countsOne2 out=_gfm_countsOne2sort;
        by     &PANELPROPS    interval_start ;
    run;

    data _null_;
        TotIntervalsRemaining = intck("&interval", &global_start, &global_end);
        call symputX ("TotIntervalsRemaining", TotIntervalsRemaining);
    run;

    /*  Add in 0s for missing intervals in between existing intervals... 
        AND missing intervals AFTER the last interval in a panel, up to global_end.
        AND Calculate exposures here. */

    /*
    A note about exposure:
        interval_end = min(intnx("&interval", interval_start, 1)-1, &global_end);
        exposure = (interval_end-interval-start+1)/(intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));

    or simply:
        exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval-start + 1 ) /
                    (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
    */ 


    data _gfm_countsOne3 (drop=i n the_count_original );
        set _gfm_countsOne2sort;
        by     &PANELPROPS    interval_start ;
        format expected_next date7.;
        retain expected_next;

        /* Case 1:  There's only one obs for this panel, and it has interval_start = &global_start */
        if first.&lastpanelprop and last.&lastpanelprop then do;
            /* output an obs with 0 count for each interval up till global_end */
            exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                    (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
            output;
            the_count = 0;
            do i=1 to &TotIntervalsRemaining;
                interval_start = intnx("&interval", interval_start, 1);
                exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                    (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
                output;
            end;
        end;
        else if first.&lastpanelprop then do;
            expected_next = intnx("&interval", interval_start, 1);
            exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                    (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
            output;
        end;
        else do;
            if expected_next LT interval_start then do;
                /* figure out how many intervals are missing */
                n = intck("&interval", expected_next-1, interval_start-1);
                the_count_original = the_count;
                /* output n obs with 0 count */
                the_count = 0;
                interval_start=expected_next;
                expected_next = .;
                do i=1 to n;
                    exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                            (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
                    output;
                    interval_start = intnx("&interval", interval_start, 1);
                end;            
                /* output the "current" obs */
                the_count = the_count_original;
                expected_next = intnx("&interval", interval_start, 1);
                exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                        (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
                output; 
            end;
            else do;
                expected_next = intnx("&interval", interval_start, 1);
                exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                        (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
                output;
            end;
            /* Check whether we're at the end of this by-group, and whether we need to add extra
                obs to get up to &global_end */
            if last.&lastpanelprop  then do;
                n = intck("&interval", interval_start, &global_end);
                the_count = 0;
                expected_next = .;  
                do i=1 to n;  /* this loop only goes if n GT 0 */
                    interval_start = intnx("&interval", interval_start, 1);
                    exposure = ( min(intnx("&interval", interval_start, 1)-1, &global_end) - interval_start + 1 ) /
                            (intnx("&interval", interval_start, 1)-intnx("&interval", interval_start, 0));
                    output;
                end;
            end;
        end;
    run;

/***** SEVENTH:  Adjust exposures using missing range table. (and round exposure to nearest .001?)*****/

    %if &nmissingranges GT 0 %then %do;
        proc sql;
            create table _gfm_countsOneMissing
            as select distinct CCUID, &PANEL_PROPS_COMMA, interval_start, the_count, 
                max(0,missing_exposure) as the_missing_exposure,
                max( 0 , exposure-max(0,missing_exposure) ) as adj_exposure,
                max( 0 , exposure-max(0,missing_exposure) ) as rounded_adj_exposure 
               /* w/o rounding, the cell means match exactly with those calculated by VaR */
			   /* Possible future change: round adj_exposure to the nearest 0.001 */
            from _gfm_countsOne3 as l left join _mom_miss4 as r

            on l.interval_start EQ r.missing_tag 
            AND ((( %if &flaginternal_bl_uid %then missing(r.miss_internal_bl_uid) OR missing(l.internal_bl_uid) OR l.internal_bl_uid EQ r.miss_internal_bl_uid; %else 1; )
            AND ( %if &flaginternal_rc_uid %then missing(r.miss_internal_rc_uid) OR missing(l.internal_rc_uid) OR l.internal_rc_uid EQ r.miss_internal_rc_uid; %else 1; )
            AND ( %if &flaglegal_entity_uid %then missing(r.miss_legal_entity_uid) OR missing(l.legal_entity_uid) OR l.legal_entity_uid EQ r.miss_legal_entity_uid; %else 1; )
            AND ( %if &flagprocess_uid %then missing(r.miss_process_uid) OR missing(l.process_uid) OR l.process_uid EQ r.miss_process_uid; %else 1; )
            AND ( %if &flagclass_1 %then missing(r.miss_class_1) OR missing(l.class_1) OR l.class_1 EQ r.miss_class_1; %else 1;)
            AND ( %if &flagclass_2 %then missing(r.miss_class_2) OR missing(l.class_2) OR l.class_2 EQ r.miss_class_2; %else 1; )
            AND ( %if &flagclass_3 %then missing(r.miss_class_3) OR missing(l.class_3) OR l.class_3 EQ r.miss_class_3; %else 1; )
            AND ( %if &flagclass_4 %then missing(r.miss_class_4) OR missing(l.class_4) OR l.class_4 EQ r.miss_class_4; %else 1; )
            AND ( %if &flagclass_5 %then missing(r.miss_class_5) OR missing(l.class_5) OR l.class_5 EQ r.miss_class_5; %else 1; )
            AND ( %if &flagclass_6 %then missing(r.miss_class_6) OR missing(l.class_6) OR l.class_6 EQ r.miss_class_6; %else 1; )
            AND ( %if &flagclass_7 %then missing(r.miss_class_7) OR missing(l.class_7) OR l.class_7 EQ r.miss_class_7; %else 1; )
            AND ( %if &flagclass_8 %then missing(r.miss_class_8) OR missing(l.class_8) OR l.class_8 EQ r.miss_class_8; %else 1; )
            AND ( %if &flagclass_9 %then missing(r.miss_class_9) OR missing(l.class_9) OR l.class_9 EQ r.miss_class_9; %else 1; )
            AND ( %if &flagclass_10 %then missing(r.miss_class_10) OR missing(l.class_10) OR l.class_10 EQ r.miss_class_10; %else 1; )
			AND ( %if &flagown %then missing(r.miss_own) OR missing(l.own) OR l.own EQ r.miss_own; %else 1; )
            )
            /*    either match all other props, or "missing" in r matches everything in l (per property), and vice versa...*/

            )
            group by CCUID, &PANEL_PROPS_COMMA , interval_start;
        quit;
    %end;
    %else %do;  /* THERE ARE NO MISSING RANGES!!! */
        data _gfm_countsOneMissing;
            set _gfm_countsOne3;
            /* calculate csquaredovere, for use in calcuting variance */
               rounded_adj_exposure = exposure; /* !!! Possible future change: round (exposure, .001) */
			   adj_exposure = exposure;
            if rounded_adj_exposure LE 0 then delete;
        run;
    %end;


/***** EIGHTH.  APPLY SCALING and pre-compute csquaredover_e and sume and sumc whatever  *****/
	    /*********** merge SCALING int Loss Data ***********/

    %if &nobs_scale ne 0 %then %do;
    	/***************- INTERNAL -***************/
		%if &dataType eq I %then %do;
			/*- merge count data with Scaling data by INTERNAL_BL_UID and YEAR -*/
			proc sql noprint; 
				create table _gfm_countsMissScale
				as select * from _gfm_countsOneMissing as L left join &scaleData as S
				on L.INTERNAL_BL_UID=S.BLUID_SC and year(L.interval_start)=S.YEAR;   
			quit;       
		%end; /*- INTERNAL -*/
		/***************- EXTERNAL -***************/
		%else %if &dataType EQ E %then %do;
			/*- merge count data with Scaling data by YEAR -*/
			proc sql noprint; 
				create table _gfm_countsMissScale
				as select * from _gfm_countsOneMissing as L left join &scaleData as S
				on year(interval_start)=YEAR;   
			quit;       
		%end;  /*- EXTERNAL -*/
	%end;
	%else %do;
		data _gfm_countsMissScale;
			set _gfm_countsOneMissing;
		run;
	%end;
    data _gfm_countsOneMissing2;
		set _gfm_countsMissScale;
        if rounded_adj_exposure LE 0 then delete;
		if missing(FSCALE) then FSCALEadj = 1; 
		else FSCALEadj=FSCALE;
		/**** APPLY FSCALE. ****/
		/* Save original count in "unscaled_count". Then adjust the_count by multiplying by FSCALEadj.  */
		unscaled_count = the_count;
		exposureadjcount = unscaled_count / rounded_adj_exposure;
		the_count = the_count * FSCALEadj;
        csquaredovere = the_count*the_count / rounded_adj_exposure;
    run;
	proc sort data=_gfm_countsOneMissing2 out=_gfm_countsOneMissing2;
        by &PANELPROPS;
    run;    

	%if &outCountData NE %str() %then %do;
		proc sort data=_gfm_countsOneMissing2 out=outCountData2;
			by CCUID interval_start;
		run;
		proc summary data=outCountData2;
			by CCUID interval_start;
			output out=&outCountData(drop=_TYPE_ _FREQ_) sum(exposureadjcount)=sumexposadjcount;
		run;
		data &outCountData;
			set &outCountData;
			format interval_end date.;
			/* Since we are displaying counts on an interval-basis (e.g. yearly), we do NOT
			*  adjust the interval_end for the global_end.  */
/*			interval_end = min(intnx("&interval", interval_start, 1)-1, &global_end);*/
			interval_end = intnx("&interval", interval_start, 1)-1;
		run;
	%end;

    proc summary data=_gfm_countsOneMissing2;
        by &PANELPROPS;
        var the_count rounded_adj_exposure;
        output out=_summary sum(the_count)=sumcount sum(rounded_adj_exposure)=sumexposure 
            sum(csquaredovere)=sumcsquaredovere
			sum(exposureadjcount)=sumexposadjcount_p
            min(CCUID)=the_CCUID 
            /* CCUID should be the same per panel, so min just grabs the value of CCUID */
        ;
    run;

/*****NINTH:  Find moments.  *****/


    /*****    A.  Find mean and variance per panel ******/
    data _summary2;
        set _summary(drop=_TYPE_);
        avgi = sumcount/sumexposure;
        if (_FREQ_ <= 1) then vari = 0;
        else vari = (sumcsquaredovere/sumexposure - avgi*avgi) * sumexposure / (_FREQ_-1);    
    run;

    /*********** B. Find mean and variance per CCUID (by summing the panel means and variance within each CCUID). */
    /***********   (one call to proc summary for whole matrix *interior cells only* to compute mean and var)  ***********/
    proc sort data=_summary2(rename=(the_CCUID=CCUID)) out=_summary2sort; 
        by CCUID; 
    run;
    proc summary data=_summary2sort;
        by CCUID;
        output out=_summary3 
            sum(avgi)=mean
            sum(vari)=variance
/*			sum(sumexposadjcount_p)=sumexposadjcount_c*/
        ;
    run;

    
    /***** C. Make pretty output data set *****/
    data &outFreqMomts;
        set _summary3(drop=_TYPE_ _FREQ_ ) ;
    run;



    %DONE:
     
    proc datasets library=work memtype=data nolist;
        delete _gfm_countsOne: _summary: _efplevels: _mom_loss: _mom_miss: _loss_work;
    quit;

    %if &abort %then %do;
       %put &orvar_error %sysfunc(sasmsg(&orvads, gfmom_abnormal_error, noquote));
       %let status=1;          
    %end;  
    %orva_status_ds( statusds=&outstatus, status=&status, reason=&reason );


%mend orva_getfreqmomts;

%macro findprops(indataset=, propdataset=);
   ods listing close;
   ods output nlevels=_efplevels;
   proc freq data=&indataset nlevels /* noprint */;
   run;
   
   %let nmissprops=0;
   data &propdataset; 
        set _efplevels end=last; 
        where NNonMissLevels NE 0;    
        if last then call symputx('nmissprops', _N_);
   run;
   %if &nmissprops eq 0 %then %do;
       data &propdataset; set _efplevels; run;
   %end;
   ods listing;
%mend findprops;









/********************************************************************************/
/******************** Example tests  ********************************************/
/*
%orva_getfreqmomts( lossData       =VARPRJ_M.lossi,
                    dataType       =I,
                    global_range   =VARPRJ_M.global_range_i,
                    interval       =year,
                         missing_range  =missingrange2,
                       outFreqMomts   =outFreqMomts,              
                    outStatus      =status_efp,
                         CCV_NAMES      =INTERNAL_BL_UID country_of_incident_uid, 
                          sharedRollup   =false 
                    );  

%orva_getfreqmomts( lossData       = VARPRJ_M.lossi,
                    dataType       =I,
                    global_range   =VARPRJ_M.global_range_i,
                    interval       =year,
                         missing_range  =missingrange2,
                       outFreqMomts   =outFreqMomts,              
                    outStatus      =status_efp,
                         CCV_NAMES      = INTERNAL_BL_UID INTERNAL_RC_UID ,
                          sharedRollup = false
                   );  

%orva_freqmomtsaggr(indataset=WORK.foo, interiorcells=outFreqMomts, num_CCVARS=2,  
                    outdataset=totalsout);
*/
