 /***************************************************************************************** 
 * $Id: orva_kri_transpose.sas,v 1.1.4.38.2.23.2.1.8.1 2012/05/03 15:17:16 sasdje Exp $
 *  Copyright (C), 2005                                                                                                                 
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.                                                                  
 *                                                                                                                                      
 *  NAME:    orva_kri_transpose.sas                                                                                                     
 *  PURPOSE: Denormalizes the KRI datasets by merging the Variables with the Values                                                     
 *           and transposing the result.  Also expands the intervals                                                   
 *           and forecasts KRI values up to the greater of (Global EndDate or Today) plus 2 years.  
 *           Finally, %orva_aggregate is called which fills in missing KRI                                                 
 *           values from parent/child values (if possible).                                                  
 *           
 *           This macro is called during data versioning.
 *                                                                                                                           
 *           output datasets contain:
 *             kribl:                                                                 
 *               INTERVAL_START INTERVAL_END INTERVAL_LENGTH OWN BLUID_KRI KRI_<uid>.
 *             krifirm:
 *                INTERVAL_START INTERVAL_END INTERVAL_LENGTH OWN KRI_<uid>.
 *                                                                                            
 *  SUPPORT: stchri                                                                                                                     
 *                                                                                                                                      
 *  PARAMETERS:                                                                                                                         
 *    kriVals     = the name of the KRI Values dataset.                                                                                 
 *    kriVars     = the name of the KRI Variables dataset.                                                                                                                                     
 *    bl          = true  - handling Business Line KRIs (default).                                                                      
 *                  false - handling Firm KRIs.                                                                                         
 *    startDate   = global start date of data.                                                                                          
 *    endDate     = global end date of data.      
 *    krilib      = libref of where to create the KRI datasets.                                                                                           
 *    ostat       = name of the out status dataset.  Will contain the following:                                                        
 *                    status = 0 on success, 1 otherwise.                                                                               
 *                    reason = printable message.                                                                                       
 *    CTool       = true, if in Consortium Tool
 *                  false, otherwise          
 *    parentKrilib = libref of the parent version's meta folder
 *                   (blank if not an incremental load)
 *    cpiData      = cpi dataset, will be merged into KRI Firm dataset only.
 *                                                                                                                                    
 * NOTES: see sample code at end of file.                                                                                               
 *                                                                                                                                      
 *****************************************************************************************/                                             
%macro orva_kri_transpose(kriVals         = ,                                                                                                   
                          kriVars         = ,                                                                                                   
                          bl              = true,                                                                                               
                          startDate       = ,                                                                                                   
                          endDate         = , 
                          krilib          = ,  
                          intervals       = ,                                                                                              
                          ostat           = ,
                          CTool           = false,
                          parentKrilib    = ,
                          cpiData         = );                                                                                             

   %let STATUS_MSG=%sysfunc(sasmsg(&orvads, status_note, noquote));
   %put &STATUS_MSG %sysfunc(sasmsg(&orvads, kritr_kri_data_transposing_status, noquote, &bl, &intervals));

   /*** create the status variables up front ***/                                                                                         
   %let reason=%sysfunc(sasmsg(&orvads, unknown_error, noquote)); 
   %orva_status_ds( statusds=&ostat, status=1, reason=&reason );  
   %let status=0;                                                                                                                       
   %let reason=%sysfunc(sasmsg(&orvads, successful_note, noquote));                                                                                                              

   /*********** calculate UNTILDATE ***********/
   /* Make UNTILDATE the greater of (Global End Date or Today) plus 2 years. 
      If we're in the consortium tool, make it endDate */
   %if &CTool eq true %then %do;
      %let untildate=&endDate;
   %end;
   %else %do;
     data _NULL_;
        if &endDate GT today() then do;
            untildate = &endDate + 365 + 366;    
        end;
        else do;
            untildate = today() + 365 + 366;    
        end;
        call symput('untildate', untildate);  
     run;
  %end;

   /*********** create SUFFIX for output dataset names ***********/
    %if &bl eq true %then %do;
        %let suffix = bl;
    %end;
    %else %do;
        %let suffix = firm;
    %end;

     /***- do we have any KRI Values? -***/
     %let nobs_vals=0;
     %if &kriVals ^= %str() %then %do;
        data _tmpKriVals; set &kriVals;
          %orva_assignOwn();
        run;
        %let dsid=%sysfunc(open(_tmpKriVals,I));                                                                                        
        %if &dsid ^= 0 %then %do;                                                                                                      
            %let nobs_vals=%sysfunc(attrn(&dsid,nlobs));                                                                                     
            %let rc=%sysfunc(close(&dsid));                                                                                             
        %end;      
     %end;

    /***- do we have any KRI Variables? -***/
     %let nobs_vars=0;
     %if &kriVars ^= %str() %then %do;
        data _tmpKriVars; set &kriVars;
          %orva_assignOwn();
        run;
        %let dsid=%sysfunc(open(_tmpKriVars,I));                                                                                        
        %if &dsid ^= 0 %then %do;                                                                                                      
           %let nobs_vars=%sysfunc(attrn(&dsid,nlobs));                                                                                     
           %let rc=%sysfunc(close(&dsid));                                                                                             
        %end;  
     %end;
 
   /*********** merge and sort ***********/                                                                
   %let nobs=0;                                                                                                                         
   %if &nobs_vars gt 0 and &nobs_vals gt 0 %then %do;                                                                                 
                                                                                                                                                                                    
         /*** Merge KRI Values with KRI Variables names ***/                                                            
         proc sort data=_tmpKriVals; by RISK_FACTOR_VARIABLE_UID; run;                                                                 
         proc sort data=_tmpKriVars; by RISK_FACTOR_VARIABLE_UID; run;                                                                                                                                     
         data _tmpKriVal;                                                                                                               
            merge _tmpKriVals (keep= RISK_FACTOR_VARIABLE_UID                                                                           
                                  RISK_FACTOR_VARIABLE_DATE FEED_REF_ID                                                                                              
                                  %if &bl eq true %then %do; INTERNAL_BL_UID %end;                                                      
                                  RISK_FACTOR_VALUE LEGAL_ENTITY_UID)                                                                                    
                  _tmpKriVars (keep= RISK_FACTOR_VARIABLE_UID                                                                           
                                  RISK_FACTOR_VARIABLE_NAME OWN);                                                                           
            by RISK_FACTOR_VARIABLE_UID;                                                                                                
              if RISK_FACTOR_VARIABLE_DATE eq . then delete;                                                                                              
         run;                                                                                                                           
                                                                                                                       
         /*** Do we have any KRI Data? ***/                                                                                             
         %let dsid=%sysfunc(open(_tmpKriVal,I));                                                                                        
         %if &dsid ^= 0 %then %do;                                                                                                      
            %let nobs=%sysfunc(attrn(&dsid,nlobs));                                                                                     
            %let rc=%sysfunc(close(&dsid));                                                                                             
         %end;                                                                                                                          
   %end;                                                                                                                                
                                                                                                                                        
   /************************* munge KRI data ************************/                                                                  
   %let NUM_KRI_VARS= 0;                                                                                                                
   %if &nobs gt 0 %then %do;                                                                                                            
                                                                                                                                        
     /*** create NAMES and LABELS macros for use later ***/                                                                               
     options sortdup=logical;                                                                                                       
     proc sort data= _tmpKriVal(keep=RISK_FACTOR_VARIABLE_UID RISK_FACTOR_VARIABLE_NAME)                                            
               out= _tmpKriUnique nodup;                                                                                            
          by RISK_FACTOR_VARIABLE_UID;                                                                                                
     run;                                                                                                                           
     data _tmpKriUnique; set _tmpKriUnique end=LAST;                                                                                
          retain names; length names $ 1024;                                                                                          
          retain labels; length labels $ 1024;                                                                                        
          labels = trim(left(labels)) || '$' || trim(left(RISK_FACTOR_VARIABLE_NAME));                                                
          RISK_FACTOR_VARIABLE_NAME = compress("KRI_" || RISK_FACTOR_VARIABLE_UID);                                                   
          names = trim(left(names)) || '$' || trim(left(RISK_FACTOR_VARIABLE_NAME));                                                  
          if LAST then call symput( 'NAMES', names );                                                                                 
          if LAST then call symput( 'LABELS', labels );                                                                               
          if LAST then call symputX( 'NUM_KRI_VARS', _N_ );                                                                              
      run;          
      
      /********************* create  kriOwners ******************************/
      /*** kriOwners dataset will have KRIOWNER_NAME and KRI_OWNER 
           this is used later by orva_krimerge() ***/  
    
      data &krilib..kriowners; set &kriVars;
        KRIOWNER_NAME = compress("KRI_" || RISK_FACTOR_VARIABLE_UID); 
        %orva_assignown();    
        KRI_OWNER = OWN;
        keep KRIOWNER_NAME KRI_OWNER;
      run;                       

        /*** rename KRI variables as KRI_<risk_factor_variable_uid> ***/                                                                                                 
        data _tmpKriVal; set _tmpKriVal;                                                                                               
            RISK_FACTOR_VARIABLE_NAME = compress("KRI_" || RISK_FACTOR_VARIABLE_UID);                                                 
            drop RISK_FACTOR_VARIABLE_UID;     
            %orva_assignown();    /* create the variable "own" from FEED_REF_ID */     
        run;                                   

        /*** transpose KRI Values ***/                                                                                                 
        proc sort data=_tmpKriVal nodups;                                                                                              
            by own
            %if &bl eq true %then %do;
                INTERNAL_BL_UID
            %end;
            RISK_FACTOR_VARIABLE_NAME 
			RISK_FACTOR_VARIABLE_DATE
			LEGAL_ENTITY_UID;
        run;
		data _tmpKriVal;
			set _tmpKriVal;
			by own
            %if &bl eq true %then %do;
                INTERNAL_BL_UID
            %end;
            RISK_FACTOR_VARIABLE_NAME 
			RISK_FACTOR_VARIABLE_DATE
			LEGAL_ENTITY_UID;
			if first.RISK_FACTOR_VARIABLE_DATE then output; /* S0572493: 
			VaR 4.1 doesn't utilize KRIs defined on Legal_Entity_Uid */
		run;
		data _tmpKriVal;
			set _tmpKriVal;
			by own
            %if &bl eq true %then %do;
                INTERNAL_BL_UID
            %end;
            RISK_FACTOR_VARIABLE_NAME 
			RISK_FACTOR_VARIABLE_DATE;
			if first.RISK_FACTOR_VARIABLE_NAME and last.RISK_FACTOR_VARIABLE_NAME then delete;
		run;
		proc sort data=_tmpKriVal nodups;
            by own
            %if &bl eq true %then %do;
                INTERNAL_BL_UID
            %end;
            RISK_FACTOR_VARIABLE_DATE;
        run;
        proc transpose data=_tmpKriVal out=_tmpKriTrans; 
            by own
            %if &bl eq true %then %do;                                                                                             
            INTERNAL_BL_UID                                                                                                        
            %end;                                                                                                                    
            RISK_FACTOR_VARIABLE_DATE;                                                                                                                 
            id RISK_FACTOR_VARIABLE_NAME;                                                                                             
            var RISK_FACTOR_VALUE ;                                                                                                    
        run;                                                                                                                           
        %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;                                                                                       
            %put &orvar_error %sysfunc(sasmsg(&orvads, kritr_cant_transpose_kri_vals_error, noquote));                                                                             
            %let abort=1;                                                                                                         
            %let reason=%sysfunc(sasmsg(&orvads, kritr_cant_transpose_kri_vals_error, noquote));                                                                           
            %goto DONE;                                                                                                             
        %end; 
        
        
      /**************** get KRI OPTIONS *********************/
      data _tmpkrioptions; set &kriVars;
          length KRI $ 32;
          KRI = "KRI_" || trim(left(RISK_FACTOR_VARIABLE_UID));   
          accumulate=  accumulation_method;
          model=  forecasting_model;
          aggr = aggregation_method;
          if trim(left(aggr)) = "MINIMUM" then aggr = "MIN";
          else if trim(left(aggr)) = "MAXIMUM" then aggr = "MAX";
          else if trim(left(aggr)) = "SUM" then aggr = "SUM";
          else aggr = "MEAN";
          if find("%trim(%left(&NAMES))",left(trim(KRI))) > 0 then output;
          keep kri accumulate model aggr lower_bound upper_bound;
      run;  

      proc sort data= _tmpkrioptions(keep=aggr)       
                out=_tmpkritoptsunique nodup;
         by aggr;
      run;
      data _NULL_; set _tmpkritoptsunique end=LAST;
         retain KRI_AGGR_METHODS;
          length KRI_AGGR_METHODS $ 512;
        KRI_AGGR_METHODS = trim(left(KRI_AGGR_METHODS)) || " " || trim(left(aggr));
        if LAST then do;
           call symput('KRI_AGGR_METHODS',trim(left(KRI_AGGR_METHODS)));
        end;
     run;

      data _tmpKriTrans; set _tmpKriTrans (drop=_NAME_); run;                                        
      proc sort data=_tmpKriTrans; by RISK_FACTOR_VARIABLE_DATE; run;

       /***************** create a dataset for each interval and forecast ********************/
        %orva_freq_preprocess(  data               = _tmpKriTrans,
                                dataout            = _tmpkri&suffix._,
                                status_preprocess  = _tmpkri_status_preprocess,
                                KRIoptions         = _tmpkrioptions,
                                untildate          = &untildate, 
                                global_start       = &startDate,
                                intervals          = &intervals,
                                blflag             = &bl,
                                CTool              = &CTool);        

        
       /***************** Post-process each dataset *************/                                                                                            
        %let ninterval = 1;
        %let currinterval=%scan(&intervals, &ninterval, ' ');
        %do %while (&currinterval ne );
        
              /***************** handle aggregation on each dataset *************/ 
              %if &bl eq true %then %do; 
			  		data _tmpbldata;
						set &krilib..bli;
						%if &CTool eq true %then %do;
							if floor(FEED_REF_ID) EQ 999 then delete;
						%end;
					run;
                   %orva_kri_aggregate(kriData    = work._tmpkri&suffix._&currinterval,
                                       blData     = _tmpbldata,
                                       kriOptions = _tmpkrioptions);
              %end; 
 
              /***************** Add labels to KRI columns in each dataset *************/  
              data work._tmpkri&suffix._&currinterval; set work._tmpkri&suffix._&currinterval;                                                                                                     
                %if &bl eq true %then %do;                                                                                                 
                     rename INTERNAL_BL_UID=BLUID_KRI;                          
                %end;                                                                                                                    
                %let num=1;                                                                                                           
                %let name=%scan(&NAMES, &num, '$');                                                                                      
                %let label=%scan(&LABELS, &num, '$');                                                                                    
                %do %while (&name ne );                                                                                                  
                    label &name = "&label";                                                                                              
                    
                    /* it is possible to have missing values for some business-lines */
                    if missing(&name) EQ 1 then &name=0;

                    %let num=%eval(&num+1);                                                                                                
                    %let name=%scan(&NAMES, &num, '$');                                                                                    
                    %let label=%scan(&LABELS, &num, '$');                                      
                %end;                                                                                                                    
             run; 

             data &krilib..kri&suffix._&currinterval (compress=binary);
                  set work._tmpkri&suffix._&currinterval;
             run;
             proc sort data=&krilib..kri&suffix._&currinterval nodups; 
             by interval_start   %if &bl eq true %then bluid_kri; ; run;


             /********* next interval *********/
             %let ninterval=%eval(&ninterval+1);
             %let currinterval=%scan(&intervals, &ninterval, ' ');
        %end;  
    %end;                                                           
    
 
    /************************* There are no KRIs ***********************/       
    %else %do;     

        /* Create a dataset with global start and global end dates.  To be expanded. */
        data _withsort;
            format interval_start interval_end date.;
            interval_start = &startDate;
            interval_end = &untildate;
        run;

        %let ninterval = 1;
        %let currinterval=%scan(&intervals, &ninterval, ' ');
        %do %while (&currinterval ne );
                
            %orva_interval_datasets(Withsort    = _withsort,
                                    intervalout    = _intervalout,
                                    interval    = &currinterval);

            data work._tmpkri&suffix._&currinterval;
                set _intervalout;
                %if &bl eq true %then %do;
                   BLUID_KRI = .;
                %end;  
                OWN = 0;         
            run;

            /******** append new kris dataset to current dataset **********/
            data &krilib..kri&suffix._&currinterval (compress=binary);
                 set work._tmpkri&suffix._&currinterval;
            run;

            /********* next interval *********/
            %let ninterval=%eval(&ninterval+1);
            %let currinterval=%scan(&intervals, &ninterval, ' ');
        %end;
    %end;  
    /* There are no KRIs */
                      
    /************************** handle CPI Data ******************************/
    %if &bl NE true AND &cpiData NE %str() %then %do;
       %let nobs_cpi=0;
       %if &cpidata ^= %str() %then %do;
           %let dsid=%sysfunc(open(&cpidata,I)); 
           %if &dsid ^= 0 %then %do;         
                  %let nobs_cpi=%sysfunc(attrn(&dsid,nlobs));  
                   %let rc=%sysfunc(close(&dsid));
           %end;  
       %end;
       %if &nobs_cpi NE 0 %then %do;
         %let ninterval = 1;
         %let currinterval=%scan(&intervals, &ninterval, ' ');
         %do %while (&currinterval ne );
            proc sql noprint;
              create table _workcpi_
              as select * from &krilib..kri&suffix._&currinterval as L left join &cpiData as C on 
                      intnx('YEAR', L.INTERVAL_START, 0) eq C.CPI_DATE ;
            quit;        
            data &krilib..kri&suffix._&currinterval (compress=binary); 
                   set _workcpi_;
                   if CPI_VALUE eq . then CPI_VALUE = 1;           
                   KRI_0 = log(CPI_VALUE);         
                   label KRI_0 = 'LogCPI';      
                   drop CPI_DATE CPI_VALUE CPI_ADJ CPI_COUNTRY_ID;
            run; 
            
         /********* next interval *********/
         %let ninterval=%eval(&ninterval+1);
         %let currinterval=%scan(&intervals, &ninterval, ' ');
      %end;
      %end;
    %end;

%DONE:                                                                                                                                  
                                                                                                                             
   %orva_status_ds( statusds=&ostat, status=&status, reason=&reason );
                                                                                                                                    
   /*- cleaup temporary datasets   -*/                                                                                       
   proc datasets library=work memtype=data nolist nodetails;
      delete _tmpKri: _intervalout _withsort _tmpbldata;       
   quit;      
   
   %put &STATUS_MSG %sysfunc(sasmsg(&orvads, kritr_kri_data_transpose_done_status, noquote, &bl, &intervals));
   
%mend orva_kri_transpose;                                                                                                               
                     

       
/**************************************                                                                                                 
data _NULL_;                                                                                                                            
   format startDate DATE.;                                                                                                              
   format endDate DATE.;                                                                                                                
   startDate ="10dec1998"d;                                                                                                             
   endDate = "20Jan2007"d;                                                                                                              
   call symput("StartDate", startDate);                                                                                                 
   call symput("EndDate", endDate);                                                                                                     
   put startDate=;                                                                                                                      
   put endDate=;                                                                                                                        
run;                                                                                                                                    
options noMPRINT noSYMBOLGEN;                                                                                                               
%orva_kri_transpose(kriVals= VARORG.risk_factor_value_firm,                                                                             
                    kriVars= VARORG.risk_factor_variable,                                                                                        
                    bl= false,                                                                                                          
                    startDate= &StartDate,                                                                                              
                    endDate= &EndDate,                                                                                                                                                                                                              
                    ostat= ostat);                                                                                                      
                                                                                                                                        
%orva_kri_transpose(kriVals= VARORG.risk_factor_value_bl,                                                                               
                    kriVars= VARORG.risk_factor_variable,                                                                                                      
                    bl= true,                                                                                                           
                    startDate= &StartDate,                                                                                              
                    endDate= &EndDate,                                                                                                                                                                                          
                    ostat= ostat);                                                                                                      
                                                                                                                                        
****************************************/                                                                                               
