/*****************************************************************************************
 * Copyright (C), 2009
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    ORVA_rare_scen_mom.sas                                             
 *  PURPOSE: Compute moments for all scenarios types
 *
 *  SUPPORT: jariou
 *
 *  PARAMETERS:
 *      scenUids     = space delimited list of Rare Event Scenarios to use.
 *      outSevMomts  = output severity moments dataset.
 *      outFreqMomts = output frequency moments dataset.
 *      outStatus    = output status dataset.
 *      debug        = true or false (defaulted to false) Indicates whether the user gets extra info
 *
 *  EXAMPLE:
 *        %orva_rare_scen_sim(scenUids=1034 2304, outScenSim=VARPRJ.rareScenSim);
 *
 *  Enumerations:
 *    EVENT_TYPE     
 *       Certain
 *       Independent
 *       Correlated
 *
 *    SCENARIO_SPEC 
 *       ModelBased
 *       BucketBased          
 *****************************************************************************************/
%macro ORVA_rare_scen_mom( 
                           scenUids     = ,
                           outSevMomts  = ,              
                           outFreqMomts = ,
                           outStatus    = status,
                           debug        = false
                           );

   data _orsm_results;
      set _NULL_;
   run;

   %let reason = Success;
   %let status = 0;

   %orva_status_ds( 
                     statusds = &outstatus, 
                     status   = 0, 
                     reason   = &reason 
                     );
 
   %let nscen = 1;
   %let scen  = %scan(&scenUids, &nscen, ' ');
   
   %do %while(&scen NE %str()); /* Level = 1 */
      %let scenarioData = %str();
      %orvu_find_dataset(  
                           nameIn  = Scenario_&scen, 
                           libs    = VARPRJ VARPRJ_M,
                           nameOut = scenarioData
                           );

      %let scenarioGroup=%str();
      %orvu_find_dataset( 
                           nameIn  = ScenarioGroup_&scen, 
                           libs    = VARPRJ VARPRJ_M,
                           nameOut = scenarioGroup
                           );   
   
      %let scenarioPlotBrkDwn = %str();
      %orvu_find_dataset(  
                           nameIn  = Scenario_PlotBrkDwn_&scen, 
                           libs    = VARPRJ VARPRJ_M,
                           nameOut = scenarioPlotBrkDwn
                           );   

      %let EVENT_TYPE    = %str();
      %let SCENARIO_SPEC = %str();

      %if &scenarioData NE %str() %then %do; 
         data _null_;
            set &scenarioData; 
            if _N_ = 1;

            if EVENT_TYPE = 0 then do;
               call symput ('EVENT_TYPE',"Certain");
            end;

            if EVENT_TYPE = 1 then do;
               call symput ('EVENT_TYPE',"Independent");
            end;

            if EVENT_TYPE = 2 then do;
               call symput ('EVENT_TYPE',"Correlated");
            end;
         run;
      %end;
      %else %do;
         %put &orvar_error %sysfunc(sasmsg(&orvads, rare_scen_no_exist_no_sim_error, noquote, &scen));
      %end;
   
      %if &scenarioPlotBrkDwn = %str() %then %do; 
         %let SCENARIO_SPEC = ModelBased;
      %end;
      %else %do;
         %let SCENARIO_SPEC = BucketBased;
      %end; 

      %if %upcase(&debug) EQ TRUE %then %do;
         %put ;
         %put |      *************************************************;
         %put |      *;
         %put |      *  Scenario           =  &scen;
         %put |      *;
         %put |      *  ----------------------------------------------;
         %put |      *;
         %put |      *  Scenario           =  &scenarioData;
         %put |      *  ScenarioGroup      =  &scenarioGroup;
         %put |      *  ScenarioPlotBrkDwn =  &scenarioPlotBrkDwn;
         %put |      *;
         %put |      *  ----------------------------------------------;
         %put |      *;
         %put |      *  EVENT_TYPE         =  &EVENT_TYPE;
         %put |      *  SCENARIO_SPEC      =  &SCENARIO_SPEC;
         %put |      *;
         %put |      *************************************************;
         %put ;
      %end; /* justCheckScenarios = TRUE */

      %if  &EVENT_TYPE = Certain %then %do; 
         %if &SCENARIO_SPEC = ModelBased %then %do;
            %orv_scenarioMomentsCertain( 
                                           dataIn   = &scenarioData,
                                           type     = model,
                                           dataOut  = _orsm_&scen
                                           );
         %end; /* SCENARIO_SPEC = ModelBased */
         %else %if &SCENARIO_SPEC = BucketBased %then %do; 
            %orv_scenarioMomentsCertain( 
                                           dataIn   = &scenarioPlotBrkDwn,
                                           type     = bucket,
                                           dataOut  = _orsm_&scen
                                           );
         %end; /* SCENARIO_SPEC = BucketBased */
         %else %do;
            %put &orvar_error %sysfunc(sasmsg(&orvads, rare_scen_spec_missing_no_sim_error , noquote, &scen));
         %end;
      %end; /* EVENT_TYPE = Certain */
      %else %if &EVENT_TYPE = Independent %then %do; 
         %if &SCENARIO_SPEC = ModelBased %then %do; 
            %orv_scenarioMomentsIndependent(   
                                               dataIn   = &scenarioData,
                                               type     = model,
                                               dataOut  = _orsm_&scen
                                               );
         %end; /* SCENARIO_SPEC = ModelBased */
         %else %if &SCENARIO_SPEC = BucketBased %then %do; 
            %orv_scenarioMomentsIndependent(   
                                               dataIn   = &scenarioPlotBrkDwn,
                                               type     = bucket,
                                               dataOut  = _orsm_&scen
                                               );
         %end; /* SCENARIO_SPEC = BucketBased */
         %else %do;
            %put &orvar_error %sysfunc(sasmsg(&orvads, rare_scen_spec_missing_no_sim_error, noquote, &scen));
         %end;
      %end; /* EVENT_TYPE = Independent */
      %else %if  &EVENT_TYPE = Correlated %then %do; 
         %if &SCENARIO_SPEC = ModelBased %then %do; 
            %orv_scenarioMomentsCorrelated( 
                                              dataIn   = &scenarioData,
                                              groups   = &scenarioGroup,
                                              type     = model,
                                              dataOut  = _orsm_&scen
                                              );
         %end; /* SCENARIO_SPEC = ModelBased */
         %else %if &SCENARIO_SPEC = BucketBased %then %do; 
            %orv_scenarioMomentsCorrelated( 
                                              dataIn   = &scenarioPlotBrkDwn,
                                              groups   = &scenarioGroup,
                                              type     = bucket,
                                              dataOut  = _orsm_&scen
                                              );
         %end; /* SCENARIO_SPEC = BucketBased */
         %else %do;
            %put &orvar_error %sysfunc(sasmsg(&orvads, rare_scen_spec_missing_no_sim_error, noquote, &scen));
         %end;
      %end; /* EVENT_TYPE = Correlated */
      %else %do;
         %put &orvar_error %sysfunc(sasmsg(&orvads, rare_scen_no_exist_no_sim_error, noquote, &scen));
      %end;

      %orvu_append(
                     base = _orsm_results,
                     new  = _orsm_&scen
                     );   
                
      %let nscen = %eval(&nscen + 1);
      %let scen  = %scan(&scenUids, &nscen, ' ');
   %end; /* Level = 1 */       

   %orv_scenarioGetCount(
                           modelScenData = &scenarioData, 
                           sevMomData    = _orsm_results
                           );

   data &outSevMomts;
      set _orsm_results;
      rename Severity_Dist = Distribution ;
      keep  
            Severity_Dist 
            SCENARIO_UID 
            CCUID 
            Mean 
            Std 
            Skewness 
            Kurtosis 
            Param_: 
            Count
            ; 
   run; 

   data &outFreqMomts;
      set _orsm_results;
      Mean     = fm1;
      Variance = fm2;
      rename Frequency_Dist = Distribution ;
      keep 
            Frequency_Dist 
            SCENARIO_UID 
            CCUID 
            Mean 
            Variance 
            Count 
            FParam_:
            ; 
   run; 
   
   %orvu_delete_data( data = _orsm_: );
%mend ORVA_rare_scen_mom;

%macro orv_scenarioMomentsCertain(
                                    dataIn  = , 
                                    type    = , 
                                    dataOut =
                                    );

   /* I need to figure out the size fo the array*/
   %if %upcase(&type) NE MODEL %then %do;
      data _NULL_;
         set &dataIn ;
         retain maxBucketCount 0;
         array buckWeights bucketwgt:;
         maxBucketCount = max( maxBucketCount , nBuckets);
         call symputX( "buckWeightsSize", maxBucketCount );
         call symputX( "buckSize", maxBucketCount + 1 );
      run;   
   %end;

   data &dataOut;
      retain 
            lastM1  0 
            lastM2  0 
            lastM3  0 
            lastM4  0 
            lastFM1 0 
            lastFM2 0
            ;
      set &dataIn;
      by ccuid;

      %if %upcase(&type) = MODEL %then %do;
         call FreqMoments(
                           Frequency_Dist, 
                           &theFreqSimCallParams., 
                           fm1, 
                           fm2 
                           );
         call SevMoments(
                           Severity_Dist, 
                           &theSevSimCallParams., 
                           m1, 
                           m2, 
                           m3, 
                           m4
                           );
      %end;
      %else %do;
         array    Buckets[*] bucket0-bucket&buckWeightsSize;
         array tmpBuckets[&buckSize] _temporary_;

         array    BucketWeights[*] bucketwgt1-bucketwgt&buckWeightsSize;
         array tmpBucketWeights[&buckWeightsSize] _temporary_;

         array    Moments[4] m1-m4;
         array tmpMoments[4] _temporary_;

         /* First define the right frequency distribution for each CCUID */
         call BucketFreqSimDef(  
                                 Frequency_Mean, 
                                 Frequency_Min, 
                                 Frequency_Max,
                                 Frequency_dist,
                                 FParam_1, 
                                 FParam_2
                                 );
         /* Now compute frequency and severity moments */
         call FreqMoments(
                           Frequency_Dist, 
                           &theFreqSimCallParams., 
                           fm1, 
                           fm2 
                           );

         do i = 1 to &buckWeightsSize;
            tmpBuckets[i]       = Buckets[i]; 
            tmpBucketWeights[i] = BucketWeights[i];
         end;

         tmpBuckets[&buckSize] = Buckets[&buckSize];

         call BucketSevMoments(
                                 tmpBuckets, 
                                 tmpBucketWeights,
                                 nBuckets,
                                 tmpMoments
                            );   
         do i = 1 to 4 ;
            Moments[i] = tmpMoments[i];
         end;
      %end;

      if first.ccuid then do;
         lastM1  = 0;
         lastM2  = 0;
         lastM3  = 0;
         lastM4  = 0;
         lastFM1 = 0;
         lastFM2 = 0;
      end;

      m1 = lastM1 + m1 * fm1;
      m2 = lastM2 + m2 * fm1;
      m3 = lastM3 + m3 * fm1;
      m4 = lastM4 + m4 * fm1;

      fm1 = fm1 + lastFm1;
      fm2 = fm2 + lastFm2;
   
      lastM1  = m1;
      lastM2  = m2;
      lastM3  = m3;
      lastM4  = m4;
      lastFM1 = fm1;
      lastFM2 = fm2;
      
      if not last.ccuid then do;
            delete;
      end;
      else do;
         m1 = m1 / fm1;
         m2 = m2 / fm1;
         m3 = m3 / fm1;
         m4 = m4 / fm1;
      end;

      keep  
            SCENARIO_UID 
            CCUID 
            Frequency_Dist 
            Severity_Dist 
            fm1 
            fm2 
            fparam_: 
            param_: 
            m1 
            m2 
            m3 
            m4 
            ;
   run;

   %orvu_create_central_moments(
                                 dataIn  = &dataOut, 
                                 dataOut = &dataOut
                                 );
%mend orv_scenarioMomentsCertain;

%macro orv_scenarioMomentsIndependent(
                                       dataIn  = , 
                                       type    = , 
                                       dataOut =
                                       );

   /* I need to figure out the size fo the array*/
   %if %upcase(&type) NE MODEL %then %do;
      data _NULL_;
         set &dataIn ;
         retain maxBucketCount 0;
         array buckWeights bucketwgt:;
         maxBucketCount = max( maxBucketCount , nBuckets);
         call symputX( "buckWeightsSize", maxBucketCount );
         call symputX( "buckSize", maxBucketCount + 1 );
      run;   
   %end;

   data &dataOut;
      set &dataIn;   
      
      %if %upcase(&type) = MODEL %then %do;
         /* Compute frequency moments */
         call FreqMoments(
                           Frequency_Dist, 
                           &theFreqSimCallParams., 
                           fm1, 
                           fm2 
                           );
      %end;
      %else %do;
         call BucketFreqSimDef(  
                                 Frequency_Mean, 
                                 Frequency_Min, 
                                 Frequency_Max,
                                 Frequency_dist, 
                                 FParam_1, 
                                 FParam_2
                                 );

         /* Compute frequency moments */
         call FreqMoments(
                           Frequency_Dist, 
                           &theFreqSimCallParams., 
                           fm1, 
                           fm2 
                           );
      %end;

      /* Adjust moments for probability of coverage */
      fm1 = fm1 * PROBABILITY;
      fm2 = fm1 * ( 1 - PROBABILITY) + fm2 * PROBABILITY * PROBABILITY ;

      %if %upcase(&type) = MODEL %then %do;
         /* Compute severity moments */
         call SevMoments(
                           Severity_Dist, 
                           &theSevSimCallParams., 
                           m1, 
                           m2, 
                           m3, 
                           m4
                           );
      %end;
      %else %do;
         array    Buckets[*] bucket0-bucket&buckWeightsSize;
         array tmpBuckets[&buckSize] _temporary_;

         array    BucketWeights[*] bucketwgt1-bucketwgt&buckWeightsSize;
         array tmpBucketWeights[&buckWeightsSize] _temporary_;

         array    Moments[4] m1-m4;
         array tmpMoments[4] _temporary_;

         do i = 1 to &buckWeightsSize;
            tmpBuckets[i] = Buckets[i]; 
            tmpBucketWeights[i] = BucketWeights[i];
         end;

         tmpBuckets[&buckSize] = Buckets[&buckSize];
         call BucketSevMoments(
                                 tmpBuckets,
                                 tmpBucketWeights,
                                 nBuckets ,
                                 tmpMoments
                               );   
         do i = 1 to 4 ;
            Moments[i] = tmpMoments[i];
         end;
      %end;
      
      keep  
            Frequency_Dist
           FParam_:
           Severity_Dist 
           SCENARIO_UID 
           SCENARIO_EVENT_UID 
           CCUID 
            fm1 
            fm2 
            m1 
            m2 
            m3 
            m4 
           Param_:
           ;
   run; 

   /***********************************************************/
   /* For the same CCUID, many SCENARIO_EVENT_UID's may exist */
   /* We need to combine those moments                        */
   /***********************************************************/

   /* First sort in the proper order to be manipulated */
   proc sort data=&dataOut;
      by CCUID ;
   run;

   /* Do the accumulation per CCUID */
   data &dataOut;
      set &dataOut;
      by CCUID;
      retain 
               Total_fM1 0 
               Total_fM2 0 
               Total_M1  0 
               Total_M2  0 
               Total_M3  0  
               Total_M4  0
            ;

      if first.CCUID then do;
            Total_fM1 = 0; 
            Total_fM2 = 0; 
            Total_M1  = 0;
            Total_M2  = 0;
            Total_M3  = 0;
            Total_M4  = 0;
      end;
      
      Total_fM1 = Total_fM1 + fM1;
      Total_fM2 = Total_fM2 + fM2;

      Total_M1 = Total_M1 + fm1 * M1;
      Total_M2 = Total_M2 + fm1 * M2;
      Total_M3 = Total_M3 + fm1 * M3;
      Total_M4 = Total_M4 + fm1 * M4;

      if last.CCUID then do;
         fM1 = Total_fM1 ;
         fM2 = Total_fM2 ;

         M1 = Total_M1 / Total_fM1 ;
         M2 = Total_M2 / Total_fM1;
         M3 = Total_M3 / Total_fM1;
         M4 = Total_M4 / Total_fM1;
         output;
      end;

      keep  
            Frequency_Dist
           FParam_:
           Severity_Dist 
           SCENARIO_UID 
           SCENARIO_EVENT_UID 
           CCUID 
            fm1 
            fm2 
            m1 
            m2 
            m3 
            m4 
           Param_:
           ;
   run;

   %orvu_create_central_moments(
                                   dataIn  = &dataOut, 
                                   dataOut = &dataOut
                                   );
%mend orv_scenarioMomentsIndependent;

%macro orv_scenarioMomentsCorrelated(
                                       dataIn  = , 
                                       groups  = , 
                                       type    = , 
                                       dataOut =
                                       );

   /* I need to figure out the size fo the array*/
   %if %upcase( &type ) NE MODEL %then %do;
      data _NULL_;
         set &dataIn ;
         retain maxBucketCount 0;
         array buckWeights bucketwgt:;
         maxBucketCount = max( maxBucketCount , nBuckets);
         call symputX( "buckWeightsSize", maxBucketCount );
         call symputX( "buckSize", maxBucketCount + 1 );
      run;   
   %end;

   /* Keep only one copy of SENARIO_EVENT_UID and the correponding frequency information */ 
   proc sort 
            data = &dataIn 
            out  = _orsm_event_ids(keep = 
                                          SCENARIO_UID
                                          SCENARIO_EVENT_UID 
                                          FREQUENCY_:
                                          FPARAM_:
                                          )
            nodupkey
            ;
      by SCENARIO_EVENT_UID; 
   run;

   /* Get the frequency moment for each Event_ID */
   data _orsm_Events;
      length Frequency_dist $ 12;
      set _orsm_event_ids;
      
      %if %upcase(&type) = MODEL %then %do;
         call FreqMoments(
                           Frequency_Dist, 
                           &theFreqSimCallParams., 
                           fm1, 
                           fm2 
                           );
      %end;
      %else %do;
         call BucketFreqSimDef(  
                                 Frequency_Mean, 
                                 Frequency_Min, 
                                 Frequency_Max,
                                 Frequency_dist, 
                                 FParam_1,
                                 FParam_2
                                 );
         call FreqMoments(
                           Frequency_Dist, 
                           &theFreqSimCallParams., 
                           fm1, 
                           fm2 
                           );
      %end;

      keep 
            Scenario_Event_UID 
            Scenario_UID 
            fm1 
            fm2
            ;
   run;

   /* Get a unique copy of all the groups with their probabilities*/
   proc sort 
            data = &groups 
            out  = _orsm_group_ids 
            nodupkey
            ;
      by 
         Event_ID 
         Group_ID
         ;
   run;

   /* Merge frequency moments with group information */
   proc sql noprint;
      CREATE TABLE 
                     _orsm_events_with_group_info (keep =  
                                                         Event_ID 
                                                         Scenario_UID 
                                                         Group_ID 
                                                         Group_Prob
                                                         fm1
                                                         fm2
                                                         )
      AS SELECT 
                     * 
      FROM 
                     _orsm_events    a, 
                     _orsm_group_ids b
      WHERE 
                     a.SCENARIO_EVENT_UID = b.EVENT_ID
                     ;
   quit;

   /* Add the CCUID info from the groups dataset */
   proc sql noprint;
      CREATE TABLE 
                     _orsm_ccuid_freq_mom(keep = 
                                             SCENARIO_UID
                                             EVENT_ID 
                                             GROUP_ID 
                                             GROUP_PROB
                                             CCUID
                                             fm1
                                             fm2
                                             )
      AS SELECT 
                     * 
      FROM 
                     _orsm_events_with_group_info a, 
                     &groups                      b
      WHERE 
                     a.GROUP_ID = b.GROUP_ID
                     ;
   quit;

   /* Incorporate the severity information for each CCUID */
   proc sql noprint;
      CREATE TABLE 
                     _orsm_freq_with_sev(keep = 
                                                   SCENARIO_UID
                                                   EVENT_ID 
                                                   GROUP_ID 
                                                   GROUP_PROB 
                                                   CCUID
                                                   %if %upcase(&type) = MODEL %then %do;
                                                      SEVERITY_DIST
                                                      PARAM_:
                                                   %end;
                                                   %else %do;
                                                      NBUCKETS
                                                      BUCKET:
                                                   %end;
                                                   fm1
                                                   fm2
                                                   )
      AS SELECT 
                     * 
      FROM 
                     _orsm_ccuid_freq_mom a, 
                     &dataIn              b
      WHERE 
                     (a.EVENT_ID = b.SCENARIO_EVENT_UID)
            AND
                     (a.CCUID = b.CCUID)
            ;
   quit;

   /* Sort so the accumulation by CCUID can be done */
   proc sort 
            data = _orsm_freq_with_sev 
            out  = _orsm_sev
            ;
      by 
         CCUID 
         EVENT_ID 
         ;
   run;

   /* Accumulate the total group probabilities                     */
   /* Also compute severity moments                                */
   data &dataOut;
      set _orsm_sev;
      by CCUID EVENT_ID;
      retain Total_Prob 0;

      if first.EVENT_ID then do;
         Total_Prob = 0;
      end;
      
      Total_Prob = Total_Prob + Group_Prob ;

      if last.EVENT_ID then do;
         Prob = Total_Prob;

         %if %upcase(&type) = MODEL %then %do;
            call SevMoments(
                              Severity_Dist, 
                              &theSevSimCallParams., 
                              m1, 
                              m2, 
                              m3, 
                              m4
                              );
         %end;
         %else %do;
            array    Buckets[*] bucket0-bucket&buckWeightsSize;
            array tmpBuckets[&buckSize] _temporary_;
   
            array    BucketWeights[*] bucketwgt1-bucketwgt&buckWeightsSize;
            array tmpBucketWeights[&buckWeightsSize] _temporary_;
   
            array    Moments[4] m1-m4;
            array tmpMoments[4] _temporary_;
   
            do i = 1 to &buckWeightsSize;
               tmpBuckets[i] = Buckets[i]; 
               tmpBucketWeights[i] = BucketWeights[i];
            end;

            tmpBuckets[&buckSize] = Buckets[&buckSize];



            call BucketSevMoments(
                                    tmpBuckets, 
                                    tmpBucketWeights, 
                                    nBuckets,
                                    tmpMoments
                                  );   

            do i = 1 to 4 ;
               Moments[i] = tmpMoments[i];
            end;
         %end;


         output;
      end;

      keep 
            SCENARIO_UID 
            CCUID 
            Prob 
            fm1 
            fm2 
            m1 
            m2 
            m3 
            m4 
            ;
   run;

   /* Do the accumulation per CCUID */
   data &dataOut;
      set &dataOut;
      by CCUID;
      retain   
            Total_fM1 0 
            Total_fM2 0 
            Total_M1  0 
            Total_M2  0 
            Total_M3  0 
            Total_M4  0
            ;

      if first.CCUID then do;
            Total_fM1 = 0; 
            Total_fM2 = 0; 
            Total_M1  = 0;
            Total_M2  = 0;
            Total_M3  = 0;
            Total_M4  = 0;
      end;
      
      tmp_fm1   = fm1 * Prob;
      Total_fM1 = Total_fM1 + tmp_fM1;
      Total_fM2 = Total_fM2 + fm1 * Prob * ( 1 - Prob) + fm2 * Prob*Prob;

      Total_M1 = Total_M1 + tmp_fm1 * M1;
      Total_M2 = Total_M2 + tmp_fm1 * M2;
      Total_M3 = Total_M3 + tmp_fm1 * M3;
      Total_M4 = Total_M4 + tmp_fm1 * M4;

      if last.CCUID then do;
         fM1 = Total_fM1 ;
         fM2 = Total_fM2 ;

         M1 = Total_M1 / Total_fM1 ;
         M2 = Total_M2 / Total_fM1;
         M3 = Total_M3 / Total_fM1;
         M4 = Total_M4 / Total_fM1;
         output;
      end;

      keep  
            SCENARIO_UID 
            CCUID 
            fm1 
            fm2 
            m1 
            m2 
            m3 
            m4 
            ;
   run;

   %orvu_create_central_moments(
                                   dataIn  = &dataOut, 
                                   dataOut = &dataOut
                                   );
%mend orv_scenarioMomentsCorrelated;

%macro orv_scenarioGetCount(
                              modelScenData = ,  
                              sevMomData    =
                              );
   /* get counts of Unique Events per cell */
   data _counts0; 
      set &modelScenData; 
      keep 
           CCUID 
           scenario_event_uid
           ; 
   run; 

   proc sort data = _counts0 nodups; 
      by 
         CCUID 
         scenario_event_uid
         ; 
   run;

   proc summary data = _counts0;
      by CCUID;
      var scenario_event_uid;
      output out = _counts(drop=_type_ _freq_)
      n = COUNT;
   run;

   proc sql noprint; 
      CREATE TABLE 
                  _withCounts 
      AS SELECT 
                  * 
      FROM        
                  &sevMomData m 
      LEFT JOIN
                  _counts c 
      ON 
                  m.CCUID = c.CCUID;
   quit;

   data &sevMomData; 
      set _withCounts; 
   run;

   %orvu_delete_data( data = 
                              _counts0 
                              _counts  
                              _withCounts
                              );
%mend orv_scenarioGetCount;
