/*****************************************************************************************
 * Copyright (C), 2008
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    ORVA_rare_sim_init.sas                                             
 *
 *  SUPPORT: jariou
 *
 *  PURPOSE:      Creates compiled functions 
 *                through FCMP for use in rare event scenario
 *
 *  PARAMETERS:
 *          cmpLib            = Library where to store the FCMP functions
 *                              It is up to the caller to make sure that this is in the 
 *                              cmp search path.
 *          package           = Package where to store the FCMP functions
 *          paramName         = Name of parameter for internal use in the functions
 *          debug             = Flag (true, false=DEFAULT) to display extra info in log.
 *
 * NOTES: Should only be called by %orva_init() shortly after connecting to sas.
 * 
 *****************************************************************************************/
%macro orva_rare_sim_init( 
                           cmpLib            = ,
                           package           = ,
                           paramName         = ,
                           debug             = false
                           );


   proc fcmp outlib = &cmplib..&package;
      /*********************************************************/
      /* Utility function to transform an array of values into */
      /* an array of the cumulative sums of those values       */
      /**********************************************************/
      subroutine cumsum( x[*], cumX[*] );
         outargs cumX;
         cumX[1] = x[1];

         do i = 2 to dim(X);
            cumX[i] = cumX[ i - 1 ] + X[i];
         end;
      endsub;

      /*********************************************************/
      /* Utility function to normalize bucket weights so that  */
      /* the sum of all is 1.0                                 */
      /**********************************************************/
      subroutine NormalizeWeights( Weights[*], nWeights );
         outargs Weights;

         if not missing(Weights[1]) then do;
            lastValidWeight = 1;

            do i = 2 to nWeights;
               if not missing(Weights[i]) then do;
                  Weights[i] = Weights[i] + Weights[i - 1];
                  lastValidWeight = i;
               end;
               else do;
                  i = nWeights;
               end;
            end;
         end;

         do i = 1 to (lastValidWeight - 1);
            Weights[i] = Weights[i] / Weights[nWeights];
         end;

         Weights[lastValidWeight] = 1;
      endsub;

      /*********************************************************************/
      /* Computes the 4 first central moments from the first 4 raw moments */
      /* For the purpose of this routine, the first 4 central moments are  */
      /* understood to be Mean, Std, Skewness, and Kurtosis                */
      /*********************************************************************/
      subroutine Centralize4Moments( RawMoments[*], CentralMoments[*] );
         outargs CentralMoments;
         CentralMoments[1] = RawMoments[1];
         CentralMoments[2] = sqrt(RawMoments[2] - RawMoments[1] * RawMoments[1]);
         CentralMoments[3] = (
                                 RawMoments[3] 
                                 - 3 * RawMoments[1] * RawMoments[2] + 2 * (m1**3) 
                              ) 
                              / 
                              (CentralMoments[2]**3)
                              ;
         CentralMoments[4] = (
                                 RawMoments[4] 
                                 - 4 * RawMoments[1] * RawMoments[3] 
                                 + 6 * (RawMoments[1]**2) * RawMoments[2] 
                                 - 3 * (RawMoments[1]**4) 
                              ) 
                              / 
                              (CentralMoments[2]**4);
      endsub;

      /*********************************/
      /* Simulates values from a table */
      /*********************************/
      function myRanTable( seed, weights[*], nWeights);
         u = ranuni(seed);
         dw = nWeights - 1;
            
         do i = 1 to dw;
            if u < weights[i] then do;
               return( i);
         end;
         end;

         return(dw + 1);
      endsub;

      /*************************************************************/
      /* Compute severity moments for a bucket scenario            */     
      /*************************************************************/
      subroutine BucketSevMoments( 
                                    Buckets[*], 
                                    Weights[*], 
                                    nBuckets  ,
                                    Moments[*] 
                                    );
         outargs Moments;
         tw = 0;

         do j = 1 to dim( Moments );
            Moments[j] = 0;
         end;

         do i = 1 to nBuckets;
            if not missing(Weights[i]) then do;
               tw = tw + Weights[i] ;

               do j = 1 to dim( Moments );
                  Moments[j] = Moments[j] + Weights[i] * (Buckets[i + 1]**(j + 1) - Buckets[i]**(j+1))/(Buckets[i + 1] - Buckets[i]) / (j + 1);
               end;
            end;
         end;

         do j = 1 to dim( Moments );
            Moments[j] = Moments[j] / tw;
         end;
      endsub;

      /* Function to simulate severity for bucket based scenarios */
      function BucketSevSim( seed, Buckets[*], Weights[*], nBuckets);
         _bin_  = myRanTable( seed, Weights, nBuckets );
         _unif_ = rand("UNIFORM");
         return(( 1 - _unif_ ) * Buckets[_bin_] + _unif_ * Buckets[_bin_ + 1]);
      endsub; /* BucketSevSim */

      /* Function to simulate frequency directly for bucket based scenarios */
      function BucketFreqSim( 
                              Frequency_Mean,
                              Frequency_Min,
                              Frequency_Max
                              );

         if not missing(Frequency_Min) then do;
            _diffMin_ = Frequency_Mean - Frequency_Min;
         end;
         else do;
            _diffMin_ = 0;
         end;

         if not missing(Frequency_Max) then do;
            _diffMax_ = Frequency_Max - Frequency_Mean;
         end;
         else do;
            _diffMax_ = 0;
         end;

         _diff_ = max(_diffMin_, _diffMax_);

         if _diff_ > 0 then do;
            _variance_ = (_diff_/3)**2;

            if _variance_ > Frequency_Mean then do;
               return(  
                     rand( 
                           "NEGBIN", 
                           Frequency_Mean/_variance_, 
                           Frequency_Mean*Frequency_Mean/(_variance_ - Frequency_Mean)
                           )
                     );
            end;
            else do;
               return(rand("POIS", Frequency_Mean));
            end;
         end;
         else do;
               return(rand("POIS", Frequency_Mean));
         end;
      endsub; /* BucketFreqSim */

      /* Subroutine to create the right frequency distribution for bucket based scenarios */
      subroutine BucketFreqSimDef(  
                                    Frequency_Mean,
                                    Frequency_Min,
                                    Frequency_Max,
                                    Frequency_dist $,
                                    FParam_1,
                                    FParam_2
                                    );
         outargs Frequency_Dist, FParam_1, FParam_2;

         if not missing(Frequency_Min) then do;
            _diffMin_ = Frequency_Mean - Frequency_Min;
         end;
         else do;
            _diffMin_ = 0;
         end;

         if not missing(Frequency_Max) then do;
            _diffMax_ = Frequency_Max - Frequency_Mean;
         end;
         else do;
            _diffMax_ = 0;
         end;

         _diff_     = max(_diffMin_, _diffMax_);
         _variance_ = (_diff_/3)**2;

         if _variance_ > Frequency_Mean then do;
            Frequency_Dist = "NEGBIN";
            FParam_2       = _variance_ / Frequency_Mean - 1 ;
            FParam_1       = Frequency_Mean / FParam_2;
         end;
         else do;
            Frequency_Dist = "POIS";
            FParam_1       = Frequency_Mean;
         end;
      endsub; /* BucketFreqSimDef */
   run;
%mend orva_rare_sim_init;
